@@ -23,7 +23,9 @@
 
 #ifdef kmalloc
 	#undef kmalloc
+	#define kmalloc mlx5_mtrack_kmalloc
 #endif
+
 #ifdef kmalloc_array
 	#undef kmalloc_array
 #endif
@@ -38,6 +40,7 @@
 #endif
 #ifdef vmalloc
 	#undef vmalloc
+	#define vmalloc mlx5_mtrack_vmalloc
 #endif
 #ifdef vzalloc
 	#undef vzalloc
@@ -62,6 +65,7 @@
 #endif
 #ifdef kmem_cache_alloc
 	#undef kmem_cache_alloc
+	#define kmem_cache_alloc mlx5_mtrack_kmem_cache_alloc
 #endif
 #ifdef kmem_cache_zalloc
 	#undef kmem_cache_zalloc
@@ -147,6 +151,12 @@
 #if defined(alloc_ordered_workqueue)
 	#undef alloc_ordered_workqueue
 #endif
+#if defined(pci_vpd_alloc)
+	#undef pci_vpd_alloc
+#endif
+#if defined(kmemdup_nul)
+	#undef kmemdup_nul
+#endif
 
 #include <linux/module.h>
 #include <linux/kernel.h>
@@ -324,7 +334,7 @@ static inline const char *memtype_free_str(enum memtrack_memtype_t memtype)
 	}
 }
 
-int find_ignore_func_info(const char *func_name)
+static inline int find_ignore_func_info(const char *func_name)
 {
 	struct memtrack_ignore_func_info_t *tmp_ignore_func_info_p;
 
@@ -335,7 +345,7 @@ int find_ignore_func_info(const char *func_name)
 	return 0;
 }
 
-int find_targeted_module_info(char *module_name)
+static inline int find_targeted_module_info(char *module_name)
 {
 	struct memtrack_targeted_modules_info_t *tmp_targeted_module_info_p;
 	list_for_each_entry(tmp_targeted_module_info_p,
@@ -753,6 +763,10 @@ int is_non_trackable_alloc_func(const char *func_name)
 	static const char * const str_str_arr[] = {
 		/* functions containing these strings consider non trackable */
 		"skb",
+		"fwctl_devnode",
+#ifdef HAVE_CLEANUP_H
+		"auxiliary_device_sysfs_irq_add",
+#endif
 	};
 	static const char * const str_str_excep_arr[] = {
 		/* functions which are exception to the str_str_arr table */
@@ -760,38 +774,7 @@ int is_non_trackable_alloc_func(const char *func_name)
 	};
 	static const char * const str_cmp_arr[] = {
 		/* functions that allocate SKBs */
-		"mlx4_en_alloc_frags",
-		"mlx4_en_alloc_frag",
-		"mlx4_en_init_allocator",
-		"mlx4_en_free_frag",
-		"mlx4_en_free_rx_desc",
-		"mlx4_en_destroy_allocator",
-		"mlx4_en_complete_rx_desc",
-		"mlx4_alloc_pages",
-		"mlx4_alloc_page",
-		"mlx4_crdump_collect_crspace",
-		"mlx4_crdump_collect_fw_health",
-		"mlx5e_page_alloc_mapped",
 		"mlx5e_put_page",
-		/* vnic skb functions */
-		"free_single_frag",
-		"vnic_alloc_rx_skb",
-		"vnic_rx_skb",
-		"vnic_alloc_frag",
-		"vnic_empty_rx_entry",
-		"vnic_init_allocator",
-		"vnic_destroy_allocator",
-		"sdp_post_recv",
-		"sdp_rx_ring_purge",
-		"sdp_post_srcavail",
-		"sk_stream_alloc_page",
-		"update_send_head",
-		"sdp_bcopy_get",
-		"sdp_destroy_resources",
-		"tcf_exts_init",
-		"tcf_hashinfo_init",
-		/* sw steering functions */
-		"dr_icm_chunk_create",
 		/* release order0 pages, for old kernels only */
 		"mlx5e_free_xdpsq_desc",
 		/* kTLS resync dump */
@@ -828,16 +811,10 @@ EXPORT_SYMBOL(is_non_trackable_alloc_func);
 int is_non_trackable_free_func(const char *func_name)
 {
 	static const char * const str_cmp_arr[] = {
-		/* sw steering functions */
-		"dr_icm_chunk_destroy",
-		/* external function in mdev module */
-		"create_store",
 		/* functions in mlxdevm.c uses memory allocated by nla_strdup */
-		"mlxdevm_nl_cmd_rate_new_doit",
-		"mlxdevm_nl_cmd_rate_del_doit",
-		"mlxdevm_rate_node_get_doit_locked",
-		"mlxdevm_cmd_rate_set_node",
-		"mlxdevm_cmd_rate_set_leaf",
+		"mlxdevm_nl_rate_new_doit",
+		"mlxdevm_nl_rate_del_doit",
+		"devm_rate_nodes_destroy",
 	};
 	size_t str_cmp_arr_size = sizeof(str_cmp_arr)/sizeof(char *);
 	int i;
@@ -998,11 +975,7 @@ int memtrack_get_page_ref_count(unsigned long addr)
 		if (cur_mem_info_p->addr == addr) {
 			/* Found given address in the database - check ref-count */
 			struct page *page = (struct page *)(cur_mem_info_p->addr);
-#ifdef HAVE_MM_PAGE__COUNT
-			ref_conut = atomic_read(&page->_count);
-#else
 			ref_conut = atomic_read(&page->_refcount);
-#endif
 			memtrack_spin_unlock(&obj_desc_p->hash_lock, flags);
 			return ref_conut;
 		}
@@ -1176,7 +1149,7 @@ static void destroy_procfs_tree(void)
 	remove_proc_entry(memtrack_proc_entry_name, NULL);
 }
 
-int inject_error_record(char *module_name, char *file_name, char *func_name,
+static inline int inject_error_record(char *module_name, char *file_name, char *func_name,
 			const char *caller_func_name, unsigned long line_num)
 {
 	struct memtrack_injected_info_t *tmp_injected_info_p,
@@ -1248,7 +1221,6 @@ int memtrack_randomize_mem(void)
 EXPORT_SYMBOL(memtrack_randomize_mem);
 
 /* module entry points */
-
 int init_module(void)
 {
 	enum memtrack_memtype_t i;
@@ -1403,3 +1375,4 @@ void cleanup_module(void)
 	kmem_cache_destroy(meminfo_cache);
 	printk(KERN_INFO "memtrack::cleanup_module done.\n");
 }
+
