@@ -35,8 +35,8 @@
 #include <linux/errno.h>
 
 #define DRV_NAME	"iw_nes"
-#define DRV_VERSION	"24.10-3.2.5"
-#define DRV_RELDATE	"29 Jun 2025"
+#define DRV_VERSION	"25.07-0.9.7"
+#define DRV_RELDATE	"04 Aug 2025"
 
 MODULE_AUTHOR("Alaa Hleihel");
 MODULE_DESCRIPTION("iw_nes dummy kernel module");
