@@ -73,7 +73,7 @@ static ssize_t mlx5e_store_tc_num(struct device *device,
 	if (err != 1)
 		return -EINVAL;
 
-	if (tc_num != MLX5E_MAX_NUM_TC && tc_num != MLX5E_MIN_NUM_TC)
+	if (tc_num != MLX5_MAX_NUM_TC && tc_num != MLX5_MIN_NUM_TC)
 		return -EINVAL;
 
 	rtnl_lock();
@@ -89,8 +89,8 @@ static  ssize_t mlx5e_show_maxrate(struct device *device,
 				   char *buf)
 {
 	struct mlx5e_priv *priv = netdev_priv(to_net_dev(device));
-	u8 max_bw_value[MLX5E_MAX_NUM_TC];
-	u8 max_bw_unit[MLX5E_MAX_NUM_TC];
+	u8 max_bw_value[MLX5_MAX_NUM_TC];
+	u8 max_bw_unit[MLX5_MAX_NUM_TC];
 	int len = 0;
 	int ret;
 	int i;
@@ -122,8 +122,8 @@ static ssize_t mlx5e_store_maxrate(struct device *device,
 	 __u64 upper_limit_mbps = roundup(255 * MLX5E_100MBPS_TO_KBPS,
 						MLX5E_GBPS_TO_KBPS);
 	struct mlx5e_priv *priv = netdev_priv(to_net_dev(device));
-	u8 max_bw_value[MLX5E_MAX_NUM_TC];
-	u8 max_bw_unit[MLX5E_MAX_NUM_TC];
+	u8 max_bw_value[MLX5_MAX_NUM_TC];
+	u8 max_bw_unit[MLX5_MAX_NUM_TC];
 	u64 tc_maxrate[IEEE_8021QAZ_MAX_TCS];
 	int i = 0;
 	char delimiter;
@@ -133,7 +133,7 @@ static ssize_t mlx5e_store_maxrate(struct device *device,
 		int len;
 		u64 input_maxrate;
 
-		if (i >= MLX5E_MAX_NUM_TC)
+		if (i >= MLX5_MAX_NUM_TC)
 			goto bad_elem_count;
 
 		len = strcspn(buf, " ");
@@ -154,7 +154,7 @@ static ssize_t mlx5e_store_maxrate(struct device *device,
 		i++;
 	} while (delimiter == ' ');
 
-	if (i != MLX5E_MAX_NUM_TC)
+	if (i != MLX5_MAX_NUM_TC)
 		goto bad_elem_count;
 
 	for (i = 0; i < IEEE_8021QAZ_MAX_TCS; i++) {
@@ -289,7 +289,7 @@ static ssize_t mlx5e_show_hfunc(struct device *device,
 
 	rtnl_lock();
 	mutex_lock(&priv->state_lock);
-	err = mlx5e_rx_res_rss_get_rxfh(priv->rx_res, 0, NULL, NULL, &hfunc);
+	err = mlx5e_rx_res_rss_get_rxfh(priv->rx_res, 0, NULL, NULL, &hfunc, NULL);
 	mutex_unlock(&priv->state_lock);
 	if (err)
 		goto out;
@@ -329,8 +329,14 @@ static ssize_t mlx5e_store_hfunc(struct device *device,
 
 	rtnl_lock();
 	mutex_lock(&priv->state_lock);
+	if (ethtool_hfunc == MLX5E_HFUNC_XOR) {
+		unsigned int ch_count = priv->channels.params.num_channels;
+
+		if (ch_count > mlx5e_rqt_max_num_channels_allowed_for_xor8())
+			goto unlock;
+	}
 	err = mlx5e_rx_res_rss_set_rxfh(priv->rx_res, 0, NULL, NULL,
-					&ethtool_hfunc);
+					&ethtool_hfunc, NULL);
 	mutex_unlock(&priv->state_lock);
 	rtnl_unlock();
 
@@ -339,6 +345,9 @@ static ssize_t mlx5e_store_hfunc(struct device *device,
 
 	return count;
 
+unlock:
+	mutex_unlock(&priv->state_lock);
+	rtnl_unlock();
 bad_input:
 	netdev_err(netdev, "Bad Input\n");
 	return -EINVAL;
@@ -347,6 +356,68 @@ bad_input:
 static DEVICE_ATTR(hfunc, S_IRUGO | S_IWUSR,
 		  mlx5e_show_hfunc, mlx5e_store_hfunc);
 
+static ssize_t mlx5e_show_xfrm(struct device *device,
+			       struct device_attribute *attr,
+			       char *buf)
+{
+	struct mlx5e_priv *priv = netdev_priv(to_net_dev(device));
+	int err, len = 0;
+	bool symmetric;
+
+	rtnl_lock();
+	mutex_lock(&priv->state_lock);
+	err = mlx5e_rx_res_rss_get_rxfh(priv->rx_res, 0, NULL, NULL, NULL, &symmetric);
+	mutex_unlock(&priv->state_lock);
+	rtnl_unlock();
+	if (err)
+		return err;
+
+	len += sprintf(buf + len, "Operational xfrm: %s\n",
+		       symmetric == true ?  "symmetric-or-xor" : "none");
+	len += sprintf(buf + len, "Supported xfrm: symmetric-or-xor none\n");
+
+	return len;
+}
+
+static ssize_t mlx5e_store_xfrm(struct device *device,
+				struct device_attribute *attr,
+				const char *buf, size_t count)
+{
+	struct mlx5e_priv *priv = netdev_priv(to_net_dev(device));
+	struct net_device *netdev = priv->netdev;
+	char xfrm[ETH_GSTRING_LEN];
+	bool symmetric;
+	int err;
+
+	err = sscanf(buf, "%31s", xfrm);
+
+	if (err != 1)
+		goto bad_input;
+
+	if (!strcmp(xfrm, "symmetric-or-xor"))
+		symmetric = true;
+	else if (!strcmp(xfrm, "none"))
+		symmetric = false;
+	else
+		goto bad_input;
+
+	rtnl_lock();
+	mutex_lock(&priv->state_lock);
+	err = mlx5e_rx_res_rss_set_rxfh(priv->rx_res, 0, NULL, NULL, NULL, &symmetric);
+	mutex_unlock(&priv->state_lock);
+	rtnl_unlock();
+	return err ? err : count;
+
+bad_input:
+	netdev_err(netdev, "Bad Input\n");
+	return -EINVAL;
+}
+
+static DEVICE_ATTR(xfrm, S_IRUGO | S_IWUSR,
+		  mlx5e_show_xfrm, mlx5e_store_xfrm);
+
+
+
 #define MLX5E_PFC_PREVEN_CRITICAL_AUTO_MSEC	100
 #define MLX5E_PFC_PREVEN_MINOR_AUTO_MSEC	85
 #define MLX5E_PFC_PREVEN_CRITICAL_DEFAULT_MSEC	8000
@@ -953,54 +1024,9 @@ static DEVICE_ATTR(force_local_lb_disable, S_IRUGO | S_IWUSR,
 		   mlx5e_show_force_local_lb,
 		   mlx5e_store_force_local_lb);
 
-static ssize_t mlx5e_show_log_rx_page_cache_mult_limit(struct device *device,
-						       struct device_attribute *attr,
-						       char *buf)
-{
-	struct net_device *dev = to_net_dev(device);
-	struct mlx5e_priv *priv = netdev_priv(dev);
-	int len;
-
-	mutex_lock(&priv->state_lock);
-	len = sprintf(buf, "log rx page cache mult limit is %u\n",
-		      priv->channels.params.log_rx_page_cache_mult);
-	mutex_unlock(&priv->state_lock);
-
-	return len;
-}
-
-static ssize_t mlx5e_store_log_rx_page_cache_mult_limit(struct device *device,
-							struct device_attribute *attr,
-							const char *buf,
-							size_t count)
-{
-	struct net_device *dev = to_net_dev(device);
-	struct mlx5e_priv *priv = netdev_priv(dev);
-	int err, udata;
-
-	err = kstrtoint(buf, 0, &udata);
-	if (err)
-		return -EINVAL;
-
-	if (udata > MLX5E_PAGE_CACHE_LOG_MAX_RQ_MULT || udata < 0) {
-		netdev_err(priv->netdev, "log rx page cache mult limit cannot exceed above %d or below 0\n",
-			   MLX5E_PAGE_CACHE_LOG_MAX_RQ_MULT);
-		return -EINVAL;
-	}
-
-	mutex_lock(&priv->state_lock);
-	priv->channels.params.log_rx_page_cache_mult = (u8)udata;
-	mutex_unlock(&priv->state_lock);
-
-	return count;
-}
-
-static DEVICE_ATTR(log_mult_limit, S_IRUGO | S_IWUSR,
-		   mlx5e_show_log_rx_page_cache_mult_limit,
-		   mlx5e_store_log_rx_page_cache_mult_limit);
-
 static struct attribute *mlx5e_settings_attrs[] = {
 	&dev_attr_hfunc.attr,
+	&dev_attr_xfrm.attr,
 	&dev_attr_pfc_stall_prevention.attr,
 	NULL,
 };
@@ -1066,16 +1092,6 @@ static struct attribute_group phy_stat_group = {
 	.attrs = mlx5e_phy_stat_attrs,
 };
 
-static struct attribute *mlx5e_log_rx_page_cache_attrs[] = {
-	&dev_attr_log_mult_limit.attr,
-	NULL,
-};
-
-static struct attribute_group rx_page_cache_group = {
-	.name = "rx_page_cache",
-	.attrs = mlx5e_log_rx_page_cache_attrs,
-};
-
 static int update_qos_sysfs(struct net_device *dev,
 			    struct mlx5_core_dev *mdev)
 {
@@ -1482,7 +1498,7 @@ static struct device_attribute *mlx5_class_attributes[] = {
 	&dev_attr_hp_oob_cnt,
 };
 
-int hp_sysfs_init(struct mlx5e_priv *priv)
+static int hp_sysfs_init(struct mlx5e_priv *priv)
 {
 	struct device *device = &priv->netdev->dev;
 	int i, err;
@@ -1496,7 +1512,7 @@ int hp_sysfs_init(struct mlx5e_priv *priv)
 	return 0;
 }
 
-void hp_sysfs_cleanup(struct mlx5e_priv *priv)
+static void hp_sysfs_cleanup(struct mlx5e_priv *priv)
 {
 	struct device *device = &priv->netdev->dev;
 	int i;
@@ -1522,21 +1538,12 @@ int mlx5e_sysfs_create(struct net_device *dev)
 	int err = 0;
 	int i;
 
-	if (mlx5_core_is_vf(priv->mdev) || mlx5_core_is_sf(priv->mdev))
+	if (mlx5_core_is_sf(priv->mdev))
 		return 0;
 
-	res->compat.ecn_root_kobj = kobject_create_and_add("ecn", &dev->dev.kobj);
-
-	for (i = 1; i < MLX5E_CONG_PROTOCOL_NUM; i++) {
-		res->compat.ecn_ctx[i].ecn_proto_kobj =
-			kobject_create_and_add(mlx5e_get_cong_protocol(i),
-					       res->compat.ecn_root_kobj);
-		mlx5e_fill_attributes(priv, i);
-	}
-
 	err = sysfs_create_group(&dev->dev.kobj, &settings_group);
 	if (err)
-		goto remove_attributes;
+		return err;
 
 	err = update_settings_sysfs(dev, priv->mdev);
 	if (err)
@@ -1558,21 +1565,26 @@ int mlx5e_sysfs_create(struct net_device *dev)
 	if (err)
 		goto remove_debug_group;
 
-	err = sysfs_create_group(&dev->dev.kobj, &rx_page_cache_group);
-
+	err = hp_sysfs_init(priv);
 	if (err)
 		goto remove_phy_stat_group;
 
-	err = hp_sysfs_init(priv);
-	if (err)
-		goto remove_rx_page_cache_group;
-
 	mlx5_eswitch_compat_sysfs_init(dev);
 
+	if (mlx5_core_is_vf(priv->mdev))
+		return 0;
+
+	res->compat.ecn_root_kobj = kobject_create_and_add("ecn", &dev->dev.kobj);
+
+	for (i = 1; i < MLX5E_CONG_PROTOCOL_NUM; i++) {
+		res->compat.ecn_ctx[i].ecn_proto_kobj =
+			kobject_create_and_add(mlx5e_get_cong_protocol(i),
+					       res->compat.ecn_root_kobj);
+		mlx5e_fill_attributes(priv, i);
+	}
+
 	return 0;
 
-remove_rx_page_cache_group:
-	sysfs_remove_group(&dev->dev.kobj, &rx_page_cache_group);
 remove_phy_stat_group:
 	sysfs_remove_group(&dev->dev.kobj, &phy_stat_group);
 remove_debug_group:
@@ -1581,14 +1593,6 @@ remove_qos_group:
 	sysfs_remove_group(&dev->dev.kobj, &qos_group);
 remove_settings_group:
 	sysfs_remove_group(&dev->dev.kobj, &settings_group);
-remove_attributes:
-	for (i = 1; i < MLX5E_CONG_PROTOCOL_NUM; i++) {
-		mlx5e_remove_attributes(priv, i);
-		kobject_put(res->compat.ecn_ctx[i].ecn_proto_kobj);
-	}
-
-	kobject_put(res->compat.ecn_root_kobj);
-	res->compat.ecn_root_kobj = NULL;
 
 	return err;
 }
@@ -1599,10 +1603,7 @@ void mlx5e_sysfs_remove(struct net_device *dev)
 	struct mlx5e_resources *res = &priv->mdev->mlx5e_res;
 	int i;
 
-	if (mlx5_core_is_vf(priv->mdev) || mlx5_core_is_sf(priv->mdev))
-		return;
-
-	if (!res->compat.ecn_root_kobj)
+	if (mlx5_core_is_sf(priv->mdev))
 		return;
 
 	mlx5_eswitch_compat_sysfs_cleanup(dev);
@@ -1611,9 +1612,14 @@ void mlx5e_sysfs_remove(struct net_device *dev)
 	sysfs_remove_group(&dev->dev.kobj, &debug_group);
 	sysfs_remove_group(&dev->dev.kobj, &settings_group);
 	sysfs_remove_group(&dev->dev.kobj, &phy_stat_group);
-	sysfs_remove_group(&dev->dev.kobj, &rx_page_cache_group);
 	hp_sysfs_cleanup(priv);
 
+	if (mlx5_core_is_vf(priv->mdev))
+		return;
+
+	if (!res->compat.ecn_root_kobj)
+		return;
+
 	for (i = 1; i < MLX5E_CONG_PROTOCOL_NUM; i++) {
 		mlx5e_remove_attributes(priv, i);
 		kobject_put(res->compat.ecn_ctx[i].ecn_proto_kobj);
