@@ -6,6 +6,7 @@
 #include "fw_reset.h"
 #include "diag/fw_tracer.h"
 #include "lib/tout.h"
+#include "mlx5_devm.h"
 
 enum {
 	MLX5_FW_RESET_FLAGS_RESET_REQUESTED,
@@ -55,7 +56,8 @@ static void mlx5_set_fw_rst_ack(struct mlx5_core_dev *dev)
 }
 
 static int mlx5_fw_reset_enable_remote_dev_reset_set(struct devlink *devlink, u32 id,
-						     struct devlink_param_gset_ctx *ctx)
+						     struct devlink_param_gset_ctx *ctx,
+						     struct netlink_ext_ack *extack)
 {
 	struct mlx5_core_dev *dev = devlink_priv(devlink);
 	struct mlx5_fw_reset *fw_reset;
@@ -251,7 +253,7 @@ static void mlx5_stop_sync_reset_poll(struct mlx5_core_dev *dev)
 {
 	struct mlx5_fw_reset *fw_reset = dev->priv.fw_reset;
 
-	del_timer_sync(&fw_reset->timer);
+	timer_delete_sync(&fw_reset->timer);
 }
 
 static int mlx5_sync_reset_clear_reset_requested(struct mlx5_core_dev *dev, bool poll_health)
@@ -263,6 +265,8 @@ static int mlx5_sync_reset_clear_reset_requested(struct mlx5_core_dev *dev, bool
 		return -EALREADY;
 	}
 
+	if (current_work() != &fw_reset->reset_timeout_work.work)
+		cancel_delayed_work(&fw_reset->reset_timeout_work);
 	mlx5_stop_sync_reset_poll(dev);
 	if (poll_health)
 		mlx5_start_health_poll(dev);
@@ -275,7 +279,6 @@ static void mlx5_sync_reset_reload_work(struct work_struct *work)
 						      reset_reload_work);
 	struct mlx5_core_dev *dev = fw_reset->dev;
 
-	cancel_delayed_work(&fw_reset->reset_timeout_work);
 	mlx5_sync_reset_clear_reset_requested(dev, false);
 	mlx5_enter_error_state(dev, true);
 	mlx5_fw_reset_complete_reload(dev, false);
@@ -334,6 +337,10 @@ static int mlx5_sync_reset_set_reset_requested(struct mlx5_core_dev *dev)
 	}
 	mlx5_stop_health_poll(dev, true);
 	mlx5_start_sync_reset_poll(dev);
+
+	if (!test_bit(MLX5_FW_RESET_FLAGS_DROP_NEW_REQUESTS, &fw_reset->reset_flags))
+		schedule_delayed_work(&fw_reset->reset_timeout_work,
+				      msecs_to_jiffies(mlx5_tout_ms(dev, PCI_SYNC_UPDATE)));
 	return 0;
 }
 
@@ -351,15 +358,12 @@ static void mlx5_fw_live_patch_event(struct work_struct *work)
 }
 
 #if IS_ENABLED(CONFIG_HOTPLUG_PCI_PCIE)
-static int mlx5_check_hotplug_interrupt(struct mlx5_core_dev *dev)
+static int mlx5_check_hotplug_interrupt(struct mlx5_core_dev *dev,
+					struct pci_dev *bridge)
 {
-	struct pci_dev *bridge = dev->pdev->bus->self;
 	u16 reg16;
 	int err;
 
-	if (!bridge)
-		return -EOPNOTSUPP;
-
 	err = pcie_capability_read_word(bridge, PCI_EXP_SLTCTL, &reg16);
 	if (err)
 		return err;
@@ -422,9 +426,15 @@ static int mlx5_check_dev_ids(struct mlx5_core_dev *dev, u16 dev_id)
 static bool mlx5_is_reset_now_capable(struct mlx5_core_dev *dev,
 				      u8 reset_method)
 {
+	struct pci_dev *bridge = dev->pdev->bus->self;
 	u16 dev_id;
 	int err;
 
+	if (!bridge) {
+		mlx5_core_warn(dev, "PCI bus bridge is not accessible\n");
+		return false;
+	}
+
 	if (!MLX5_CAP_GEN(dev, fast_teardown)) {
 		mlx5_core_warn(dev, "fast teardown is not supported by firmware\n");
 		return false;
@@ -432,7 +442,7 @@ static bool mlx5_is_reset_now_capable(struct mlx5_core_dev *dev,
 
 #if IS_ENABLED(CONFIG_HOTPLUG_PCI_PCIE)
 	if (reset_method != MLX5_MFRL_REG_PCI_RESET_METHOD_HOT_RESET) {
-		err = mlx5_check_hotplug_interrupt(dev);
+		err = mlx5_check_hotplug_interrupt(dev, bridge);
 		if (err)
 			return false;
 	}
@@ -449,7 +459,6 @@ static void mlx5_sync_reset_request_event(struct work_struct *work)
 	struct mlx5_fw_reset *fw_reset = container_of(work, struct mlx5_fw_reset,
 						      reset_request_work);
 	struct mlx5_core_dev *dev = fw_reset->dev;
-	unsigned long sync_reset_timeout;
 	bool nack_request = false;
 	u8 sync_flow;
 	int err;
@@ -474,8 +483,6 @@ static void mlx5_sync_reset_request_event(struct work_struct *work)
 	if (mlx5_sync_reset_set_reset_requested(dev))
 		return;
 
-	sync_reset_timeout = msecs_to_jiffies(mlx5_tout_ms(dev, PCI_SYNC_UPDATE));
-	schedule_delayed_work(&fw_reset->reset_timeout_work, sync_reset_timeout);
 	err = mlx5_fw_reset_set_reset_sync_ack(dev);
 	if (err)
 		mlx5_core_warn(dev, "PCI Sync FW Update Reset Ack Failed. Error code: %d\n", err);
@@ -490,19 +497,15 @@ static int mlx5_pci_link_toggle(struct mlx5_core_dev *dev, u16 dev_id)
 	unsigned long timeout;
 	struct pci_dev *sdev;
 	int cap, err;
-	u32 reg32;
 	u16 reg16;
 
-	if (!bridge)
-		return -EOPNOTSUPP;
-
 	cap = pci_find_capability(bridge, PCI_CAP_ID_EXP);
 	if (!cap)
 		return -EOPNOTSUPP;
 
 	list_for_each_entry(sdev, &bridge_bus->devices, bus_list) {
-		pci_save_state(sdev);
 		pci_cfg_access_lock(sdev);
+		pci_save_state(sdev);
 	}
 	/* PCI link toggle */
 	err = pcie_capability_set_word(bridge, PCI_EXP_LNKCTL, PCI_EXP_LNKCTL_LD);
@@ -558,39 +561,11 @@ static int mlx5_pci_link_toggle(struct mlx5_core_dev *dev, u16 dev_id)
 	}
 
 restore:
-	timeout = jiffies + msecs_to_jiffies(mlx5_tout_ms(dev, PCI_TOGGLE));
-
 	list_for_each_entry(sdev, &bridge_bus->devices, bus_list) {
-		u64 reg64, reg64_comp;
-		int retry_ctr = 0;
-
+		pci_restore_state(sdev);
 		pci_cfg_access_unlock(sdev);
-		reg64_comp = ((u64)sdev->saved_config_space[5] << 32) |
-			      ((u64)sdev->saved_config_space[4] & 0xfff00000);
-		do {
-			msleep(10);
-			sdev->state_saved = true;
-			pci_restore_state(sdev);
-			err = pci_read_config_dword(sdev, 0x14, &reg32);
-			if (err)
-				return pcibios_err_to_errno(err);
-			reg64 = (u64)reg32 << 32;
-			err = pci_read_config_dword(sdev, 0x10, &reg32);
-			if (err)
-				return pcibios_err_to_errno(err);
-			reg64 |= (u64)reg32 & 0xfff00000;
-			retry_ctr++;
-		} while ((reg64_comp != reg64) && !time_after(jiffies, timeout));
-		if (1 != retry_ctr)
-			mlx5_core_warn(dev, "devfn=0x%.8x restore had to retry %d times\n",
-				       sdev->devfn, retry_ctr);
-		if (reg64_comp != reg64) {
-			mlx5_core_err(dev, "devfn=0x%.8x BAR not set after %llu ms\n",
-				      sdev->devfn, mlx5_tout_ms(dev, PCI_TOGGLE));
-			err = -ETIMEDOUT;
-			break;
-		}
 	}
+
 	return err;
 }
 
@@ -607,9 +582,6 @@ static int mlx5_sync_pci_reset(struct mlx5_core_dev *dev, u8 reset_method)
 	u16 dev_id;
 	int err;
 
-	if (mlx5_core_is_ecpf(dev))
-		return -ENOTSUPP;
-
 	err = pci_read_config_word(dev->pdev, PCI_DEVICE_ID, &dev_id);
 	if (err)
 		return pcibios_err_to_errno(err);
@@ -642,7 +614,6 @@ static void mlx5_sync_reset_now_event(struct work_struct *work)
 	struct mlx5_core_dev *dev = fw_reset->dev;
 	int err;
 
-	cancel_delayed_work(&fw_reset->reset_timeout_work);
 	if (mlx5_sync_reset_clear_reset_requested(dev, false))
 		return;
 
@@ -741,7 +712,6 @@ static void mlx5_sync_reset_abort_event(struct work_struct *work)
 						      reset_abort_work);
 	struct mlx5_core_dev *dev = fw_reset->dev;
 
-	cancel_delayed_work(&fw_reset->reset_timeout_work);
 	if (mlx5_sync_reset_clear_reset_requested(dev, true))
 		return;
 	mlx5_core_warn(dev, "PCI Sync FW Update Reset Aborted.\n");
@@ -875,6 +845,41 @@ static const struct devlink_param mlx5_fw_reset_devlink_params[] = {
 			      mlx5_fw_reset_enable_remote_dev_reset_set, NULL),
 };
 
+static int mlx5_devm_fw_reset_enable_remote_dev_reset_set(struct mlxdevm *mlxdevm, u32 id,
+							  struct mlxdevm_param_gset_ctx *ctx,
+							  struct netlink_ext_ack *extack)
+{
+	struct mlx5_core_dev *dev = mlx5_devm_core_dev_get(mlxdevm);
+	struct mlx5_fw_reset *fw_reset;
+
+	fw_reset = dev->priv.fw_reset;
+
+	if (ctx->val.vbool)
+		clear_bit(MLX5_FW_RESET_FLAGS_NACK_RESET_REQUEST, &fw_reset->reset_flags);
+	else
+		set_bit(MLX5_FW_RESET_FLAGS_NACK_RESET_REQUEST, &fw_reset->reset_flags);
+	return 0;
+}
+
+static int mlx5_devm_fw_reset_enable_remote_dev_reset_get(struct mlxdevm *mlxdevm, u32 id,
+							  struct mlxdevm_param_gset_ctx *ctx)
+{
+	struct mlx5_core_dev *dev = mlx5_devm_core_dev_get(mlxdevm);
+	struct mlx5_fw_reset *fw_reset;
+
+	fw_reset = dev->priv.fw_reset;
+
+	ctx->val.vbool = !test_bit(MLX5_FW_RESET_FLAGS_NACK_RESET_REQUEST,
+				   &fw_reset->reset_flags);
+	return 0;
+}
+
+static const struct mlxdevm_param mlx5_fw_reset_mlxdevm_params[] = {
+	MLXDEVM_PARAM_GENERIC(ENABLE_REMOTE_DEV_RESET, BIT(MLXDEVM_PARAM_CMODE_RUNTIME),
+			      mlx5_devm_fw_reset_enable_remote_dev_reset_get,
+			      mlx5_devm_fw_reset_enable_remote_dev_reset_set, NULL),
+};
+
 int mlx5_fw_reset_init(struct mlx5_core_dev *dev)
 {
 	struct mlx5_fw_reset *fw_reset;
@@ -898,11 +903,14 @@ int mlx5_fw_reset_init(struct mlx5_core_dev *dev)
 	err = devl_params_register(priv_to_devlink(dev),
 				   mlx5_fw_reset_devlink_params,
 				   ARRAY_SIZE(mlx5_fw_reset_devlink_params));
-	if (err) {
-		destroy_workqueue(fw_reset->wq);
-		kfree(fw_reset);
-		return err;
-	}
+	if (err)
+		goto err_devlink_params;
+
+	err = devm_params_register(&mlx5_devm_device_get(dev)->device,
+				   mlx5_fw_reset_mlxdevm_params,
+				   ARRAY_SIZE(mlx5_fw_reset_mlxdevm_params));
+	if (err)
+		goto err_mlxdevm_params;
 
 	INIT_WORK(&fw_reset->fw_live_patch_work, mlx5_fw_live_patch_event);
 	INIT_WORK(&fw_reset->reset_request_work, mlx5_sync_reset_request_event);
@@ -914,6 +922,15 @@ int mlx5_fw_reset_init(struct mlx5_core_dev *dev)
 
 	init_completion(&fw_reset->done);
 	return 0;
+
+err_mlxdevm_params:
+	devl_params_unregister(priv_to_devlink(dev),
+			       mlx5_fw_reset_devlink_params,
+			       ARRAY_SIZE(mlx5_fw_reset_devlink_params));
+err_devlink_params:
+	destroy_workqueue(fw_reset->wq);
+	kfree(fw_reset);
+	return err;
 }
 
 void mlx5_fw_reset_cleanup(struct mlx5_core_dev *dev)
@@ -923,6 +940,9 @@ void mlx5_fw_reset_cleanup(struct mlx5_core_dev *dev)
 	if (!fw_reset)
 		return;
 
+	devm_params_unregister(&mlx5_devm_device_get(dev)->device,
+			       mlx5_fw_reset_mlxdevm_params,
+			       ARRAY_SIZE(mlx5_fw_reset_mlxdevm_params));
 	devl_params_unregister(priv_to_devlink(dev),
 			       mlx5_fw_reset_devlink_params,
 			       ARRAY_SIZE(mlx5_fw_reset_devlink_params));
