@@ -12,6 +12,15 @@
 
 struct mlx5_sf_dev {
 	struct auxiliary_device adev;
+	/* Add sysfs struct in base code until
+	 * 4097699 fix is accepted upstream.
+	 * Needed to build base code w/o backports.
+	 */
+	struct {
+		struct xarray irqs;
+		struct mutex lock; /* Synchronize irq sysfs creation */
+		bool irq_dir_exists;
+	} sysfs;
 	struct mlx5_core_dev *parent_mdev;
 	struct mlx5_core_dev *mdev;
 	phys_addr_t bar_base_addr;
@@ -27,6 +36,12 @@ struct mlx5_sf_dev {
 #endif
 };
 
+struct mlx5_sf_peer_devlink_event_ctx {
+	u16 fn_id;
+	struct devlink *devlink;
+	int err;
+};
+
 void mlx5_sf_dev_table_create(struct mlx5_core_dev *dev);
 void mlx5_sf_dev_table_destroy(struct mlx5_core_dev *dev);
 
