@@ -40,9 +40,9 @@
 #include "eswitch.h"
 #ifdef CONFIG_MLX5_ESWITCH
 #include "esw/vf_meter.h"
-#include "esw/qos.h"
 #endif
 #include "esw/legacy.h"
+#include "esw/qos.h"
 
 struct vf_attributes {
 	struct attribute attr;
@@ -108,9 +108,9 @@ static ssize_t vf_paging_attr_store(struct kobject *kobj,
 
 struct vf_group_attributes {
 	struct attribute attr;
-	ssize_t (*show)(struct mlx5_esw_rate_group *, struct vf_group_attributes *,
+	ssize_t (*show)(struct mlx5_esw_sched_node *, struct vf_group_attributes *,
 			char *buf);
-	ssize_t (*store)(struct mlx5_esw_rate_group *, struct vf_group_attributes *,
+	ssize_t (*store)(struct mlx5_esw_sched_node *, struct vf_group_attributes *,
 			 const char *buf, size_t count);
 };
 
@@ -119,7 +119,7 @@ static ssize_t vf_group_attr_show(struct kobject *kobj,
 {
 	struct vf_group_attributes *ga =
 		container_of(attr, struct vf_group_attributes, attr);
-	struct mlx5_esw_rate_group *g = container_of(kobj, struct mlx5_esw_rate_group, kobj);
+	struct mlx5_esw_sched_node *g = container_of(kobj, struct mlx5_esw_sched_node, kobj);
 
 	if (!ga->show)
 		return -EIO;
@@ -133,7 +133,7 @@ static ssize_t vf_group_attr_store(struct kobject *kobj,
 {
 	struct vf_group_attributes *ga =
 		container_of(attr, struct vf_group_attributes, attr);
-	struct mlx5_esw_rate_group *g = container_of(kobj, struct mlx5_esw_rate_group, kobj);
+	struct mlx5_esw_sched_node *g = container_of(kobj, struct mlx5_esw_sched_node, kobj);
 
 	if (!ga->store)
 		return -EIO;
@@ -141,7 +141,7 @@ static ssize_t vf_group_attr_store(struct kobject *kobj,
 	return ga->store(g, ga, buf, size);
 }
 
-static ssize_t max_tx_rate_group_show(struct mlx5_esw_rate_group *g,
+static ssize_t max_tx_rate_group_show(struct mlx5_esw_sched_node *g,
 				      struct vf_group_attributes *oa,
 				      char *buf)
 {
@@ -149,11 +149,11 @@ static ssize_t max_tx_rate_group_show(struct mlx5_esw_rate_group *g,
 		       "usage: write <Rate (Mbit/s)> to set VF group max rate\n");
 }
 
-static ssize_t max_tx_rate_group_store(struct mlx5_esw_rate_group *g,
+static ssize_t max_tx_rate_group_store(struct mlx5_esw_sched_node *g,
 				       struct vf_group_attributes *oa,
 				       const char *buf, size_t count)
 {
-	struct mlx5_core_dev *dev = g->dev;
+	struct mlx5_core_dev *dev = g->esw->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
 	u32 max_rate;
 	int err;
@@ -162,12 +162,12 @@ static ssize_t max_tx_rate_group_store(struct mlx5_esw_rate_group *g,
 	if (err != 1)
 		return -EINVAL;
 
-	err = mlx5_esw_qos_set_sysfs_group_max_rate(esw, g, max_rate);
+	err = mlx5_esw_qos_set_sysfs_node_max_rate(esw, g, max_rate);
 
 	return err ? err : count;
 }
 
-static ssize_t min_tx_rate_group_show(struct mlx5_esw_rate_group *g,
+static ssize_t min_tx_rate_group_show(struct mlx5_esw_sched_node *g,
 				      struct vf_group_attributes *oa,
 				      char *buf)
 {
@@ -175,11 +175,11 @@ static ssize_t min_tx_rate_group_show(struct mlx5_esw_rate_group *g,
 		       "usage: write <Rate (Mbit/s)> to set VF group min rate\n");
 }
 
-static ssize_t min_tx_rate_group_store(struct mlx5_esw_rate_group *g,
+static ssize_t min_tx_rate_group_store(struct mlx5_esw_sched_node *g,
 				       struct vf_group_attributes *oa,
 				       const char *buf, size_t count)
 {
-	struct mlx5_core_dev *dev = g->dev;
+	struct mlx5_core_dev *dev = g->esw->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
 	u32 min_rate;
 	int err;
@@ -188,7 +188,7 @@ static ssize_t min_tx_rate_group_store(struct mlx5_esw_rate_group *g,
 	if (err != 1)
 		return -EINVAL;
 
-	err = mlx5_esw_qos_set_sysfs_group_min_rate(esw, g, min_rate);
+	err = mlx5_esw_qos_set_sysfs_node_min_rate(esw, g, min_rate);
 
 	return err ? err : count;
 }
@@ -627,27 +627,42 @@ static ssize_t max_tx_rate_show(struct mlx5_sriov_vf *g,
 		       "usage: write <Rate (Mbit/s)> to set VF max rate\n");
 }
 
+static bool mlx5_esw_qos_is_needed(struct mlx5_esw_sched_node *parent, u64 tx_max,
+				  u64 tx_share)
+{
+	return parent || tx_max || tx_share;
+}
+
 static ssize_t max_tx_rate_store(struct mlx5_sriov_vf *g,
 				 struct vf_attributes *oa,
 				 const char *buf, size_t count)
 {
 	struct mlx5_core_dev *dev = g->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
-	struct mlx5_vport *evport = mlx5_eswitch_get_vport(esw, g->vf + 1);
+	struct mlx5_vport *vport = mlx5_eswitch_get_vport(esw, g->vf + 1);
 	u32 max_tx_rate;
-	u32 min_tx_rate;
 	int err;
 
-	mutex_lock(&esw->state_lock);
-	min_tx_rate = evport->qos.min_rate;
-	mutex_unlock(&esw->state_lock);
+	err = kstrtou32(buf, 10, &max_tx_rate);
+	if (err)
+		return err;
 
-	err = sscanf(buf, "%u", &max_tx_rate);
-	if (err != 1)
-		return -EINVAL;
+	if (!vport->qos.sched_node) {
+		/* QoS is not enabled, return if setting 0 to avoid enabling
+		 * vport QoS unnecessarily.
+		 */
+		if (!max_tx_rate)
+			return count;
+	} else if (!mlx5_esw_qos_is_needed(vport->qos.sched_node->parent,
+					   max_tx_rate,
+					   vport->qos.sched_node->min_rate)) {
+		mlx5_esw_qos_vport_disable(vport);
+		return count;
+	}
 
-	err = mlx5_eswitch_set_vport_rate(dev->priv.eswitch, g->vf + 1,
-					  max_tx_rate, min_tx_rate);
+	esw_qos_lock(esw);
+	err = mlx5_esw_qos_set_vport_max_rate(vport, max_tx_rate, NULL);
+	esw_qos_unlock(esw);
 	return err ? err : count;
 }
 
@@ -666,17 +681,22 @@ static ssize_t group_store(struct mlx5_sriov_vf *g,
 {
 	struct mlx5_core_dev *dev = g->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
-	u32 group_id;
+	struct mlx5_vport *vport;
+	u32 node_id;
 	int err;
 
-	err = sscanf(buf, "%u", &group_id);
+	err = sscanf(buf, "%u", &node_id);
 	if (err != 1)
 		return -EINVAL;
 
-	if (group_id > MLX5_ESW_QOS_SYSFS_GROUP_MAX_ID)
+	if (node_id > MLX5_ESW_QOS_SYSFS_GROUP_MAX_ID)
 		return -EINVAL;
 
-	err = mlx5_esw_qos_vport_update_sysfs_group(esw, g->vf + 1, group_id);
+	vport = mlx5_eswitch_get_vport(esw, g->vf + 1);
+	if (IS_ERR(vport))
+		return PTR_ERR(vport);
+
+	err = mlx5_esw_qos_vport_update_sysfs_node(esw, node_id, vport);
 
 	return err ? err : count;
 }
@@ -695,23 +715,33 @@ static ssize_t min_tx_rate_store(struct mlx5_sriov_vf *g,
 {
 	struct mlx5_core_dev *dev = g->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
-	struct mlx5_vport *evport = mlx5_eswitch_get_vport(esw, g->vf + 1);
+	struct mlx5_vport *vport = mlx5_eswitch_get_vport(esw, g->vf + 1);
 	u32 min_tx_rate;
-	u32 max_tx_rate;
 	int err;
 
-	mutex_lock(&esw->state_lock);
-	max_tx_rate = evport->qos.max_rate;
-	mutex_unlock(&esw->state_lock);
+	err = kstrtou32(buf, 10, &min_tx_rate);
+	if (err)
+		return err;
 
-	err = sscanf(buf, "%u", &min_tx_rate);
-	if (err != 1)
-		return -EINVAL;
+	if (!vport->qos.sched_node) {
+		/* QoS is not enabled, return if setting 0 to avoid enabling
+		 * vport QoS unnecessarily.
+		 */
+		if (!min_tx_rate)
+			return count;
+	} else if (!mlx5_esw_qos_is_needed(vport->qos.sched_node->parent,
+					   vport->qos.sched_node->max_rate,
+					   min_tx_rate)) {
+		mlx5_esw_qos_vport_disable(vport);
+		return count;
+	}
 
-	err = mlx5_eswitch_set_vport_rate(dev->priv.eswitch, g->vf + 1,
-					  max_tx_rate, min_tx_rate);
+	esw_qos_lock(esw);
+	err = mlx5_esw_qos_set_vport_min_rate(vport, min_tx_rate, NULL);
+	esw_qos_unlock(esw);
 	return err ? err : count;
 }
+
 #define _sprintf(p, buf, format, arg...)				\
 	((PAGE_SIZE - (int)(p - buf)) <= 0 ? 0 :			\
 	scnprintf(p, PAGE_SIZE - (int)(p - buf), format, ## arg))
@@ -793,22 +823,21 @@ static ssize_t config_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 	p += _sprintf(p, buf, "SpoofCheck : %s\n", ivi->spoofchk ? "ON" : "OFF");
 	p += _sprintf(p, buf, "Trust      : %s\n", ivi->trusted ? "ON" : "OFF");
 	p += _sprintf(p, buf, "LinkState  : %s",   policy_str(ivi->link_state));
-	if (evport->qos.enabled) {
-		p += _sprintf(p, buf, "MinTxRate  : %d\n", evport->qos.min_rate);
-		p += _sprintf(p, buf, "MaxTxRate  : %d\n", evport->qos.max_rate);
-		if (evport->qos.group)
-			p += _sprintf(p, buf, "RateGroup  : %d\n",
-				      evport->qos.group->group_id);
-		else
-			p += _sprintf(p, buf, "RateGroup  : 0\n");
-	} else {
-		p += _sprintf(p, buf, "MinTxRate  : 0\nMaxTxRate  : 0\nRateGroup  : 0\n");
-	}
-	p += _sprintf(p, buf, "VGT+       : %s\n",
-		      !!bitmap_weight(ivi->vlan_trunk_8021q_bitmap,
-				      VLAN_N_VID) ? "ON" : "OFF");
+        if (evport->qos.sched_node) {
+                struct mlx5_esw_sched_node *parent = mlx5_esw_qos_vport_get_parent(evport);
 
-	p += _sprintf(p, buf, "RateGroup  : %d\n", ivi->group);
+                p += _sprintf(p, buf, "MinTxRate  : %d\n", evport->qos.sched_node->min_rate);
+                p += _sprintf(p, buf, "MaxTxRate  : %d\n", evport->qos.sched_node->max_rate);
+                if (parent)
+                        p += _sprintf(p, buf, "RateGroup  : %d\n", parent->node_id);
+                else
+                        p += _sprintf(p, buf, "RateGroup  : 0\n");
+        } else {
+                p += _sprintf(p, buf, "MinTxRate  : 0\nMaxTxRate  : 0\nRateGroup  : 0\n");
+        }
+        p += _sprintf(p, buf, "VGT+       : %s\n",
+                      !!bitmap_weight(ivi->vlan_trunk_8021q_bitmap,
+		      VLAN_N_VID) ? "ON" : "OFF");
 	mutex_unlock(&esw->state_lock);
 
 	return (ssize_t)(p - buf);
@@ -821,11 +850,11 @@ static ssize_t config_store(struct mlx5_sriov_vf *g,
 	return -ENOTSUPP;
 }
 
-static ssize_t config_group_show(struct mlx5_esw_rate_group *g,
+static ssize_t config_group_show(struct mlx5_esw_sched_node *g,
 				 struct vf_group_attributes *oa,
 				 char *buf)
 {
-	struct mlx5_core_dev *dev = g->dev;
+	struct mlx5_core_dev *dev = g->esw->dev;
 	struct mlx5_eswitch *esw = dev->priv.eswitch;
 	char *p = buf;
 
@@ -833,8 +862,7 @@ static ssize_t config_group_show(struct mlx5_esw_rate_group *g,
 	    mlx5_core_is_pf(esw->dev))
 		return -EPERM;
 
-	if (!mutex_trylock(&esw->state_lock))
-		return -EBUSY;
+	mutex_lock(&esw->state_lock);
 	p += _sprintf(p, buf, "Num VFs    : %d\n", g->num_vports);
 	p += _sprintf(p, buf, "MaxRate    : %d\n", g->max_rate);
 	p += _sprintf(p, buf, "MinRate    : %d\n", g->min_rate);
@@ -844,7 +872,7 @@ static ssize_t config_group_show(struct mlx5_esw_rate_group *g,
 	return (ssize_t)(p - buf);
 }
 
-static ssize_t config_group_store(struct mlx5_esw_rate_group *g,
+static ssize_t config_group_store(struct mlx5_esw_sched_node *g,
 				  struct vf_group_attributes *oa,
 				  const char *buf, size_t count)
 {
@@ -994,7 +1022,7 @@ static ssize_t page_limit_show(struct mlx5_sriov_vf *g, struct vf_attributes *oa
 	int err;
 
 	if (mlx5_vhca_icm_ctrl_supported(esw->dev)) {
-		err = mlx5_esw_query_vport_vhca_id(esw, g->vf + 1, &vhca_id);
+		err = mlx5_vport_get_vhca_id(esw->dev, g->vf + 1, &vhca_id);
 		if (err)
 			return err;
 
@@ -1012,6 +1040,7 @@ out:
 	return sprintf(buf, "%u\n", page_limit);
 }
 
+#define PAGE_LIMIT_UNLIMITED UINT_MAX
 static ssize_t page_limit_store(struct mlx5_sriov_vf *g, struct vf_attributes *oa,
 				const char *buf, size_t count)
 {
@@ -1026,12 +1055,12 @@ static ssize_t page_limit_store(struct mlx5_sriov_vf *g, struct vf_attributes *o
 		return -EINVAL;
 
 	if (mlx5_vhca_icm_ctrl_supported(esw->dev)) {
-		err = mlx5_esw_query_vport_vhca_id(esw, g->vf + 1, &vhca_id);
+		err = mlx5_vport_get_vhca_id(esw->dev, g->vf + 1, &vhca_id);
 		if (err)
 			return err;
 
 		if (!limit)
-			limit = UINT_MAX;
+			limit = PAGE_LIMIT_UNLIMITED;
 
 		err = mlx5_set_max_alloc_icm_th(g->dev, vhca_id, limit);
 		if (err)
@@ -1069,17 +1098,43 @@ static ssize_t num_pages_store(struct mlx5_sriov_vf *g, struct vf_attributes *oa
 
 #endif /* CONFIG_MLX5_ESWITCH */
 
+static int get_num_vfs(struct device *device)
+{
+	struct pci_dev *pdev = container_of(device, struct pci_dev, dev);
+	struct mlx5_core_dev *dev  = pci_get_drvdata(pdev);
+	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
+
+	return (sriov->num_vfs);
+}
+
 static ssize_t num_vf_store(struct device *device, struct device_attribute *attr,
 			    const char *buf, size_t count)
 {
 	struct pci_dev *pdev = container_of(device, struct pci_dev, dev);
+	struct mlx5_core_dev *dev  = pci_get_drvdata(pdev);
+	int curr_vfs = get_num_vfs(device);
 	int req_vfs;
 	int err;
 
-	if (kstrtoint(buf, 0, &req_vfs) || req_vfs < 0 ||
-	    req_vfs > pci_sriov_get_totalvfs(pdev))
+	if (kstrtoint(buf, 0, &req_vfs))
 		return -EINVAL;
 
+	if (req_vfs > pci_sriov_get_totalvfs(pdev))
+		return -ERANGE;
+
+	if (curr_vfs == req_vfs)
+		return count;
+
+	if (!req_vfs)
+		goto configure;
+
+	if (curr_vfs) {
+		mlx5_core_warn(dev, "%d VFs already enabled. Disable before enabling %d VFs\n",
+			       curr_vfs, req_vfs);
+		return -EBUSY;
+	}
+
+configure:
 	err = mlx5_core_sriov_configure(pdev, req_vfs);
 	if (err < 0)
 		return err;
@@ -1090,11 +1145,7 @@ static ssize_t num_vf_store(struct device *device, struct device_attribute *attr
 static ssize_t num_vf_show(struct device *device, struct device_attribute *attr,
 			   char *buf)
 {
-	struct pci_dev *pdev = container_of(device, struct pci_dev, dev);
-	struct mlx5_core_dev *dev  = pci_get_drvdata(pdev);
-	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
-
-	return sprintf(buf, "%d\n", sriov->num_vfs);
+	return sprintf(buf, "%d\n", get_num_vfs(device));
 }
 
 static DEVICE_ATTR(mlx5_num_vfs, 0600, num_vf_show, num_vf_store);
@@ -1287,15 +1338,15 @@ void mlx5_sriov_sysfs_cleanup(struct mlx5_core_dev *dev)
 	sriov->config = NULL;
 }
 
-int mlx5_create_vf_group_sysfs(struct mlx5_core_dev *dev,
-			       u32 group_id, struct kobject *group_kobj)
+int mlx5_create_vf_node_sysfs(struct mlx5_core_dev *dev, u32 node_id,
+			      struct kobject *group_kobj)
 {
 	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
 	int err;
 
 #ifdef CONFIG_MLX5_ESWITCH
 	err = kobject_init_and_add(group_kobj, &vf_group, sriov->groups_config,
-				   "%d", group_id);
+				   "%d", node_id);
 	if (err)
 		return err;
 
@@ -1308,27 +1359,32 @@ int mlx5_create_vf_group_sysfs(struct mlx5_core_dev *dev,
 #ifdef CONFIG_MLX5_ESWITCH
 struct mlx5_group_kobj {
 	struct work_struct work;
-	struct mlx5_esw_rate_group *group;
+	struct mlx5_esw_sched_node *node;
 };
 
 static void destroy_vf_group_work(struct work_struct *work)
 {
 	struct mlx5_group_kobj *obj = container_of(work, struct mlx5_group_kobj, work);
-	struct mlx5_esw_rate_group *group = obj->group;
+	struct mlx5_esw_sched_node *node = obj->node;
 
-	kobject_put(&group->kobj);
-	complete_all(&group->free_group_comp);
+	kobject_put(&node->kobj);
+	complete_all(&node->free_node_comp);
 	kfree(obj);
 }
 #endif
 
-void mlx5_destroy_vf_group_sysfs(struct mlx5_esw_rate_group *group)
+void mlx5_destroy_vf_node_sysfs(struct mlx5_esw_sched_node *node)
 {
 #ifdef CONFIG_MLX5_ESWITCH
 	struct mlx5_group_kobj *kobj = kzalloc(sizeof *kobj, GFP_ATOMIC);
 
+	if (!kobj) {
+		mlx5_core_err(node->esw->dev, "failed to allocate mem\n");
+		return;
+	}
+
 	INIT_WORK(&kobj->work, destroy_vf_group_work);
-	kobj->group = group;
+	kobj->node = node;
 	queue_work(system_wq, &kobj->work);
 #endif
 }
@@ -1386,7 +1442,7 @@ static int mlx5_init_vfs_sysfs_init_meter(struct mlx5_sriov_vf *vf,
 	return 0;
 }
 
-int mlx5_create_vfs_sysfs_meters(struct mlx5_core_dev *dev, int num_vfs)
+static int mlx5_create_vfs_sysfs_meters(struct mlx5_core_dev *dev, int num_vfs)
 {
 	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
 	struct mlx5_sriov_vf_meters *meters;
@@ -1572,7 +1628,6 @@ void mlx5_destroy_vfs_sysfs(struct mlx5_core_dev *dev, int num_vfs)
 #ifdef CONFIG_MLX5_ESWITCH
 	mlx5_destroy_vfs_sysfs_meters(dev, num_vfs);
 #endif
-
 	for (vf = 0; vf < num_vfs; vf++) {
 		tmp = &sriov->vfs[vf];
 		kobject_put(&tmp->kobj);
