@@ -191,10 +191,7 @@ static struct request *nvme_alloc_user_request(struct request_queue *q,
 
 static int nvme_map_user_request(struct request *req, u64 ubuffer,
 		unsigned bufflen, void __user *meta_buffer, unsigned meta_len,
-		u32 meta_seed,
-#ifdef HAVE_IO_URING_CMD_H
-		struct io_uring_cmd *ioucmd,
-#endif
+		struct iov_iter *iter,
 #ifndef HAVE_BIO_INTEGRITY_MAP_USER
 		void **metap,
 #endif
@@ -202,44 +199,39 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 {
 	struct request_queue *q = req->q;
 	struct nvme_ns *ns = q->queuedata;
-#ifdef HAVE_ENUM_BIO_REMAPPED
+#ifdef HAVE_ENUM_BIO_REMAPPED	
 	struct block_device *bdev = ns ? ns->disk->part0 : NULL;
+	bool supports_metadata = bdev && blk_get_integrity(bdev->bd_disk);
 #else
 	struct gendisk *disk = ns ? ns->disk : NULL;
+	bool supports_metadata = disk && blk_get_integrity(disk);
 #endif
+	struct nvme_ctrl *ctrl = nvme_req(req)->ctrl;
+	bool has_metadata = meta_buffer && meta_len;
 	struct bio *bio = NULL;
 #ifndef HAVE_BIO_INTEGRITY_MAP_USER
 	void *meta = NULL;
 #endif
 	int ret;
 
+	if (!nvme_ctrl_sgl_supported(ctrl))
+		dev_warn_once(ctrl->device, "using unchecked data buffer\n");
+	if (has_metadata) {
+		if (!supports_metadata)
+			return -EINVAL;
+
+		if (!nvme_ctrl_meta_sgl_supported(ctrl))
+			dev_warn_once(ctrl->device,
+				      "using unchecked metadata buffer\n");
+	}
 #ifdef HAVE_NVME_IOCTL_IO64_CMD_VEC
 #ifdef HAVE_BLK_RQ_MAP_USER_IO
-#ifdef HAVE_IO_URING_CMD_H
-	if (ioucmd && (ioucmd->flags & IORING_URING_CMD_FIXED)) {
-		struct iov_iter iter;
-
-		/* fixedbufs is only for non-vectored io */
-		if (WARN_ON_ONCE(flags & NVME_IOCTL_VEC))
-			return -EINVAL;
-		ret = io_uring_cmd_import_fixed(ubuffer, bufflen,
-#ifdef HAVE_IO_URING_CMD_IMPORT_FIXED_6_PARAMS
-				rq_data_dir(req), &iter, ioucmd, flags);
-#else
-				rq_data_dir(req), &iter, ioucmd);
-#endif
-		if (ret < 0)
-			goto out;
-		ret = blk_rq_map_user_iov(q, req, NULL, &iter, GFP_KERNEL);
-	} else {
+	if (iter)
+		ret = blk_rq_map_user_iov(q, req, NULL, iter, GFP_KERNEL);
+	else
 		ret = blk_rq_map_user_io(req, NULL, nvme_to_user_ptr(ubuffer),
 				bufflen, GFP_KERNEL, flags & NVME_IOCTL_VEC, 0,
 				0, rq_data_dir(req));
-	}
-#else
-	ret = blk_rq_map_user_io(req, NULL, nvme_to_user_ptr(ubuffer), bufflen,
-			GFP_KERNEL, flags & NVME_IOCTL_VEC, 0, 0, rq_data_dir(req));
-#endif
 #else
 	if (!((flags & NVME_IOCTL_VEC)))
 		ret = blk_rq_map_user(q, req, NULL,  nvme_to_user_ptr(ubuffer), bufflen,
@@ -252,7 +244,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 		ret = import_iovec(rq_data_dir(req), nvme_to_user_ptr(ubuffer), bufflen,
 				UIO_FASTIOV, &iov, &iter);
 		if (ret < 0)
-			goto out;
+			return ret;
 		ret = blk_rq_map_user_iov(q, req, NULL, &iter,
 				GFP_KERNEL);
 		kfree(iov);
@@ -264,30 +256,28 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 #endif /* HAVE_NVME_IOCTL_IO64_CMD_VEC */
 
 	if (ret)
-		goto out;
+		return ret;
 
 	bio = req->bio;
 #ifdef HAVE_BIO_INTEGRITY_MAP_USER
-	if (bdev) {
+	if (bdev)
 		bio_set_dev(bio, bdev);
-		if (meta_buffer && meta_len) {
+
+	if (has_metadata) {
 #ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER
 #ifdef HAVE_BLK_RQ_INTEGRITY_MAP_USER_GET_3_PARAM
-			ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len);
+		ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len);
 #else
-			ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len,
-							meta_seed);
+		ret = blk_rq_integrity_map_user(req, meta_buffer, meta_len, 0);
 #endif
 #else
-			ret = bio_integrity_map_user(bio, meta_buffer, meta_len,
-						     meta_seed);
+		ret = bio_integrity_map_user(bio, meta_buffer, meta_len, 0);
 #endif
-			if (ret)
-				goto out_unmap;
+		if (ret)
+			goto out_unmap;
 #ifndef HAVE_BLK_RQ_INTEGRITY_MAP_USER
-			req->cmd_flags |= REQ_INTEGRITY;
+		req->cmd_flags |= REQ_INTEGRITY;
 #endif
-		}
 	}
 #else
 #ifdef HAVE_BIO_BI_DISK
@@ -308,9 +298,7 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 	}
 	if (disk && meta_buffer && meta_len) {
 #endif /* HAVE_BIO_BI_DISK */
-		meta = nvme_add_user_metadata(req, meta_buffer, meta_len,
-					meta_seed);
-
+		meta = nvme_add_user_metadata(req, meta_buffer, meta_len, 0);
 		if (IS_ERR(meta)) {
 			ret = PTR_ERR(meta);
 			goto out_unmap;
@@ -324,14 +312,12 @@ static int nvme_map_user_request(struct request *req, u64 ubuffer,
 out_unmap:
 	if (bio)
 		blk_rq_unmap_user(bio);
-out:
-	blk_mq_free_request(req);
 	return ret;
 }
 
 static int nvme_submit_user_cmd(struct request_queue *q,
 		struct nvme_command *cmd, u64 ubuffer, unsigned bufflen,
-		void __user *meta_buffer, unsigned meta_len, u32 meta_seed,
+		void __user *meta_buffer, unsigned meta_len,
 		u64 *result, unsigned timeout, unsigned int flags)
 {
 #if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)
@@ -355,23 +341,19 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 	req->timeout = timeout;
 	if (ubuffer && bufflen) {
 		ret = nvme_map_user_request(req, ubuffer, bufflen, meta_buffer,
-				meta_len, meta_seed,
-#ifdef HAVE_IO_URING_CMD_H
-				NULL,
-#endif
+				meta_len, NULL,
 #ifndef HAVE_BIO_INTEGRITY_MAP_USER
 				&meta,
 #endif
 				flags);
-
 		if (ret)
-			return ret;
+			goto out_free_req;
 	}
 
 	bio = req->bio;
 	ctrl = nvme_req(req)->ctrl;
 
-#if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)
+#if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM) || defined(HAVE_BLK_EXECUTE_RQ_3_PARAM)	
 	effects = nvme_passthru_start(ctrl, ns, cmd->common.opcode);
 #if defined(HAVE_BLK_EXECUTE_RQ_2_PARAM)
 	ret = nvme_execute_rq(req, false);
@@ -411,7 +393,10 @@ static int nvme_submit_user_cmd(struct request_queue *q,
 	if (effects)
 		nvme_passthru_end(ctrl, ns, effects, cmd, ret);
 #endif
+	return ret;
 
+out_free_req:
+	blk_mq_free_request(req);
 	return ret;
 }
 
@@ -470,11 +455,11 @@ static int nvme_submit_io(struct nvme_ns *ns, struct nvme_user_io __user *uio)
 	c.rw.control = cpu_to_le16(io.control);
 	c.rw.dsmgmt = cpu_to_le32(io.dsmgmt);
 	c.rw.reftag = cpu_to_le32(io.reftag);
-	c.rw.apptag = cpu_to_le16(io.apptag);
-	c.rw.appmask = cpu_to_le16(io.appmask);
+	c.rw.lbat = cpu_to_le16(io.apptag);
+	c.rw.lbatm = cpu_to_le16(io.appmask);
 
 	return nvme_submit_user_cmd(ns->queue, &c, io.addr, length, metadata,
-			meta_len, lower_32_bits(io.slba), NULL, 0, 0);
+			meta_len, NULL, 0, 0);
 }
 
 static bool nvme_validate_passthru_nsid(struct nvme_ctrl *ctrl,
@@ -482,8 +467,7 @@ static bool nvme_validate_passthru_nsid(struct nvme_ctrl *ctrl,
 {
 	if (ns && nsid != ns->head->ns_id) {
 		dev_err(ctrl->device,
-			"%s: nsid (%u) in cmd does not match nsid (%u)"
-			"of namespace\n",
+			"%s: nsid (%u) in cmd does not match nsid (%u) of namespace\n",
 			current->comm, nsid, ns->head->ns_id);
 		return false;
 	}
@@ -533,7 +517,7 @@ static int nvme_user_cmd(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 
 	status = nvme_submit_user_cmd(ns ? ns->queue : ctrl->admin_q, &c,
 			cmd.addr, cmd.data_len, nvme_to_user_ptr(cmd.metadata),
-			cmd.metadata_len, 0, &result, timeout, 0);
+			cmd.metadata_len, &result, timeout, 0);
 
 	if (status >= 0) {
 		if (put_user(result, &ucmd->result))
@@ -584,7 +568,7 @@ static int nvme_user_cmd64(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 
 	status = nvme_submit_user_cmd(ns ? ns->queue : ctrl->admin_q, &c,
 			cmd.addr, cmd.data_len, nvme_to_user_ptr(cmd.metadata),
-			cmd.metadata_len, 0, &cmd.result, timeout, flags);
+			cmd.metadata_len, &cmd.result, timeout, flags);
 
 	if (status >= 0) {
 		if (put_user(cmd.result, &ucmd->result))
@@ -617,7 +601,11 @@ struct nvme_uring_cmd_pdu {
 static inline struct nvme_uring_cmd_pdu *nvme_uring_cmd_pdu(
 		struct io_uring_cmd *ioucmd)
 {
+#ifdef HAVE_IO_URING_CMD_TO_PDU
+	return io_uring_cmd_to_pdu(ioucmd, struct nvme_uring_cmd_pdu);
+#else
 	return (struct nvme_uring_cmd_pdu *)&ioucmd->pdu;
+#endif
 }
 
 static void nvme_uring_task_cb(struct io_uring_cmd *ioucmd,
@@ -636,20 +624,34 @@ static enum rq_end_io_ret nvme_uring_cmd_end_io(struct request *req,
 	struct io_uring_cmd *ioucmd = req->end_io_data;
 	struct nvme_uring_cmd_pdu *pdu = nvme_uring_cmd_pdu(ioucmd);
 
-	if (nvme_req(req)->flags & NVME_REQ_CANCELLED)
+	if (nvme_req(req)->flags & NVME_REQ_CANCELLED) {
 		pdu->status = -EINTR;
-	else
+	} else {
 		pdu->status = nvme_req(req)->status;
+		if (!pdu->status)
+			pdu->status = blk_status_to_errno(err);
+	}
 	pdu->result = le64_to_cpu(nvme_req(req)->result.u64);
 
 	/*
-	 * For iopoll, complete it directly.
+	 * For iopoll, complete it directly. Note that using the uring_cmd
+	 * helper for this is safe only because we check blk_rq_is_poll().
+	 * As that returns false if we're NOT on a polled queue, then it's
+	 * safe to use the polled completion helper.
+	 *
 	 * Otherwise, move the completion to task work.
 	 */
-	if (blk_rq_is_poll(req))
+	if (blk_rq_is_poll(req)) {
+#ifdef HAVE_IO_URING_CMD_IOPOLL_DONE
+		if (pdu->bio)
+			blk_rq_unmap_user(pdu->bio);
+		io_uring_cmd_iopoll_done(ioucmd, pdu->result, pdu->status);
+#else
 		nvme_uring_task_cb(ioucmd, IO_URING_F_UNLOCKED);
-	else
+#endif
+	} else {
 		io_uring_cmd_do_in_task_lazy(ioucmd, nvme_uring_task_cb);
+	}
 
 	return RQ_END_IO_FREE;
 }
@@ -662,6 +664,8 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	struct request_queue *q = ns ? ns->queue : ctrl->admin_q;
 	struct nvme_uring_data d;
 	struct nvme_command c;
+	struct iov_iter iter;
+	struct iov_iter *map_iter = NULL;
 	struct request *req;
 	blk_opf_t rq_flags = REQ_ALLOC_CACHE;
 	blk_mq_req_flags_t blk_flags = 0;
@@ -701,6 +705,24 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	d.metadata_len = READ_ONCE(cmd->metadata_len);
 	d.timeout_ms = READ_ONCE(cmd->timeout_ms);
 
+	if (d.data_len && (ioucmd->flags & IORING_URING_CMD_FIXED)) {
+		/* fixedbufs is only for non-vectored io */
+		if (vec)
+			return -EINVAL;
+
+		ret = io_uring_cmd_import_fixed(d.addr, d.data_len,
+#ifdef HAVE_IO_URING_CMD_IMPORT_FIXED_6_PARAMS
+			nvme_is_write(&c) ? WRITE : READ, &iter, ioucmd,
+			issue_flags);
+#else
+			nvme_is_write(&c) ? WRITE : READ, &iter, ioucmd);
+#endif
+		if (ret < 0)
+			return ret;
+
+		map_iter = &iter;
+	}
+
 	if (issue_flags & IO_URING_F_NONBLOCK) {
 		rq_flags |= REQ_NOWAIT;
 		blk_flags = BLK_MQ_REQ_NOWAIT;
@@ -713,12 +735,12 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 		return PTR_ERR(req);
 	req->timeout = d.timeout_ms ? msecs_to_jiffies(d.timeout_ms) : 0;
 
-	if (d.addr && d.data_len) {
-		ret = nvme_map_user_request(req, d.addr,
-			d.data_len, nvme_to_user_ptr(d.metadata),
-			d.metadata_len, 0, ioucmd, vec);
+	if (d.data_len) {
+		ret = nvme_map_user_request(req, d.addr, d.data_len,
+			nvme_to_user_ptr(d.metadata), d.metadata_len,
+			map_iter, vec);
 		if (ret)
-			return ret;
+			goto out_free_req;
 	}
 
 	/* to free bio on completion, as req->bio will be null at that time */
@@ -728,6 +750,10 @@ static int nvme_uring_cmd_io(struct nvme_ctrl *ctrl, struct nvme_ns *ns,
 	req->end_io = nvme_uring_cmd_end_io;
 	blk_execute_rq_nowait(req, false);
 	return -EIOCBQUEUED;
+
+out_free_req:
+	blk_mq_free_request(req);
+	return ret;
 }
 #endif // HAVE_IO_URING_CMD_H
 
@@ -865,8 +891,6 @@ static int nvme_ns_uring_cmd(struct nvme_ns *ns, struct io_uring_cmd *ioucmd,
 	struct nvme_ctrl *ctrl = ns->ctrl;
 	int ret;
 
-	BUILD_BUG_ON(sizeof(struct nvme_uring_cmd_pdu) > sizeof(ioucmd->pdu));
-
 	ret = nvme_uring_cmd_checks(issue_flags);
 	if (ret)
 		return ret;
