@@ -457,7 +457,8 @@ static int resources_create(struct resources *res)
         (res->dev_rev_id != DI_BF3) &&
         (res->dev_rev_id != DI_BF4) &&
         (res->dev_rev_id != DI_CX7) &&
-        (res->dev_rev_id != DI_CX8)) {
+        (res->dev_rev_id != DI_CX8) &&
+        (res->dev_rev_id != DI_CX9)) {
         fprintf(stderr, "-E- Unsupported HW device id (%x)\n", res->dev_rev_id);
         return -1;
     }
@@ -969,8 +970,15 @@ int fifth_gen_set_sw_sniffer(struct resources *res, int mode)
     struct SNIFFER_STRUCT set_port_sniffer;
     int                   rc;
 
-    /* To disable write protection */
-    mwrite4(res->mf, 0x23f0, 0xbadc0ffe);
+    
+    if (res->dev_rev_id == DI_CX8 || res->dev_rev_id == DI_CX9)
+    {
+        mwrite4(res->mf, 0x1a023f0, 0xbadc0ffe);
+    }
+    else
+    {
+        mwrite4(res->mf, 0x23f0, 0xbadc0ffe);
+    }
 
     memset(&set_port_sniffer, 0, sizeof(struct SNIFFER_STRUCT));
     set_port_sniffer.port = config.ib_port;
@@ -999,8 +1007,9 @@ int set_sw_sniffer(struct resources *res, int mode, int is_tx, int is_rx)
         (res->dev_rev_id == DI_CX4LX) || (res->dev_rev_id == DI_CX5) ||
         (res->dev_rev_id == DI_CX6) || (res->dev_rev_id == DI_CX6DX) ||
         (res->dev_rev_id == DI_CX7) || (res->dev_rev_id != DI_CX8) ||
-        (res->dev_rev_id != DI_BF2) || (res->dev_rev_id != DI_BF2A1) ||
-        (res->dev_rev_id != DI_BF3) || (res->dev_rev_id != DI_BF4))
+        (res->dev_rev_id != DI_CX9) || (res->dev_rev_id != DI_BF2) ||
+        (res->dev_rev_id != DI_BF2A1) || (res->dev_rev_id != DI_BF3) ||
+        (res->dev_rev_id != DI_BF4))
     {
         return fifth_gen_set_sw_sniffer(res, mode);
     }
