@@ -81,7 +81,7 @@ MODULE_INFO(retpoline, "Y");
 #endif
 MODULE_AUTHOR("Alex Nezhinsky, Dan Bar Dov, Or Gerlitz");
 
-static struct scsi_host_template iscsi_iser_sht;
+static const struct scsi_host_template iscsi_iser_sht;
 static struct iscsi_transport iscsi_iser_transport;
 static struct scsi_transport_template *iscsi_iser_scsi_transport;
 static struct workqueue_struct *release_wq;
@@ -116,12 +116,6 @@ bool iser_pi_enable = false;
 module_param_named(pi_enable, iser_pi_enable, bool, S_IRUGO);
 MODULE_PARM_DESC(pi_enable, "Enable T10-PI offload support (default:disabled)");
 
-#ifndef HAVE_SCSI_CMND_PROT_FLAGS
-int iser_pi_guard;
-module_param_named(pi_guard, iser_pi_guard, int, S_IRUGO);
-MODULE_PARM_DESC(pi_guard, "T10-PI guard_type, 0:CRC|1:IP_CSUM (default:IP_CSUM)");
-#endif
-
 static int iscsi_iser_set(const char *val, const struct kernel_param *kp)
 {
 	int ret;
@@ -230,27 +224,6 @@ int iser_initialize_task_headers(struct iscsi_task *task,
 	return 0;
 }
 
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
-/**
- * set_last_ping_on_nopout_task()
- * @task: iscsi task
- *
- * Workaround libiscsi not setting iscsi_conn->last_ping
- * in case of failure.
- * fixed in kernels > 4.3
- */
-static inline void
-set_last_ping_on_nopout_task(struct iscsi_task *task)
-{
-	u8 task_opcode = (task->hdr->opcode & ISCSI_OPCODE_MASK);
-
-	if (task_opcode == ISCSI_OP_NOOP_OUT)
-		task->conn->last_ping = jiffies;
-}
-#endif
-
-
 /**
  * iscsi_iser_task_init() - Initialize iscsi-iser task
  * @task: iscsi task
@@ -267,17 +240,8 @@ static int iscsi_iser_task_init(struct iscsi_task *task)
 
 	ret = iser_initialize_task_headers(task, &iser_task->desc);
 	if (ret) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
-		u8 task_opcode = (task->hdr->opcode & ISCSI_OPCODE_MASK);
-
-		iser_err("Failed to init task %p, opcode %d, err = %d",
-			 iser_task, task_opcode, ret);
-
-		set_last_ping_on_nopout_task(task);
-#else
 		iser_err("Failed to init task %p, err = %d\n",
 			 iser_task, ret);
-#endif
 
 		return ret;
 	}
@@ -312,17 +276,6 @@ static int iscsi_iser_mtask_xmit(struct iscsi_conn *conn,
 	iser_dbg("mtask xmit [cid %d itt 0x%x]\n", conn->id, task->itt);
 
 	error = iser_send_control(conn, task);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 3, 0)
-	if (error) {
-		u8 task_opcode = (task->hdr->opcode & ISCSI_OPCODE_MASK);
-
-		iser_err("Failed to send task %p, opcode %d, err = %d",
-			 task->dd_data, task_opcode, error);
-
-		set_last_ping_on_nopout_task(task);
-
-	}
-#endif
 
 	/* since iser xmits control with zero copy, tasks can not be recycled
 	 * right after sending them.
@@ -444,10 +397,10 @@ static void iscsi_iser_cleanup_task(struct iscsi_task *task)
  * @task:     iscsi task
  * @sector:   error sector if exsists (output)
  *
- * Return: zero if no data-integrity errors have occured
- *         0x1: data-integrity error occured in the guard-block
- *         0x2: data-integrity error occured in the reference tag
- *         0x3: data-integrity error occured in the application tag
+ * Return: zero if no data-integrity errors have occurred
+ *         0x1: data-integrity error occurred in the guard-block
+ *         0x2: data-integrity error occurred in the reference tag
+ *         0x3: data-integrity error occurred in the application tag
  *
  *         In addition the error sector is marked.
  */
@@ -635,9 +588,7 @@ static inline unsigned int iser_dif_prot_caps(int prot_caps)
 
 	if (prot_caps & IB_PROT_T10DIF_TYPE_1)
 		ret |= SHOST_DIF_TYPE1_PROTECTION |
-#ifdef HAVE_SCSI_CMND_PROT_FLAGS
 		       SHOST_DIX_TYPE0_PROTECTION |
-#endif
 		       SHOST_DIX_TYPE1_PROTECTION;
 	if (prot_caps & IB_PROT_T10DIF_TYPE_2)
 		ret |= SHOST_DIF_TYPE2_PROTECTION |
@@ -705,15 +656,8 @@ iscsi_iser_session_create(struct iscsi_endpoint *ep,
 
 			shost->sg_prot_tablesize = shost->sg_tablesize;
 			scsi_host_set_prot(shost, iser_dif_prot_caps(sig_caps));
-#ifdef HAVE_SCSI_CMND_PROT_FLAGS
 			scsi_host_set_guard(shost, SHOST_DIX_GUARD_IP |
 						   SHOST_DIX_GUARD_CRC);
-#else
-			if (iser_pi_guard)
-				scsi_host_set_guard(shost, SHOST_DIX_GUARD_IP);
-			else
-				scsi_host_set_guard(shost, SHOST_DIX_GUARD_CRC);
-#endif
 		}
 #ifdef HAVE_SCSI_HOST_VIRT_BOUNDARY_MASK
 		if (!(ib_dev->attrs.kernel_cap_flags & IBK_SG_GAPS_REG))
@@ -826,13 +770,7 @@ static void iscsi_iser_conn_get_stats(struct iscsi_cls_conn *cls_conn,
 	stats->r2t_pdus = conn->r2t_pdus_cnt; /* always 0 */
 	stats->tmfcmd_pdus = conn->tmfcmd_pdus_cnt;
 	stats->tmfrsp_pdus = conn->tmfrsp_pdus_cnt;
-#ifdef HAVE_VIRT_BOUNDARY
 	stats->custom_length = 0;
-#else
-	stats->custom_length = 1;
-	strcpy(stats->custom[0].desc, "fmr_unalign_cnt");
-	stats->custom[0].value = conn->fmr_unalign_cnt;
-#endif
 }
 
 static int iscsi_iser_get_ep_param(struct iscsi_endpoint *ep,
@@ -1032,7 +970,7 @@ static umode_t iser_attr_is_visible(int param_type, int param)
 	return 0;
 }
 
-#ifdef USE_SLAVE_ALLOC_HANDLER
+#ifndef HAVE_SCSI_HOST_VIRT_BOUNDARY_MASK
 static int iscsi_iser_slave_alloc(struct scsi_device *sdev)
 {
 	struct iscsi_session *session;
@@ -1057,20 +995,14 @@ static int iscsi_iser_slave_alloc(struct scsi_device *sdev)
 }
 #endif
 
-static struct scsi_host_template iscsi_iser_sht = {
+static const struct scsi_host_template iscsi_iser_sht = {
 	.module                 = THIS_MODULE,
 	.name                   = "iSCSI Initiator over iSER",
 	.queuecommand           = iscsi_queuecommand,
-#ifdef HAVE_SCSI_CHANGE_QUEUE_DEPTH
 	.change_queue_depth	= scsi_change_queue_depth,
-#else
-	.change_queue_depth	= iscsi_change_queue_depth,
-#endif
 	.sg_tablesize           = ISCSI_ISER_DEF_SG_TABLESIZE,
 	.cmd_per_lun            = ISER_DEF_CMD_PER_LUN,
-#ifdef HAVE_ISCSI_EH_CMD_TIMED_OUT
 	.eh_timed_out		= iscsi_eh_cmd_timed_out,
-#endif
 	.eh_abort_handler       = iscsi_eh_abort,
 	.eh_device_reset_handler= iscsi_eh_device_reset,
 	.eh_target_reset_handler = iscsi_eh_recover_target,
@@ -1078,14 +1010,12 @@ static struct scsi_host_template iscsi_iser_sht = {
 #ifdef ENABLE_CLUSTERING
 	.use_clustering		= ENABLE_CLUSTERING,
 #endif
-#ifdef USE_SLAVE_ALLOC_HANDLER
+#ifndef HAVE_SCSI_HOST_VIRT_BOUNDARY_MASK
 	.slave_alloc		= iscsi_iser_slave_alloc,
 #endif
 	.proc_name              = "iscsi_iser",
 	.this_id                = -1,
-#ifdef HAVE_SCSI_HOST_TEMPLATE_TRACK_QUEUE_DEPTH
 	.track_queue_depth	= 1,
-#endif
 #ifdef HAVE_ISCSI_CMD
 	.cmd_size		= sizeof(struct iscsi_cmd),
 #endif
@@ -1137,13 +1067,6 @@ static int __init iser_init(void)
 
 	iser_dbg("Starting iSER datamover...\n");
 
-#ifndef HAVE_SCSI_CMND_PROT_FLAGS
-	if (iser_pi_guard < 0 || iser_pi_guard > 1) {
-		iser_err("Invalid pi_guard value of %d\n", iser_pi_guard);
-		return -EINVAL;
-	}
-#endif
-
 	memset(&ig, 0, sizeof(struct iser_global));
 
 	ig.desc_cache = kmem_cache_create("iser_descriptors",
