@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
+ * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
  *
  * This software product is a proprietary product of NVIDIA CORPORATION &
  * AFFILIATES (the "Company") and all right, title, and interest in and to the
@@ -34,6 +34,7 @@ VLOG_DEFINE_THIS_MODULE(ovs_doca);
 #define OVS_DOCA_CONGESTION_THRESHOLD_MAX 90
 
 /* Indicates successful initialization of DOCA. */
+static atomic_bool doca_enabled = ATOMIC_VAR_INIT(false);
 static atomic_bool doca_initialized = ATOMIC_VAR_INIT(false);
 static FILE *log_stream = NULL;       /* Stream for DOCA log redirection */
 static struct doca_log_backend *ovs_doca_log = NULL;
@@ -66,6 +67,15 @@ ovs_doca_meter_counters_per_esw_base_id(void)
 
 bool
 ovs_doca_enabled(void)
+{
+    bool enabled;
+
+    atomic_read_relaxed(&doca_enabled, &enabled);
+    return enabled;
+}
+
+bool
+ovs_doca_initialized(void)
 {
     bool initialized;
 
@@ -218,6 +228,13 @@ ovs_doca_dynamic_config(const struct smap *config)
         return;
     }
 
+    /* Once the user request a DOCA run, we must modify all future logic
+     * (DPDK port probing) to take it into account, even if it results
+     * in a failure. Once set, this value won't change. */
+    if (!ovs_doca_enabled()) {
+        atomic_store(&doca_enabled, true);
+    }
+
     req_doca_async = smap_get_bool(config, "doca-async", true);
     if (req_doca_async != ovs_doca_async) {
         VLOG_INFO("Changing DOCA insertion mode from %s to %s.",
@@ -249,6 +266,7 @@ ovs_doca_init(const struct smap *ovs_other_config)
     uint32_t req_ctl_pipe_infra_size;
     struct doca_flow_cfg cfg = {};
     static bool enabled = false;
+    uint16_t rss_queues;
     doca_error_t err;
 
     /* Dynamic configuration:
@@ -336,12 +354,15 @@ ovs_doca_init(const struct smap *ovs_other_config)
                            doca_congestion_threshold);
         }
 
-        cfg.queues = smap_get_uint(ovs_other_config, "n-offload-threads", 1);
+        cfg.pipe_queues = smap_get_uint(ovs_other_config, "n-offload-threads", 1);
         cfg.resource.nb_counters = OVS_DOCA_MAX_MEGAFLOWS_COUNTERS;
         cfg.mode_args = "switch,hws,cpds";
         cfg.queue_depth = OVS_DOCA_QUEUE_DEPTH;
         cfg.cb = ovs_doca_entry_process_cb;
         cfg.pipe_process_cb = ovs_doca_pipe_process_cb;
+        cfg.rss.nr_queues = 1;
+        rss_queues = 0;
+        cfg.rss.queues_array = &rss_queues;
         /* Set the sum of counters we want for both ports */
         cfg.nr_shared_resources[DOCA_FLOW_SHARED_RESOURCE_COUNT] =
             ovs_doca_max_shared_counters();
@@ -409,7 +430,7 @@ ovs_doca_status(const struct ovsrec_open_vswitch *cfg)
     if (!cfg) {
         return;
     }
-    ovsrec_open_vswitch_set_doca_initialized(cfg, ovs_doca_enabled());
+    ovsrec_open_vswitch_set_doca_initialized(cfg, ovs_doca_initialized());
     ovsrec_open_vswitch_set_doca_version(cfg, doca_version_runtime());
 }
 
