@@ -201,4 +201,22 @@ struct read_dword_from_config_space
 };
 
 #define PCICONF_RESCAN _IOC(_IOC_NONE, PCICONF_MAGIC, 16, 0)
+
+#define PCICONF_HOT_RESET _IOR(PCICONF_MAGIC, 17, struct hot_reset_pcie_switch)
+
+struct dbdf
+{
+    int domain;
+    int bus;
+    int device;
+    int function;
+};
+
+struct hot_reset_pcie_switch
+{
+    struct dbdf prime;
+    struct dbdf aux;
+    int is_socket_direct;
+};
+
 #endif
