@@ -35,8 +35,8 @@
 #include <linux/errno.h>
 
 #define DRV_NAME	"9pnet_rdma"
-#define DRV_VERSION	"24.10-4.1.4"
-#define DRV_RELDATE	"07 Jan 2026"
+#define DRV_VERSION	"25.07-0.9.7"
+#define DRV_RELDATE	"04 Aug 2025"
 
 MODULE_AUTHOR("Alaa Hleihel");
 MODULE_DESCRIPTION("9pnet_rdma dummy kernel module");
