@@ -38,6 +38,12 @@ int find_all_vendor_devices(unsigned int vendor_id);
      since it could be removed by hotplug event. */
 LIST_HEAD(nnt_device_list);
 
+unsigned short supported_rma_device_ids[] = {QUANTUM3_RMA_PCI_ID,  QUANTUM4_RMA_PCI_ID,   QUANTUM5_RMA_PCI_ID,
+                                             CONNECTX8_RMA_PCI_ID, ARCUSEE_RMA_PCI_ID,    QUANTUM2_RMA_PCI_ID,
+                                             CONNECTX7_RMA_PCI_ID, BLUEFIELD3_RMA_PCI_ID, SPECTRUM4_RMA_PCI_ID,
+                                             SPECTRUM5_RMA_PCI_ID, SPECTRUM6_RMA_PCI_ID};
+const unsigned int SUPPORTED_RMA_DEVICE_IDS_TABLE_SIZE = sizeof(supported_rma_device_ids) / sizeof(unsigned short);
+
 int get_nnt_device(struct file* file, struct nnt_device** nnt_device)
 {
     int error_code = 0;
@@ -49,12 +55,6 @@ int get_nnt_device(struct file* file, struct nnt_device** nnt_device)
     else
     {
         *nnt_device = file->private_data;
-
-        if (!(*nnt_device)->pci_device)
-        {
-            printk(KERN_ERR "mst_pciconf driver: pci_device pointer is NULL\n");
-            error_code = -EINVAL;
-        }
     }
 
     return error_code;
@@ -208,6 +208,27 @@ int create_nnt_device(struct pci_dev* pci_device, enum nnt_device_type device_ty
     nnt_device->pci_device = pci_device;
     nnt_device->device_type = device_type;
 
+    if (device_type == NNT_PCICONF)
+    {
+        DEBUG_PRINTK("Created NNT_PCICONF device - Device ID: %d, Domain: 0x%x, Bus: 0x%x, Slot: %d, Function: %d\n\n",
+                     pci_device->device, pci_domain_nr(pci_device->bus), pci_device->bus->number,
+                     PCI_SLOT(pci_device->devfn), PCI_FUNC(pci_device->devfn));
+    }
+    else if (device_type == NNT_PCI_MEMORY)
+    {
+        DEBUG_PRINTK(
+          "Created NNT_PCI_MEMORY device - Device ID: %d, Domain: 0x%x, Bus: 0x%x, Slot: %d, Function: %d\n\n",
+          pci_device->device, pci_domain_nr(pci_device->bus), pci_device->bus->number, PCI_SLOT(pci_device->devfn),
+          PCI_FUNC(pci_device->devfn));
+    }
+    else if (device_type == NNT_PCICONF_RECOVERY)
+    {
+        DEBUG_PRINTK(
+          "Created NNT_PCICONF_RECOVERY device - Device ID: %d, Domain: 0x%x, Bus: 0x%x, Slot: %d, Function: %d\n\n",
+          pci_device->device, pci_domain_nr(pci_device->bus), pci_device->bus->number, PCI_SLOT(pci_device->devfn),
+          PCI_FUNC(pci_device->devfn));
+    }
+
     /* Add the nnt device structure to the list. */
     list_add_tail(&nnt_device->entry, &nnt_device_list);
 
@@ -263,9 +284,23 @@ int is_switch_ib(unsigned short pci_device_id)
     return pci_device_id == SWITCHIB_PCI_ID || pci_device_id == SWITCHIB2_PCI_ID;
 }
 
+int is_rma_device(unsigned short pci_device_id)
+{
+    int i;
+    for (i = 0; i < SUPPORTED_RMA_DEVICE_IDS_TABLE_SIZE; i++)
+    {
+        if (supported_rma_device_ids[i] == pci_device_id)
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
 int is_livefish_device(unsigned short pci_device_id)
 {
-    return pci_device_id >= CONNECTX3_LIVEFISH_ID && pci_device_id < CONNECTX3_PCI_ID;
+    return pci_device_id >= CONNECTX3_LIVEFISH_ID && pci_device_id < CONNECTX3_PCI_ID &&
+           !(is_rma_device(pci_device_id));
 }
 
 int is_nic(unsigned short pci_device_id)
@@ -287,7 +322,13 @@ int is_toolspf(unsigned short pci_device_id)
 int is_pciconf_device(unsigned short pci_device_id)
 {
     int result = is_nic(pci_device_id) || is_toolspf(pci_device_id) || is_livefish_device(pci_device_id) ||
-                 is_switch(pci_device_id) || is_gpu_pci_device(pci_device_id);
+                 is_rma_device(pci_device_id) || is_switch(pci_device_id) || is_gpu_pci_device(pci_device_id);
+
+    DEBUG_PRINTK(
+      "is_pciconf_device check for device 0x%x: result=%d (is_nic: %d, is_toolspf: %d, is_livefish: %d, is_rma: %d, is_switch: %d, is_gpu: %d)\n",
+      pci_device_id, result, is_nic(pci_device_id), is_toolspf(pci_device_id), is_livefish_device(pci_device_id),
+      is_rma_device(pci_device_id), is_switch(pci_device_id), is_gpu_pci_device(pci_device_id));
+
     return result;
 }
 
@@ -296,6 +337,12 @@ int is_pcicr_device(unsigned short pci_device_id)
     int result = (is_gpu_pci_device(pci_device_id) || is_switch(pci_device_id) || is_toolspf(pci_device_id) ||
                   is_connectx3(pci_device_id)) &&
                  (!is_livefish_device(pci_device_id));
+
+    DEBUG_PRINTK(
+      "is_pcicr_device check for device 0x%x: result=%d (is_gpu: %d, is_toolspf: %d, is_connectx3: %d, !is_livefish: %d)\n",
+      pci_device_id, result, is_gpu_pci_device(pci_device_id), is_toolspf(pci_device_id), is_connectx3(pci_device_id),
+      !is_livefish_device(pci_device_id));
+
     return result;
 }
 
@@ -356,33 +403,67 @@ ReturnOnFinished:
 unsigned int get_vsc_address_by_type(struct nnt_device* current_nnt_device, int vsc_type)
 {
     unsigned int vsc_address = 0;
+    int type_check_result = 0;
+
+    DEBUG_PRINTK("Starting VSC search for device ID: %d, looking for VSC type: %d\n",
+                 current_nnt_device->pci_device->device, vsc_type);
+
+    // DEBUG_PRINTK("Current PCI device details:\n");
+    // DEBUG_PRINTK("  - Device ID: 0x%x\n", current_nnt_device->pci_device->device);
+    // DEBUG_PRINTK("  - Vendor ID: 0x%x\n", current_nnt_device->pci_device->vendor);
+    // DEBUG_PRINTK("  - Subsystem ID: 0x%x\n", current_nnt_device->pci_device->subsystem_device);
+    // DEBUG_PRINTK("  - Subsystem Vendor: 0x%x\n", current_nnt_device->pci_device->subsystem_vendor);
+    // DEBUG_PRINTK("  - Header Type: 0x%x\n", current_nnt_device->pci_device->hdr_type);
+    // DEBUG_PRINTK("  - Class: 0x%x\n", current_nnt_device->pci_device->class);
+    // DEBUG_PRINTK("  - Bus: 0x%x\n", current_nnt_device->pci_device->bus->number);
+    // DEBUG_PRINTK("  - Device: %d\n", PCI_SLOT(current_nnt_device->pci_device->devfn));
+    // DEBUG_PRINTK("  - Function: %d\n", PCI_FUNC(current_nnt_device->pci_device->devfn));
+    // DEBUG_PRINTK("  - Domain: 0x%x\n", pci_domain_nr(current_nnt_device->pci_device->bus));
+    // DEBUG_PRINTK("  - Error State: %d\n", current_nnt_device->pci_device->error_state);
 
     // Attempt to find the first Vendor-Specific Capability (VSC)
     vsc_address = pci_find_capability(current_nnt_device->pci_device, VENDOR_SPECIFIC_CAPABILITY_ID);
+    DEBUG_PRINTK("pci_find_capability (VSC) returned: 0x%x for device ID: %d\n", vsc_address,
+                 current_nnt_device->pci_device->device);
 
     // Check if the first VSC is of the specified type
-    if (vsc_address && is_vsc_type(current_nnt_device, vsc_address, vsc_type))
+    if (vsc_address)
     {
-        // printk(KERN_DEBUG "Device ID :%d has VSC of type: %d at offset:%x of the config space\n",
-        //        current_nnt_device->pci_device->device, vsc_type, vsc_address);
-        return vsc_address;
+        DEBUG_PRINTK("Found VSC at address 0x%x\n", vsc_address);
+        type_check_result = is_vsc_type(current_nnt_device, vsc_address, vsc_type);
+        if (type_check_result == 1)
+        {
+            return vsc_address;
+        }
+        else if (type_check_result == -1)
+        {
+            DEBUG_PRINTK("Error reading VSC type at address 0x%x for device ID: %d\n", vsc_address,
+                         current_nnt_device->pci_device->device);
+            return vsc_address;
+        }
     }
 
     // Iterate through the capabilities linked list to find the specified VSC type
     while ((vsc_address =
               pci_find_next_capability(current_nnt_device->pci_device, vsc_address, VENDOR_SPECIFIC_CAPABILITY_ID)))
     {
-        if (is_vsc_type(current_nnt_device, vsc_address, vsc_type))
+        DEBUG_PRINTK("Checking next VSC at address 0x%x\n", vsc_address);
+        type_check_result = is_vsc_type(current_nnt_device, vsc_address, vsc_type);
+        if (type_check_result == 1)
         {
-            // printk(KERN_DEBUG "Device ID :%d has VSC of type: %d at offset:%x of the config space\n",
-            //        current_nnt_device->pci_device->device, vsc_type, vsc_address);
+            return vsc_address;
+        }
+        else if (type_check_result == -1)
+        {
+            DEBUG_PRINTK("Error reading VSC type at address 0x%x for device ID: %d\n", vsc_address,
+                         current_nnt_device->pci_device->device);
             return vsc_address;
         }
     }
 
-    // printk(KERN_DEBUG "No VSC of type: %d found for device ID: %d\n", vsc_type,
-    // current_nnt_device->pci_device->device);
-    return 0; // Return 0 if no VSC of the specified type is found
+    DEBUG_PRINTK("No VSC of type: %d found for device ID: %d after checking all capabilities\n", vsc_type,
+                 current_nnt_device->pci_device->device);
+    return 0;
 }
 
 /* Return 1 if VSC is of type vsc_type */
@@ -410,27 +491,36 @@ int create_devices(dev_t device_number, struct file_operations* fop, int is_allo
     int minor = 0;
     int error = 0;
 
+    DEBUG_PRINTK("Starting create_devices with is_alloc_chrdev_region=%d\n\n", is_alloc_chrdev_region);
+
     /* Create necessary number of the devices. */
     list_for_each_entry_safe(current_nnt_device, temp_nnt_device, &nnt_device_list, entry)
     {
+        DEBUG_PRINTK("Processing device: %s, type=%d, minor=%d\n",
+                     current_nnt_device->device_name,
+                     current_nnt_device->device_type,
+                     minor);
+
         /* Create the device file. */
         if ((error = create_device_file(current_nnt_device, device_number, minor, fop, is_alloc_chrdev_region)) != 0)
+        {
+            DEBUG_PRINTK("Failed to create device file, error: %d\n", error);
             goto ReturnOnFinished;
+        }
 
         /* Members initialization. */
         current_nnt_device->pciconf_device.functional_vsc_offset = get_vsc_address_by_type(
           current_nnt_device, FUNCTIONAL_VSC); // Exists when device is in Functional/Zombiefish mode
         if (!current_nnt_device->pciconf_device.functional_vsc_offset)
         {
+            DEBUG_PRINTK("No functional VSC found, checking for recovery VSC\n");
             current_nnt_device->pciconf_device.recovery_vsc_offset =
               get_vsc_address_by_type(current_nnt_device, RECOVERY_VSC);
             if (current_nnt_device->pciconf_device.recovery_vsc_offset != 0 &&
                 current_nnt_device->pciconf_device.recovery_vsc_offset != RECOVERY_VSC_OFFSET_IN_CONFIG_SPACE)
             {
-                printk(
-                  KERN_ERR
-                  "Found that recovery VSC offset is: %d. Which does not allign with definition which is 0x54 or 0.\n",
-                  current_nnt_device->pciconf_device.recovery_vsc_offset);
+                printk(KERN_ERR "ERROR - Invalid recovery VSC offset: 0x%x (expected 0x54 or 0).\n",
+                       current_nnt_device->pciconf_device.recovery_vsc_offset);
                 return -1;
             }
         }
@@ -441,26 +531,30 @@ int create_devices(dev_t device_number, struct file_operations* fop, int is_allo
         if (!current_nnt_device->pciconf_device.functional_vsc_offset)
         {
             current_nnt_device->device_type = NNT_PCICONF_RECOVERY;
-            // printk(KERN_DEBUG
-            //        "Device with device ID: %d is in LF or Late LF mode. current_nnt_device->device_type set to:
-            //        %d\n", current_nnt_device->pci_device->device, current_nnt_device->device_type);
+            DEBUG_PRINTK(
+              "Device with device ID: %d is in LF or Late LF mode. current_nnt_device->device_type set to: %d\n",
+              current_nnt_device->pci_device->device, current_nnt_device->device_type);
         }
 
         switch (current_nnt_device->device_type)
         {
             case NNT_PCICONF:
+                DEBUG_PRINTK("Setting up NNT_PCICONF access methods for %s\n\n", current_nnt_device->device_name);
                 current_nnt_device->access.read = read_pciconf;
                 current_nnt_device->access.write = write_pciconf;
                 current_nnt_device->access.init = init_pciconf;
                 break;
 
             case NNT_PCICONF_RECOVERY:
+                DEBUG_PRINTK("Setting up NNT_PCICONF_RECOVERY access methods for %s\n\n",
+                             current_nnt_device->device_name);
                 current_nnt_device->access.read = read_pciconf_no_vsec;
                 current_nnt_device->access.write = write_pciconf_no_vsec;
                 current_nnt_device->access.init = init_pciconf_no_vsec;
                 break;
 
             case NNT_PCI_MEMORY:
+                DEBUG_PRINTK("Setting up NNT_PCI_MEMORY access methods for %s\n\n", current_nnt_device->device_name);
                 current_nnt_device->access.read = read_memory;
                 current_nnt_device->access.write = write_memory;
                 current_nnt_device->access.init = init_memory;
@@ -470,6 +564,10 @@ int create_devices(dev_t device_number, struct file_operations* fop, int is_allo
         if (is_alloc_chrdev_region)
         {
             error = current_nnt_device->access.init(current_nnt_device);
+            if (error)
+            {
+                DEBUG_PRINTK("Failed to initialize device access, error: %d\n", error);
+            }
         }
 
         minor++;
@@ -488,18 +586,32 @@ int create_nnt_devices(dev_t device_number,
 {
     struct pci_dev* pci_device = NULL;
     int error_code = 0;
+    int is_pciconf = 0;
+    int is_pcicr = 0;
+
+    DEBUG_PRINTK("create_nnt_devices called with:\n");
+    DEBUG_PRINTK("  nnt_device_flag = %d (0=NNT_PCICONF_DEVICES, 1=NNT_PCI_DEVICES, 2=NNT_ALL_DEVICES)\n",
+                 nnt_device_flag);
+    DEBUG_PRINTK("  with_unknown = %d\n", with_unknown);
+    DEBUG_PRINTK("  vendor_id = 0x%x\n", vendor_id);
+    DEBUG_PRINTK("  is_alloc_chrdev_region = %d\n\n", is_alloc_chrdev_region);
 
     /* Find all Nvidia PCI devices. */
     while ((pci_device = pci_get_device(vendor_id, PCI_ANY_ID, pci_device)) != NULL)
     {
+        is_pciconf = is_pciconf_device(pci_device->device);
+        is_pcicr = is_pcicr_device(pci_device->device);
+
+        DEBUG_PRINTK("Device 0x%x: is_pciconf=%d, is_pcicr=%d\n", pci_device->device, is_pciconf, is_pcicr);
+
         if ((nnt_device_flag == NNT_PCICONF_DEVICES) || (nnt_device_flag == NNT_ALL_DEVICES))
         {
             /* Create pciconf device. */
-            if (with_unknown || is_pciconf_device(pci_device->device))
+            if (with_unknown || is_pciconf)
             {
                 if ((error_code = create_nnt_device(pci_device, NNT_PCICONF, is_alloc_chrdev_region)) != 0)
                 {
-                    printk(KERN_ERR "Failed to create pci conf device\n");
+                    printk(KERN_ERR "Failed to create NNT_PCICONF device\n");
                     goto ReturnOnFinished;
                 }
             }
@@ -508,11 +620,11 @@ int create_nnt_devices(dev_t device_number,
         if ((nnt_device_flag == NNT_PCI_DEVICES) || (nnt_device_flag == NNT_ALL_DEVICES))
         {
             /* Create pci memory device. */
-            if (with_unknown || is_pcicr_device(pci_device->device))
+            if (with_unknown || is_pcicr)
             {
                 if ((error_code = create_nnt_device(pci_device, NNT_PCI_MEMORY, is_alloc_chrdev_region)) != 0)
                 {
-                    printk(KERN_ERR "Failed to create pci memory device\n");
+                    printk(KERN_ERR "Failed to create NNT_PCI_MEMORY device\n");
                     goto ReturnOnFinished;
                 }
             }
@@ -695,6 +807,7 @@ int rescan(void)
                 }
 
                 current_nnt_device->pci_device = new_pci_device;
+                current_nnt_device->pci_device->error_state = pci_channel_io_normal;
                 break;
             }
         }
