@@ -13,6 +13,7 @@ int create_nnt_devices(dev_t device_number,
 void destroy_nnt_devices(int is_alloc_chrdev_region);
 void destroy_nnt_devices_bc(void);
 int destroy_nnt_device_bc(struct nnt_device* nnt_device);
+int is_rma_device(unsigned short pci_device_id);
 int is_pciconf_device(unsigned short pci_device_id);
 int is_pcicr_device(unsigned short pci_device_id);
 int get_amount_of_nvidia_devices(void);
@@ -22,5 +23,8 @@ int set_private_data_bc(struct file* file, unsigned int bus, unsigned int devfn,
 int get_nnt_device(struct file* file, struct nnt_device** nnt_device);
 int mutex_lock_nnt(struct file* file);
 void mutex_unlock_nnt(struct file* file);
+int rescan(void);
+unsigned int get_vsc_address_by_type(struct nnt_device* current_nnt_device, int vsc_type);
+int is_vsc_type(struct nnt_device* current_nnt_device, unsigned int vsec_address, unsigned int vsc_type);
 
 #endif
