@@ -1,35 +1,7 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #include "cq_mgr_rx.h"
@@ -105,9 +77,6 @@ cq_mgr_rx::cq_mgr_rx(ring_simple *p_ring, ib_ctx_handler *p_ib_ctx_handler, int
 
 void cq_mgr_rx::configure(int cq_size)
 {
-    xlio_ibv_cq_init_attr attr;
-    memset(&attr, 0, sizeof(attr));
-
     struct ibv_context *context = m_p_ib_ctx_handler->get_ibv_context();
     int comp_vector = 0;
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
@@ -119,8 +88,20 @@ void cq_mgr_rx::configure(int cq_size)
         comp_vector = g_p_app->get_worker_id() % context->num_comp_vectors;
     }
 #endif
-    m_p_ibv_cq = xlio_ibv_create_cq(context, cq_size - 1, (void *)this, m_comp_event_channel,
-                                    comp_vector, &attr);
+
+    struct ibv_cq_init_attr_ex attr = {};
+    struct mlx5dv_cq_init_attr dvattr = {};
+
+    attr.cqe = cq_size - 1; // This parameter is incremented by 1 in libibverbs
+    attr.cq_context = (void *)this;
+    attr.channel = m_comp_event_channel;
+    attr.comp_vector = comp_vector;
+    attr.wc_flags = IBV_WC_STANDARD_FLAGS;
+    attr.comp_mask = IBV_CQ_INIT_ATTR_MASK_FLAGS;
+    attr.flags = IBV_CREATE_CQ_ATTR_IGNORE_OVERRUN;
+
+    struct ibv_cq_ex *cq_ex = mlx5dv_create_cq(context, &attr, &dvattr);
+    m_p_ibv_cq = ibv_cq_ex_to_cq(cq_ex);
     BULLSEYE_EXCLUDE_BLOCK_START
     if (!m_p_ibv_cq) {
         cq_logerr("Failed to create CQ, this: %p, ctx: %p size: %d compch: %p", this, context,
