@@ -1,33 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #include "ring_bond.h"
@@ -49,7 +23,7 @@ ring_bond::ring_bond(int if_index)
     , m_lock_ring_rx("ring_bond:lock_rx")
     , m_lock_ring_tx("ring_bond:lock_tx")
 {
-    net_device_val *p_ndev = NULL;
+    net_device_val *p_ndev = nullptr;
 
     /* Configure ring() fields */
     set_parent(this);
@@ -57,7 +31,7 @@ ring_bond::ring_bond(int if_index)
 
     /* Sanity check */
     p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
-    if (NULL == p_ndev) {
+    if (!p_ndev) {
         ring_logpanic("Invalid if_index = %d", if_index);
     }
 
@@ -89,17 +63,17 @@ ring_bond::~ring_bond()
 
     if (m_p_n_rx_channel_fds) {
         delete[] m_p_n_rx_channel_fds;
-        m_p_n_rx_channel_fds = NULL;
+        m_p_n_rx_channel_fds = nullptr;
     }
 }
 
 void ring_bond::print_val()
 {
     ring_logdbg("%d: %p: parent %p type %s", m_if_index, this,
-                ((uintptr_t)this == (uintptr_t)m_parent ? 0 : m_parent), "bond");
+                ((uintptr_t)this == (uintptr_t)m_parent ? nullptr : m_parent), "bond");
 }
 
-bool ring_bond::attach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink, bool force_5t)
+bool ring_bond::attach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, bool force_5t)
 {
     bool ret = true;
     struct flow_sink_t value = {flow_spec_5t, sink};
@@ -117,7 +91,7 @@ bool ring_bond::attach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink, bool
     return ret;
 }
 
-bool ring_bond::detach_flow(flow_tuple &flow_spec_5t, pkt_rcvr_sink *sink)
+bool ring_bond::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
 {
     bool ret = true;
     struct flow_sink_t value = {flow_spec_5t, sink};
@@ -145,7 +119,7 @@ void ring_bond::restart()
 {
     net_device_val *p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
 
-    if (NULL == p_ndev) {
+    if (!p_ndev) {
         return;
     }
     const slave_data_vector_t &slaves = p_ndev->get_slave_array();
@@ -161,12 +135,11 @@ void ring_bond::restart()
             ring_tap *p_ring_tap = dynamic_cast<ring_tap *>(p_ring_bond_netvsc->m_tap_ring);
             if (p_ring_tap) {
                 size_t num_ring_rx_fds = 0;
-                int *ring_rx_fds_array = NULL;
+                int *ring_rx_fds_array = nullptr;
                 int epfd = -1;
                 int fd = -1;
                 int rc = 0;
                 size_t i, j, k;
-                NOT_IN_USE(rc); // Suppress --enable-opt-log=high warning
 
                 if (slaves.empty()) {
                     ring_rx_fds_array =
@@ -176,7 +149,7 @@ void ring_bond::restart()
                         epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
                         if (epfd > 0) {
                             fd = ring_rx_fds_array[k];
-                            rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_DEL, fd, NULL);
+                            rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, nullptr);
                             ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd, rc,
                                         errno);
                         }
@@ -187,14 +160,14 @@ void ring_bond::restart()
                             epfd = si->get_rx_epfd();
                             if (epfd > 0) {
                                 fd = ring_rx_fds_array[k];
-                                rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_DEL, fd, NULL);
+                                rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, NULL);
                                 ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
                                             rc, errno);
                             }
                             epfd = si->get_epoll_context_fd();
                             if (epfd > 0) {
                                 fd = ring_rx_fds_array[k];
-                                rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_DEL, fd, NULL);
+                                rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, NULL);
                                 ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
                                             rc, errno);
                             }
@@ -205,8 +178,8 @@ void ring_bond::restart()
                     p_ring_tap->inc_vf_plugouts();
                     p_ring_bond_netvsc->slave_destroy(
                         p_ring_bond_netvsc->m_vf_ring->get_if_index());
-                    p_ring_bond_netvsc->m_vf_ring = NULL;
-                    p_ring_tap->set_vf_ring(NULL);
+                    p_ring_bond_netvsc->m_vf_ring = nullptr;
+                    p_ring_tap->set_vf_ring(nullptr);
                 } else {
                     for (i = 0; i < slaves.size(); i++) {
                         if (slaves[i]->if_index != p_ring_tap->get_if_index()) {
@@ -220,11 +193,11 @@ void ring_bond::restart()
                             for (k = 0; k < num_ring_rx_fds; k++) {
                                 epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
                                 if (epfd > 0) {
-                                    epoll_event ev = {0, {0}};
+                                    epoll_event ev = {0, {nullptr}};
                                     fd = ring_rx_fds_array[k];
                                     ev.events = EPOLLIN;
                                     ev.data.fd = fd;
-                                    rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &ev);
+                                    rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
                                     ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
                                                 rc, errno);
                                 }
@@ -240,18 +213,18 @@ void ring_bond::restart()
                                         fd = ring_rx_fds_array[k];
                                         ev.events = EPOLLIN;
                                         ev.data.fd = fd;
-                                        rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &ev);
+                                        rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
                                         ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd,
                                                     epfd, rc, errno);
                                     }
                                     epfd = si->get_epoll_context_fd();
                                     if (epfd > 0) {
-#define CQ_FD_MARK 0xabcd /* see socket_fd_api */
+#define CQ_FD_MARK 0xabcd /* see sockinfo */
                                         epoll_event ev = {0, {0}};
                                         fd = ring_rx_fds_array[k];
                                         ev.events = EPOLLIN | EPOLLPRI;
                                         ev.data.u64 = (((uint64_t)CQ_FD_MARK << 32) | fd);
-                                        rc = orig_os_api.epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &ev);
+                                        rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
                                         ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd,
                                                     epfd, rc, errno);
                                     }
@@ -261,6 +234,7 @@ void ring_bond::restart()
                         }
                     }
                 }
+                NOT_IN_USE(rc); // Suppress --enable-opt-log=high warning
             }
         }
     } else {
@@ -291,13 +265,18 @@ void ring_bond::restart()
                 if (slaves[j]->active) {
                     ring_logdbg("ring %d active", i);
                     if (slaves[j]->lag_tx_port_affinity != 1) {
-                        tmp_ring->start_active_qp_mgr();
+                        tmp_ring->start_active_queue_tx();
+                        /* coverity[sleep] */
+                        tmp_ring->start_active_queue_rx();
                     }
                     m_bond_rings[i]->m_active = true;
                 } else {
                     ring_logdbg("ring %d not active", i);
                     if (slaves[j]->lag_tx_port_affinity != 1) {
-                        tmp_ring->stop_active_qp_mgr();
+                        /* coverity[sleep] */
+                        tmp_ring->stop_active_queue_tx();
+                        /* coverity[sleep] */
+                        tmp_ring->stop_active_queue_rx();
                     }
                     m_bond_rings[i]->m_active = false;
                 }
@@ -307,14 +286,14 @@ void ring_bond::restart()
         popup_xmit_rings();
 
         int ret = 0;
-        uint64_t poll_sn = cq_mgr::m_n_global_sn;
+        uint64_t poll_sn = cq_mgr_rx::m_n_global_sn_rx;
         ret = request_notification(CQT_RX, poll_sn);
         if (ret < 0) {
-            ring_logdbg("failed arming rx cq_mgr (errno=%d %m)", errno);
+            ring_logdbg("failed arming cq_mgr_rx (errno=%d %m)", errno);
         }
         ret = request_notification(CQT_TX, poll_sn);
         if (ret < 0) {
-            ring_logdbg("failed arming tx cq_mgr (errno=%d %m)", errno);
+            ring_logdbg("failed arming cq_mgr_tx (errno=%d %m)", errno);
         }
 
         if (m_type == net_device_val::ACTIVE_BACKUP) {
@@ -362,7 +341,7 @@ void ring_bond::adapt_cq_moderation()
 mem_buf_desc_t *ring_bond::mem_buf_tx_get(ring_user_id_t id, bool b_block, pbuf_type type,
                                           int n_num_mem_bufs /* default = 1 */)
 {
-    mem_buf_desc_t *ret = NULL;
+    mem_buf_desc_t *ret = nullptr;
 
     std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
     ret = m_xmit_rings[id]->mem_buf_tx_get(id, b_block, type, n_num_mem_bufs);
@@ -401,7 +380,7 @@ void ring_bond::mem_buf_rx_release(mem_buf_desc_t *p_mem_buf_desc)
         }
     }
     if (i == m_bond_rings.size()) {
-        buffer_pool::free_rx_lwip_pbuf_custom(&p_mem_buf_desc->lwip_pbuf.pbuf);
+        buffer_pool::free_rx_lwip_pbuf_custom(&p_mem_buf_desc->lwip_pbuf);
     }
 }
 
@@ -427,7 +406,7 @@ void ring_bond::send_ring_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe
     } else {
         ring_logfunc("active ring=%p, silent packet drop (%p), (HA event?)", m_xmit_rings[id],
                      p_mem_buf_desc);
-        p_mem_buf_desc->p_next_desc = NULL;
+        p_mem_buf_desc->p_next_desc = nullptr;
         if (likely(p_mem_buf_desc->p_desc_owner == m_bond_rings[id])) {
             m_bond_rings[id]->mem_buf_tx_release(p_mem_buf_desc, true);
         } else {
@@ -449,7 +428,7 @@ int ring_bond::send_lwip_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe,
 
     ring_logfunc("active ring=%p, silent packet drop (%p), (HA event?)", m_xmit_rings[id],
                  p_mem_buf_desc);
-    p_mem_buf_desc->p_next_desc = NULL;
+    p_mem_buf_desc->p_next_desc = nullptr;
     /* no need to free the buffer here, as for lwip buffers we have 2 ref counts, */
     /* one for caller, and one for completion. for completion, we ref count in    */
     /* send_lwip_buffer(). Since we are not going in, the caller will free the    */
@@ -474,32 +453,26 @@ bool ring_bond::get_hw_dummy_send_support(ring_user_id_t id, xlio_ibv_send_wr *p
     return false;
 }
 
-int ring_bond::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array /*NULL*/)
+bool ring_bond::poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
+                                            void *pv_fd_ready_array /*NULL*/)
 {
     if (m_lock_ring_rx.trylock()) {
-        errno = EAGAIN;
-        return 0;
+        return false;
     }
 
-    int temp = 0;
-    int ret = 0;
+    bool all_drained = true;
 
     for (uint32_t i = 0; i < m_recv_rings.size(); i++) {
         if (m_recv_rings[i]->is_up()) {
             // TODO consider returning immediately after finding something, continue next time from
             // next ring
-            temp = m_recv_rings[i]->poll_and_process_element_rx(p_cq_poll_sn, pv_fd_ready_array);
-            if (temp > 0) {
-                ret += temp;
-            }
+            all_drained &=
+                m_recv_rings[i]->poll_and_process_element_rx(p_cq_poll_sn, pv_fd_ready_array);
         }
     }
     m_lock_ring_rx.unlock();
-    if (ret > 0) {
-        return ret;
-    } else {
-        return temp;
-    }
+
+    return all_drained;
 }
 
 int ring_bond::poll_and_process_element_tx(uint64_t *p_cq_poll_sn)
@@ -555,53 +528,30 @@ int ring_bond::drain_and_proccess()
     }
 }
 
-int ring_bond::wait_for_notification_and_process_element(int cq_channel_fd, uint64_t *p_cq_poll_sn,
-                                                         void *pv_fd_ready_array /*NULL*/)
+void ring_bond::wait_for_notification_and_process_element(uint64_t *p_cq_poll_sn,
+                                                          void *pv_fd_ready_array /*NULL*/)
 {
-    if (m_lock_ring_rx.trylock()) {
-        errno = EAGAIN;
-        return -1;
-    }
-
-    int temp = 0;
-    int ret = 0;
+    m_lock_ring_rx.lock();
 
     for (uint32_t i = 0; i < m_recv_rings.size(); i++) {
         if (m_recv_rings[i]->is_up()) {
-            temp = m_recv_rings[i]->wait_for_notification_and_process_element(
-                cq_channel_fd, p_cq_poll_sn, pv_fd_ready_array);
-            if (temp > 0) {
-                ret += temp;
-            }
+            m_recv_rings[i]->wait_for_notification_and_process_element(p_cq_poll_sn,
+                                                                       pv_fd_ready_array);
         }
     }
+
     m_lock_ring_rx.unlock();
-    if (ret > 0) {
-        return ret;
-    } else {
-        return temp;
-    }
 }
 
 int ring_bond::request_notification(cq_type_t cq_type, uint64_t poll_sn)
 {
-    ring_slave_vector_t *bond_rings;
     int ret = 0;
     int temp;
+    lock_mutex_recursive &ring_lock = (CQT_RX == cq_type ? m_lock_ring_rx : m_lock_ring_tx);
 
-    if (likely(CQT_RX == cq_type)) {
-        if (m_lock_ring_rx.trylock()) {
-            errno = EAGAIN;
-            return 1;
-        }
-    } else {
-        if (m_lock_ring_tx.trylock()) {
-            errno = EAGAIN;
-            return 1;
-        }
-    }
+    ring_lock.lock();
 
-    bond_rings = (cq_type == CQT_RX ? &m_recv_rings : &m_xmit_rings);
+    ring_slave_vector_t *bond_rings = (cq_type == CQT_RX ? &m_recv_rings : &m_xmit_rings);
     for (uint32_t i = 0; i < (*bond_rings).size(); i++) {
         if ((*bond_rings)[i]->is_up()) {
             temp = (*bond_rings)[i]->request_notification(cq_type, poll_sn);
@@ -613,11 +563,9 @@ int ring_bond::request_notification(cq_type_t cq_type, uint64_t poll_sn)
             }
         }
     }
-    if (likely(CQT_RX == cq_type)) {
-        m_lock_ring_rx.unlock();
-    } else {
-        m_lock_ring_tx.unlock();
-    }
+
+    ring_lock.unlock();
+
     return ret;
 }
 
@@ -670,7 +618,7 @@ bool ring_bond::reclaim_recv_buffers(mem_buf_desc_t *)
 
 void ring_bond::update_cap(ring_slave *slave)
 {
-    if (NULL == slave) {
+    if (!slave) {
         m_max_inline_data = (uint32_t)(-1);
         m_max_send_sge = (uint32_t)(-1);
         return;
@@ -743,7 +691,7 @@ int ring_bond::devide_buffers_helper(mem_buf_desc_t *p_mem_buf_desc_list,
             }
         }
         temp = head->p_next_desc;
-        head->p_next_desc = NULL;
+        head->p_next_desc = nullptr;
         if (i == m_bond_rings.size()) {
             // handle no owner
             ring_logdbg("No matching ring %p to return buffer", current->p_desc_owner);
@@ -759,7 +707,7 @@ int ring_bond::devide_buffers_helper(mem_buf_desc_t *p_mem_buf_desc_list,
 
 void ring_bond::popup_xmit_rings()
 {
-    ring_slave *cur_slave = NULL;
+    ring_slave *cur_slave = nullptr;
     size_t i, j;
 
     m_xmit_rings.clear();
@@ -792,7 +740,7 @@ void ring_bond::popup_recv_rings()
     net_device_val *p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
 
     m_recv_rings.clear();
-    if (NULL == p_ndev) {
+    if (!p_ndev) {
         return;
     }
     const slave_data_vector_t &slaves = p_ndev->get_slave_array();
@@ -825,7 +773,7 @@ void ring_bond::update_rx_channel_fds()
 {
     if (m_p_n_rx_channel_fds) {
         delete[] m_p_n_rx_channel_fds;
-        m_p_n_rx_channel_fds = NULL;
+        m_p_n_rx_channel_fds = nullptr;
     }
     if (m_recv_rings.size() == 0) {
         return;
@@ -955,7 +903,7 @@ int ring_bond::socketxtreme_poll(struct xlio_socketxtreme_completion_t *, unsign
 
 void ring_bond::slave_destroy(int if_index)
 {
-    ring_slave *cur_slave = NULL;
+    ring_slave *cur_slave = nullptr;
     ring_slave_vector_t::iterator iter;
 
     for (iter = m_bond_rings.begin(); iter != m_bond_rings.end(); iter++) {
@@ -976,7 +924,7 @@ void ring_bond_eth::slave_create(int if_index)
     ring_slave *cur_slave;
 
     cur_slave = new ring_eth(if_index, this);
-    if (cur_slave == NULL) {
+    if (!cur_slave) {
         ring_logpanic("Error creating bond ring: memory allocation error");
     }
 
@@ -995,11 +943,11 @@ void ring_bond_eth::slave_create(int if_index)
 
 void ring_bond_netvsc::slave_create(int if_index)
 {
-    ring_slave *cur_slave = NULL;
-    net_device_val *p_ndev = NULL;
+    ring_slave *cur_slave = nullptr;
+    net_device_val *p_ndev = nullptr;
 
     p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
-    if (NULL == p_ndev) {
+    if (!p_ndev) {
         ring_logpanic("Error creating bond ring");
     }
 
