@@ -1,35 +1,7 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,6 +24,7 @@
 #include "utils/compiler.h"
 #include "utils/rdtsc.h"
 #include "util/xlio_stats.h"
+#include "util/hugepage_mgr.h"
 #include "util/utils.h"
 #include "event/event_handler_manager.h"
 #include "event/poll_group.h"
@@ -121,6 +94,11 @@ static int free_libxlio_resources()
 
     g_b_exit = true;
 
+    if (safe_mce_sys().print_report) {
+        buffer_pool::print_full_report(VLOG_INFO);
+        g_hugepage_mgr.print_report();
+    }
+
     // Destroy polling groups before fd_collection to clear XLIO sockets from the fd_collection
     poll_group::destroy_all_groups();
 
@@ -197,10 +175,6 @@ static int free_libxlio_resources()
     }
     g_socketxtreme_ec_pool = NULL;
 
-    if (safe_mce_sys().print_report) {
-        buffer_pool::print_full_report(VLOG_INFO);
-    }
-
     if (g_buffer_pool_zc) {
         delete g_buffer_pool_zc;
     }
@@ -272,8 +246,6 @@ static int free_libxlio_resources()
     vlog_stop();
 
     if (g_stats_file) {
-        // cosmetics - remove when adding iomux block
-        fprintf(g_stats_file, "======================================================\n");
         fclose(g_stats_file);
         g_stats_file = nullptr;
     }
@@ -501,6 +473,8 @@ void print_xlio_global_settings()
                       safe_mce_sys().handle_segfault ? "Enabled " : "Disabled");
     VLOG_PARAM_STRING("Print a report", safe_mce_sys().print_report, MCE_DEFAULT_PRINT_REPORT,
                       SYS_VAR_PRINT_REPORT, safe_mce_sys().print_report ? "Enabled " : "Disabled");
+    VLOG_PARAM_STRING("Quick start", safe_mce_sys().quick_start, MCE_DEFAULT_QUICK_START,
+                      SYS_VAR_QUICK_START, safe_mce_sys().quick_start ? "Enabled " : "Disabled");
 
     VLOG_PARAM_NUMSTR("Ring allocation logic TX", safe_mce_sys().ring_allocation_logic_tx,
                       MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX, SYS_VAR_RING_ALLOCATION_LOGIC_TX,
@@ -1186,7 +1160,6 @@ void reset_globals()
     g_p_route_table_mgr = nullptr;
     g_bind_no_port = nullptr;
     g_p_rule_table_mgr = nullptr;
-    g_stats_file = nullptr;
     g_p_net_device_table_mgr = nullptr;
     g_p_neigh_table_mgr = nullptr;
     g_p_lwip = nullptr;
@@ -1286,19 +1259,7 @@ extern "C" int xlio_init(void)
     check_locked_mem();
     check_netperf_flags();
 
-    if (*safe_mce_sys().stats_filename) {
-        if (check_if_regular_file(safe_mce_sys().stats_filename)) {
-            vlog_printf(VLOG_WARNING,
-                        "FAILED to create " PRODUCT_NAME
-                        " statistics file. %s is not a regular file.\n",
-                        safe_mce_sys().stats_filename);
-        } else if (!(g_stats_file = fopen(safe_mce_sys().stats_filename, "w"))) {
-            vlog_printf(VLOG_WARNING, " Couldn't open statistics file: %s\n",
-                        safe_mce_sys().stats_filename);
-        }
-    }
-    safe_mce_sys().stats_file = g_stats_file;
-
+    open_stats_file();
     sock_redirect_main();
 
     return 0;
