@@ -1,35 +1,7 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright (c) 2023-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #include "hugepage_mgr.h"
@@ -96,6 +68,33 @@ bool hugepage_mgr::is_hugepage_acceptable(size_t hugepage, size_t size)
         hugepage_metric(hugepage, size) <= HUGEPAGE_METRIC_ACCEPTABLE;
 }
 
+bool hugepage_mgr::check_resident_pages(void *ptr, size_t size, size_t page_size)
+{
+    const size_t pages_nr = size / page_size;
+    size_t resident_nr = 0;
+    char *page_ptr = reinterpret_cast<char *>(ptr);
+
+    /* Checking a single page per hugepage in a loop is more efficient than a single mincore()
+     * syscall for the entire range. A single syscall would also require an array allocation
+     * which can grow to tens of MB for the preallocated memory region.
+     */
+    for (size_t i = 0; i < pages_nr; ++i) {
+        unsigned char vec;
+        int rc = mincore(page_ptr, 1, &vec);
+        if (rc < 0) {
+            __log_info_dbg("mincore() failed to verify hugepages (errno=%d)", errno);
+            return false;
+        }
+        resident_nr += (vec & 1U);
+        page_ptr += page_size;
+    }
+    if (resident_nr != pages_nr) {
+        __log_info_dbg("Not all hugepages are resident (allocated=%zu resident=%zu)", pages_nr,
+                       resident_nr);
+    }
+    return resident_nr == pages_nr;
+}
+
 void *hugepage_mgr::alloc_hugepages_helper(size_t &size, size_t hugepage)
 {
     size_t hugepage_mask = hugepage - 1;
@@ -113,9 +112,26 @@ void *hugepage_mgr::alloc_hugepages_helper(size_t &size, size_t hugepage)
                MAP_PRIVATE | MAP_ANONYMOUS | MAP_POPULATE | MAP_HUGETLB | map_flags, -1, 0);
     if (ptr == MAP_FAILED) {
         ptr = nullptr;
-        __log_info_dbg("mmap failed (errno=%d), skipping hugepage %zu kB", errno, hugepage / 1024U);
-    } else {
+        __log_info_dbg("mmap failed (errno=%d)", errno);
+    } else if (!safe_mce_sys().quick_start) {
+        /* Check whether all the pages are resident. Allocation beyond the cgroup limit can be
+         * successful and lead to a SIGBUS on an access. For example, the limit can be configured
+         * for a container.
+         */
+        if (!check_resident_pages(ptr, actual_size, hugepage)) {
+            int rc = munmap(ptr, actual_size);
+            if (rc < 0) {
+                __log_info_dbg("munmap failed (errno=%d)", errno);
+            }
+            ptr = nullptr;
+        }
+    }
+    if (ptr) {
+        // Success.
         size = actual_size;
+    } else {
+        // Failure.
+        __log_info_dbg("Skipping hugepage %zu kB", hugepage / 1024U);
     }
     return ptr;
 }
@@ -207,7 +223,7 @@ void hugepage_mgr::print_report(bool short_report /*=false*/)
                     m_hugepages[hugepage].nr_hugepages_allocated,
                     m_hugepages[hugepage].nr_allocations);
     }
-    vlog_printf(VLOG_INFO, "  Total: allocations=%u fails=%u\n", m_stats.allocations,
+    vlog_printf(VLOG_INFO, "  Total: allocations=%u unsatisfied=%u\n", m_stats.allocations,
                 m_stats.fails);
     vlog_printf(VLOG_INFO, "  Total: allocated=%zuMB requested=%zuMB unused_space=%zuMB\n",
                 m_stats.total_allocated / ONE_MB, m_stats.total_requested / ONE_MB,
