@@ -1,35 +1,7 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifdef HAVE_CONFIG_H
@@ -761,6 +733,7 @@ void mce_sys_var::get_env_params()
     service_enable = MCE_DEFAULT_SERVICE_ENABLE;
 
     print_report = MCE_DEFAULT_PRINT_REPORT;
+    quick_start = MCE_DEFAULT_QUICK_START;
     log_level = VLOG_DEFAULT;
     log_details = MCE_DEFAULT_LOG_DETAILS;
     log_colors = MCE_DEFAULT_LOG_COLORS;
@@ -1084,6 +1057,10 @@ void mce_sys_var::get_env_params()
         print_report = atoi(env_ptr) ? true : false;
     }
 
+    if ((env_ptr = getenv(SYS_VAR_QUICK_START))) {
+        quick_start = atoi(env_ptr) ? true : false;
+    }
+
     if ((env_ptr = getenv(SYS_VAR_LOG_FILENAME))) {
         read_env_variable_with_pid(log_filename, sizeof(log_filename), env_ptr);
     }
@@ -1145,7 +1122,7 @@ void mce_sys_var::get_env_params()
         stats_fd_num_max = (uint32_t)atoi(env_ptr);
         stats_fd_num_monitor = std::min(stats_fd_num_max, MAX_STATS_FD_NUM);
         if (stats_fd_num_max > MAX_STATS_FD_NUM) {
-            vlog_printf(VLOG_WARNING, " Can only monitor maximum %d sockets in statistics \n",
+            vlog_printf(VLOG_INFO, "xlio_stats monitoring will be limited by %d sockets\n",
                         MAX_STATS_FD_NUM);
         }
     }
@@ -1185,8 +1162,8 @@ void mce_sys_var::get_env_params()
         tx_max_inline = (uint32_t)atoi(env_ptr);
     }
     if (tx_max_inline > MAX_SUPPORTED_IB_INLINE_SIZE) {
-        vlog_printf(VLOG_WARNING, "%s  must be smaller or equal to %d [%d]\n",
-                    SYS_VAR_TX_MAX_INLINE, MAX_SUPPORTED_IB_INLINE_SIZE, tx_max_inline);
+        vlog_printf(VLOG_WARNING, "%s must be smaller or equal to %d [%d]\n", SYS_VAR_TX_MAX_INLINE,
+                    MAX_SUPPORTED_IB_INLINE_SIZE, tx_max_inline);
         tx_max_inline = MAX_SUPPORTED_IB_INLINE_SIZE;
     }
 
@@ -1288,7 +1265,7 @@ void mce_sys_var::get_env_params()
         rx_poll_num = atoi(env_ptr);
     }
     if (rx_poll_num < MCE_MIN_RX_NUM_POLLS || rx_poll_num > MCE_MAX_RX_NUM_POLLS) {
-        vlog_printf(VLOG_WARNING, " Rx Poll loops should be between %d and %d [%d]\n",
+        vlog_printf(VLOG_WARNING, "Rx Poll loops should be between %d and %d [%d]\n",
                     MCE_MIN_RX_NUM_POLLS, MCE_MAX_RX_NUM_POLLS, rx_poll_num);
         rx_poll_num = MCE_DEFAULT_RX_NUM_POLLS;
     }
@@ -1296,7 +1273,7 @@ void mce_sys_var::get_env_params()
         rx_poll_num_init = atoi(env_ptr);
     }
     if (rx_poll_num_init < MCE_MIN_RX_NUM_POLLS || rx_poll_num_init > MCE_MAX_RX_NUM_POLLS) {
-        vlog_printf(VLOG_WARNING, " Rx Poll loops should be between %d and %d [%d]\n",
+        vlog_printf(VLOG_WARNING, "Rx Poll loops should be between %d and %d [%d]\n",
                     MCE_MIN_RX_NUM_POLLS, MCE_MAX_RX_NUM_POLLS, rx_poll_num_init);
         rx_poll_num_init = MCE_DEFAULT_RX_NUM_POLLS_INIT;
     }
@@ -1337,7 +1314,7 @@ void mce_sys_var::get_env_params()
     }
     if (rx_prefetch_bytes < MCE_MIN_RX_PREFETCH_BYTES ||
         rx_prefetch_bytes > MCE_MAX_RX_PREFETCH_BYTES) {
-        vlog_printf(VLOG_WARNING, " Rx prefetch bytes size out of range [%d] (min=%d, max=%d)\n",
+        vlog_printf(VLOG_WARNING, "Rx prefetch bytes size out of range [%d] (min=%d, max=%d)\n",
                     rx_prefetch_bytes, MCE_MIN_RX_PREFETCH_BYTES, MCE_MAX_RX_PREFETCH_BYTES);
         rx_prefetch_bytes = MCE_DEFAULT_RX_PREFETCH_BYTES;
     }
@@ -1348,10 +1325,9 @@ void mce_sys_var::get_env_params()
     if (rx_prefetch_bytes_before_poll != 0 &&
         (rx_prefetch_bytes_before_poll < MCE_MIN_RX_PREFETCH_BYTES ||
          rx_prefetch_bytes_before_poll > MCE_MAX_RX_PREFETCH_BYTES)) {
-        vlog_printf(VLOG_WARNING,
-                    " Rx prefetch bytes size out of range [%d] (min=%d, max=%d, disabled=0)\n",
-                    rx_prefetch_bytes_before_poll, MCE_MIN_RX_PREFETCH_BYTES,
-                    MCE_MAX_RX_PREFETCH_BYTES);
+        vlog_printf(
+            VLOG_WARNING, "Rx prefetch bytes size out of range [%d] (min=%d, max=%d, disabled=0)\n",
+            rx_prefetch_bytes_before_poll, MCE_MIN_RX_PREFETCH_BYTES, MCE_MAX_RX_PREFETCH_BYTES);
         rx_prefetch_bytes_before_poll = MCE_DEFAULT_RX_PREFETCH_BYTES_BEFORE_POLL;
     }
 
@@ -1400,7 +1376,7 @@ void mce_sys_var::get_env_params()
     }
 
     if (select_poll_num < MCE_MIN_RX_NUM_POLLS || select_poll_num > MCE_MAX_RX_NUM_POLLS) {
-        vlog_printf(VLOG_WARNING, " Select Poll loops can not be below zero [%d]\n",
+        vlog_printf(VLOG_WARNING, "Select Poll loops can not be below zero [%d]\n",
                     select_poll_num);
         select_poll_num = MCE_DEFAULT_SELECT_NUM_POLLS;
     }
@@ -1503,7 +1479,7 @@ void mce_sys_var::get_env_params()
         cq_poll_batch_max = (uint32_t)atoi(env_ptr);
     }
     if (cq_poll_batch_max < MCE_MIN_CQ_POLL_BATCH || cq_poll_batch_max > MCE_MAX_CQ_POLL_BATCH) {
-        vlog_printf(VLOG_WARNING, " Rx number of cq poll batchs should be between %d and %d [%d]\n",
+        vlog_printf(VLOG_WARNING, "Rx number of cq poll batchs should be between %d and %d [%d]\n",
                     MCE_MIN_CQ_POLL_BATCH, MCE_MAX_CQ_POLL_BATCH, cq_poll_batch_max);
         cq_poll_batch_max = MCE_DEFAULT_CQ_POLL_BATCH;
     }
@@ -1617,7 +1593,7 @@ void mce_sys_var::get_env_params()
 
     if (tcp_timer_resolution_msec < timer_resolution_msec) {
         vlog_printf(VLOG_WARNING,
-                    " TCP timer resolution [%s=%d] cannot be smaller than timer resolution "
+                    "TCP timer resolution [%s=%d] cannot be smaller than timer resolution "
                     "[%s=%d]. Setting TCP timer resolution to %d msec.\n",
                     SYS_VAR_TCP_TIMER_RESOLUTION_MSEC, tcp_timer_resolution_msec,
                     SYS_VAR_TIMER_RESOLUTION_MSEC, timer_resolution_msec, timer_resolution_msec);
@@ -1642,7 +1618,7 @@ void mce_sys_var::get_env_params()
     }
     if (env_to_cpuset(internal_thread_affinity_str, &internal_thread_affinity)) {
         vlog_printf(VLOG_WARNING,
-                    " Failed to set internal thread affinity: %s...  deferring to cpu-0.\n",
+                    "Failed to set internal thread affinity: %s...  deferring to cpu-0.\n",
                     internal_thread_affinity_str);
     }
 
