@@ -1,35 +1,7 @@
 /*
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- * SPDX-License-Identifier: GPL-2.0-only or BSD-3-Clause
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifndef SYS_VARS_H
@@ -340,6 +312,7 @@ public:
     uint32_t mce_spec;
 
     bool print_report;
+    bool quick_start;
     vlog_levels_t log_level;
     uint32_t log_details;
     char log_filename[PATH_MAX];
@@ -502,7 +475,6 @@ public:
     uint32_t tcp_send_buffer_size;
     uint32_t tx_segs_ring_batch_tcp;
     uint32_t tx_segs_pool_batch_tcp;
-    FILE *stats_file;
     /* This field should be used to store and use data for XLIO_EXTRA_API_IOCTL */
     struct {
         struct {
@@ -528,7 +500,6 @@ private:
     // prevent unautothrized creation of objects
     mce_sys_var()
         : sysctl_reader(sysctl_reader_t::instance())
-        , stats_file(nullptr)
         , m_ioctl {}
     {
         // coverity[uninit_member]
@@ -558,6 +529,7 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_HANDLE_SIGINTR      "XLIO_HANDLE_SIGINTR"
 #define SYS_VAR_HANDLE_SIGSEGV      "XLIO_HANDLE_SIGSEGV"
 #define SYS_VAR_STATS_FD_NUM        "XLIO_STATS_FD_NUM"
+#define SYS_VAR_QUICK_START         "XLIO_QUICK_START"
 
 #define SYS_VAR_RING_ALLOCATION_LOGIC_TX "XLIO_RING_ALLOCATION_LOGIC_TX"
 #define SYS_VAR_RING_ALLOCATION_LOGIC_RX "XLIO_RING_ALLOCATION_LOGIC_RX"
@@ -710,7 +682,8 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_APP_ID                   ("XLIO_DEFAULT_APPLICATION_ID")
 #define MCE_DEFAULT_HANDLE_SIGINTR           (true)
 #define MCE_DEFAULT_HANDLE_SIGFAULT          (false)
-#define MCE_DEFAULT_STATS_FD_NUM             0
+#define MCE_DEFAULT_STATS_FD_NUM             (0)
+#define MCE_DEFAULT_QUICK_START              (false)
 #define MCE_DEFAULT_RING_ALLOCATION_LOGIC_TX (RING_LOGIC_PER_THREAD)
 #define MCE_DEFAULT_RING_ALLOCATION_LOGIC_RX (RING_LOGIC_PER_THREAD)
 #define MCE_DEFAULT_RING_MIGRATION_RATIO_TX  (-1)
@@ -777,7 +750,7 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_CQ_AIM_MAX_COUNT               (500)
 #define MCE_DEFAULT_CQ_AIM_MAX_PERIOD_USEC         (1000)
 #define MCE_DEFAULT_CQ_AIM_INTERVAL_MSEC           (1000)
-#define MCE_DEFAULT_CQ_AIM_INTERRUPTS_RATE_PER_SEC (1000)
+#define MCE_DEFAULT_CQ_AIM_INTERRUPTS_RATE_PER_SEC (10000)
 #define MCE_DEFAULT_CQ_POLL_BATCH                  (16)
 #define MCE_DEFAULT_PROGRESS_ENGINE_INTERVAL_MSEC  (10)
 #define MCE_DEFAULT_PROGRESS_ENGINE_WCE_MAX        (10000)
