@@ -1,33 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifndef XLIO_STATS_H
@@ -225,44 +199,47 @@ typedef struct socket_listen_counters {
     }
 } socket_listen_counters_t;
 
-typedef struct socket_stats_t {
-    int fd;
-    uint32_t inode;
-    uint32_t tcp_state; // enum tcp_state
-    uint8_t socket_type; // SOCK_STREAM, SOCK_DGRAM, ...
-    bool padding1;
-    sa_family_t sa_family;
-    bool b_is_offloaded;
-    bool b_blocking;
-    bool b_mc_loop;
-    bool padding2;
-    in_port_t bound_port;
-    in_port_t connected_port;
-    ip_address bound_if;
-    ip_address connected_ip;
-    ip_address mc_tx_if;
-    pid_t threadid_last_rx;
-    pid_t threadid_last_tx;
-    uint32_t n_rx_ready_pkt_count;
-    uint32_t n_rx_ready_byte_limit;
-    uint64_t n_rx_ready_byte_count;
+struct socket_stats_t {
+    // Data Path
     uint64_t n_tx_ready_byte_count;
-    uint32_t n_rx_zcopy_pkt_count;
+    uint64_t n_rx_ready_byte_count;
+    uint32_t n_rx_ready_pkt_count;
     socket_counters_t counters;
+    socket_strq_counters_t strq_counters;
 #ifdef DEFINED_UTLS
-    bool tls_tx_offload;
-    bool tls_rx_offload;
-    uint16_t tls_version;
-    uint16_t tls_cipher;
     socket_tls_counters_t tls_counters;
 #endif /* DEFINED_UTLS */
-    socket_strq_counters_t strq_counters;
     socket_listen_counters_t listen_counters;
+
+    // Control Path
     std::bitset<MC_TABLE_SIZE> mc_grp_map;
     ring_logic_t ring_alloc_logic_rx;
     ring_logic_t ring_alloc_logic_tx;
+    ip_address bound_if;
+    ip_address connected_ip;
+    ip_address mc_tx_if;
+    int fd;
+    uint32_t inode;
+    uint32_t tcp_state; // enum tcp_state
+    uint32_t n_rx_zcopy_pkt_count;
+    pid_t threadid_last_rx;
+    pid_t threadid_last_tx;
     uint64_t ring_user_id_rx;
     uint64_t ring_user_id_tx;
+    sa_family_t sa_family;
+    in_port_t bound_port;
+    in_port_t connected_port;
+    uint8_t socket_type; // SOCK_STREAM, SOCK_DGRAM, ...
+    bool b_is_offloaded;
+    bool b_blocking;
+    bool b_mc_loop;
+#ifdef DEFINED_UTLS
+    uint16_t tls_version;
+    uint16_t tls_cipher;
+    bool tls_tx_offload;
+    bool tls_rx_offload;
+#endif /* DEFINED_UTLS */
+    socket_stats_t *_next_stat;
 
     void reset()
     {
@@ -274,8 +251,8 @@ typedef struct socket_stats_t {
         bound_if = connected_ip = mc_tx_if = ip_address(in6addr_any);
         bound_port = connected_port = (in_port_t)0;
         threadid_last_rx = threadid_last_tx = pid_t(0);
-        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_rx_ready_byte_limit =
-            n_rx_zcopy_pkt_count = n_tx_ready_byte_count = 0;
+        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_rx_zcopy_pkt_count =
+            n_tx_ready_byte_count = 0;
         memset(&counters, 0, sizeof(counters));
 #ifdef DEFINED_UTLS
         tls_tx_offload = tls_rx_offload = false;
@@ -287,7 +264,6 @@ typedef struct socket_stats_t {
         mc_grp_map.reset();
         ring_user_id_rx = ring_user_id_tx = 0;
         ring_alloc_logic_rx = ring_alloc_logic_tx = RING_LOGIC_PER_INTERFACE;
-        padding1 = padding2 = 0;
     };
 
     void set_bound_if(sock_addr &sock)
@@ -312,10 +288,11 @@ typedef struct socket_stats_t {
         : bound_if(in6addr_any)
         , connected_ip(in6addr_any)
         , mc_tx_if(in6addr_any)
+        , _next_stat(nullptr)
     {
         reset();
     };
-} socket_stats_t;
+};
 
 typedef struct {
     bool b_enabled;
@@ -370,6 +347,8 @@ typedef struct {
     ring_type_t n_type;
     union {
         struct {
+            uint64_t n_tx_tso_pkt_count;
+            uint64_t n_tx_tso_byte_count;
             uint64_t n_rx_interrupt_requests;
             uint64_t n_rx_interrupt_received;
             uint32_t n_rx_cq_moderation_count;
@@ -379,6 +358,8 @@ typedef struct {
             uint64_t n_tx_dev_mem_byte_count;
             uint64_t n_tx_dev_mem_oob;
             uint32_t n_tx_dev_mem_allocated;
+            uint32_t n_tx_num_bufs;
+            uint32_t n_zc_num_bufs;
         } simple;
         struct {
             char s_tap_name[IFNAMSIZ];
@@ -401,6 +382,7 @@ typedef struct {
     uint32_t n_buffer_pool_size;
     uint32_t n_buffer_pool_no_bufs;
     uint32_t n_buffer_pool_expands;
+    uint32_t n_buffer_pool_created;
 } bpool_stats_t;
 
 typedef struct {
@@ -502,7 +484,9 @@ typedef struct sh_mem_t {
         global_inst_arr->init();
         mc_info.max_grp_num = 0;
         for (uint32_t i = 0; i < MC_TABLE_SIZE; i++) {
+            // coverity[missing_lock:FALSE] /*Turn off coverity missing_lock check*/
             mc_info.mc_grp_tbl[i].mc_grp = {ip_address::any_addr(), 0};
+            // coverity[missing_lock:FALSE]
             mc_info.mc_grp_tbl[i].sock_num = 0;
         }
         memset(&iomux, 0, sizeof(iomux));
