@@ -385,7 +385,12 @@ enum ctx_device {
 	ERDMA			= 29,
 	HNS			= 30,
 	CONNECTX8		= 31,
-  INTEL_GEN2		= 32,
+	INTEL_GEN2		= 32,
+	CONNECTX9		= 33,
+	TCU1		= 34,
+	YUNSILICON_ANDES	= 35,
+	YUNSILICON_DIAMOND	= 36,
+	YUNSILICON_DIAMOND_NEXT	= 37,
 };
 
 /* Units for rate limiter */
@@ -442,7 +447,24 @@ enum memory_type {
 	MEMORY_ROCM,
 	MEMORY_NEURON,
 	MEMORY_HL,
-	MEMORY_MLU
+	MEMORY_MLU,
+	MEMORY_OPENCL
+};
+
+enum cuda_mem_type {
+	CUDA_MEM_DEVICE = 0,
+	CUDA_MEM_MANAGED,
+	CUDA_MEM_HOSTALLOC,
+	CUDA_MEM_HOSTREGISTER,
+	CUDA_MEM_MALLOC,
+	CUDA_MEM_TYPES
+};
+
+enum gpu_touch_type {
+	GPU_NO_TOUCH,
+	GPU_TOUCH_ONCE,
+	GPU_TOUCH_INFINITE,
+	GPU_TOUCH_TYPES
 };
 
 struct perftest_parameters {
@@ -571,12 +593,20 @@ struct perftest_parameters {
 	struct memory_ctx		*(*memory_create)(struct perftest_parameters *params);
 	int				cuda_device_id;
 	char				*cuda_device_bus_id;
+	int				cuda_mem_type;
 	int				use_cuda_dmabuf;
+	int				use_cuda_pcie_mapping;
+	int				use_data_direct;
 	int				rocm_device_id;
+	int				use_rocm_dmabuf;
 	int				neuron_core_id;
 	int				use_neuron_dmabuf;
 	char				*hl_device_bus_id;
 	int				mlu_device_id;
+	int				use_mlu_dmabuf;
+	int                             opencl_platform_id;
+	int                             opencl_device_id;
+	int                             gpu_touch;
 	char				*mmap_file;
 	unsigned long			mmap_offset;
 	/* New test params format pilot. will be used in all flags soon,. */
@@ -604,7 +634,7 @@ struct perftest_parameters {
 	char				*out_json_file_name;
 	struct cpu_util_data 		cpu_util_data;
 	int 				latency_gap;
-	int 				flow_label;
+	int*  				flow_label;
 	int 				retry_count;
 	int 				dont_xchg_versions;
 	int 				ipv6;
@@ -640,6 +670,9 @@ struct perftest_parameters {
 	int 			ah_allocated;
 	int				use_write_with_imm;
 	int				use_unsolicited_write;
+	int				use_ddp;
+	int				no_ddp;
+	int				connectionless;
 	uint16_t			cqe_poll;
 	int				use_cqe_poll;
 };
