@@ -183,8 +183,8 @@ struct pingpong_context {
 	struct ibv_pd				*pd;
 	#ifdef HAVE_TD_API
 	struct ibv_td				*td;
-	struct ibv_pd				*pad;
 	#endif
+	struct ibv_pd				*pad;
 	struct ibv_mr				**mr;
 	struct ibv_mr				*null_mr;
 	struct ibv_cq				*send_cq;
@@ -216,7 +216,6 @@ struct pingpong_context {
 	int					tx_depth;
 	uint64_t				*scnt;
 	uint64_t				*ccnt;
-	int					is_contig_supported;
 	uint32_t				*r_dctn;
 	uint32_t				*dci_stream_id;
 	int 					dek_number;
@@ -1022,4 +1021,6 @@ int rdma_cm_destroy_cma(struct pingpong_context *ctx,
 */
 int error_handler(char *error_message);
 
+void check_bf_support(struct pingpong_context *ctx);
+
 #endif /* PERFTEST_RESOURCES_H */
