@@ -33,6 +33,9 @@
 /* RShim timer interval in milliseconds. */
 #define RSHIM_TIMER_INTERVAL 1
 
+/* Network polling interval in timer ticks when idle (no network activity) */
+#define RSHIM_NET_POLL_IDLE_INTERVAL_TICKS 10
+
 /* Intervals to check the locked mode. */
 #define RSHIM_CHECK_LOCKED_MODE_MS      100
 #define RSHIM_CHECK_LOCKED_MODE_TICKS   (RSHIM_CHECK_LOCKED_MODE_MS / RSHIM_TIMER_INTERVAL)
@@ -64,6 +67,7 @@ static int rshim_keepalive_ticks = RSHIM_KEEPALIVE_PERIOD / RSHIM_TIMER_INTERVAL
 #define RSHIM_OSP_REQ_MAGIC_NUM 0x4F53505F524551ULL /* OSP_REQ */
 #define RSHIM_OSP_ACK_MAGIC_NUM 0x4F53505F41434BULL /* OSP_ACK */
 
+
 const char *magic_to_str(uint64_t magic)
 {
   switch(magic){
@@ -220,7 +224,7 @@ char *rshim_static_dev_name;
 /* Default configuration file. */
 const char *rshim_cfg_file = DEFAULT_RSHIM_CONFIG_FILE;
 static int rshim_display_level;
-int rshim_boot_timeout = 300;
+static int rshim_boot_timeout = 300;
 int rshim_drop_mode = -1;
 int rshim_usb_reset_delay = 1;
 bool rshim_has_usb_reset_delay;
@@ -1754,6 +1758,7 @@ ssize_t rshim_fifo_write(rshim_backend_t *bd, const char *buffer,
 
 static void rshim_work_handler(rshim_backend_t *bd)
 {
+  rsh_scratchpad3_t sp3 = { .word = 0 };
   int rc;
 
   pthread_mutex_lock(&bd->mutex);
@@ -1762,8 +1767,12 @@ static void rshim_work_handler(rshim_backend_t *bd)
 
   if (bd->keepalive && bd->has_rshim && !bd->debug_code && !bd->drop_mode &&
       !bd->in_access_check) {
-    bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad1,
-                    RSHIM_KEEPALIVE_MAGIC_NUM, RSHIM_REG_SIZE_8B);
+    bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad3,
+                   &sp3.word, RSHIM_REG_SIZE_8B);
+    if (sp3.dbg_cmd == RSH_DBG_CMD_NONE) {
+      bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad1,
+                      RSHIM_KEEPALIVE_MAGIC_NUM, RSHIM_REG_SIZE_8B);
+    }
     bd->keepalive = 0;
   }
 
@@ -2445,7 +2454,7 @@ static void rshim_timer_func(rshim_backend_t *bd) {
 static void rshim_timer_run(void)
 {
   rshim_backend_t *bd;
-  int i;
+  int i, tx_activity, rx_activity;
 
   rshim_timer_ticks++;
 
@@ -2469,8 +2478,16 @@ static void rshim_timer_run(void)
 
       /* Push out remaining data if not sent out in the epoll loop. */
       if (bd->net_fd >= 0) {
-        rshim_net_tx(bd);
-        rshim_net_rx(bd);
+        if (rshim_timer_ticks % bd->net_poll_interval_ticks == 0) {
+          tx_activity = rshim_net_tx(bd);
+          rx_activity = rshim_net_rx(bd);
+
+          /* Adjust polling interval based on activity */
+          if (tx_activity || rx_activity)
+            bd->net_poll_interval_ticks = 1;
+          else
+            bd->net_poll_interval_ticks = RSHIM_NET_POLL_IDLE_INTERVAL_TICKS;
+        }
       }
     }
   }
@@ -2661,6 +2678,8 @@ int rshim_register(rshim_backend_t *bd)
 
   pthread_mutex_init(&bd->ringlock, NULL);
 
+  bd->net_poll_interval_ticks = 1;
+
   for (i = 0; i < TMFIFO_MAX_CHAN; i++) {
     pthread_cond_init(&bd->read_fifo[i].operable, NULL);
     pthread_cond_init(&bd->write_fifo[i].operable, NULL);
@@ -2840,58 +2859,34 @@ static void rshim_set_timer(int timer_fd, int interval)
   timerfd_settime(timer_fd, 0, &ts, NULL);
 }
 
-static void rshim_main(int argc, char *argv[])
+int rshim_init(int *epollfd, int *timerfd)
 {
-  int i, fd, num, rc, epoll_fd, timer_fd;
-  bool rshim_pcie_lf_init_done = false;
-  uint8_t index;
-#ifdef __FreeBSD__
-  const int MAXEVENTS = 16;
-#else
-  const int MAXEVENTS = 64;
-#endif
-  struct epoll_event events[MAXEVENTS];
   struct epoll_event event;
-  rshim_backend_t *bd;
-  time_t t0, t1;
-  uint8_t tmp;
+  int rc, epoll_fd, timer_fd;
 
   sem_init(&rshim_stop_sem, 0, 0);
 
   memset(&event, 0, sizeof(event));
-  memset(events, 0, sizeof(events));
-
-#ifdef HAVE_RSHIM_FUSE
-#ifdef __linux__
-  rc = system("modprobe cuse");
-  if (rc == -1)
-    RSHIM_DBG("Failed the load cuse: %m\n");
-#endif
-
-#ifdef __FreeBSD__
-  if (feature_present("cuse") == 0)
-    if (system("kldload cuse") == -1)
-      RSHIM_DBG("Failed the load cuse\n");
-#endif
-#endif
 
   /* Create the epoll fd */
   epoll_fd = epoll_create1(EPOLL_CLOEXEC);
   if (epoll_fd == -1) {
-    RSHIM_ERR("epoll_create1 failed: %m\n");
+    RSHIM_ERR("epoll_create1 failed\n");
     exit(-1);
   }
   rshim_epoll_fd = epoll_fd;
 
   /* Create and add work fd. */
   if (pipe(rshim_work_fd) == -1) {
-    RSHIM_ERR("Failed to create pipe");
+    RSHIM_ERR("Failed to create pipe\n");
     exit(-1);
   }
+
   if (fcntl(rshim_work_fd[0], F_SETFL, O_NONBLOCK) < 0) {
-    RSHIM_ERR("Failed to set nonblock pipe");
+    RSHIM_ERR("Failed to set nonblock pipe\n");
     exit(-1);
   }
+
   event.data.fd = rshim_work_fd[0];
   event.events = EPOLLIN;
   rc = epoll_ctl(epoll_fd, EPOLL_CTL_ADD, rshim_work_fd[0], &event);
@@ -2903,18 +2898,64 @@ static void rshim_main(int argc, char *argv[])
   /* Add periodic timer. */
   timer_fd = timerfd_create(CLOCK_MONOTONIC, 0);
   if (timer_fd == -1) {
-    fprintf(stderr, "timerfd_create failed: %m\n");
+    RSHIM_ERR("timerfd_create failed\n");
     exit(1);
   }
   rshim_set_timer(timer_fd, RSHIM_TIMER_INTERVAL);
   event.data.fd = timer_fd;
-  event.events = EPOLLIN | EPOLLOUT;
+  event.events = EPOLLIN;
   rc = epoll_ctl(epoll_fd, EPOLL_CTL_ADD, timer_fd, &event);
   if (rc == -1) {
-    fprintf(stderr, "epoll_ctl failed: %m\n");
+    RSHIM_ERR("epoll_ctl failed\n");
     exit(1);
   }
 
+  if (epollfd)
+    *epollfd = epoll_fd;
+
+  if (timerfd)
+    *timerfd = timer_fd;
+
+  return 0;
+}
+
+static void rshim_main(int argc, char *argv[])
+{
+  int i, fd, num, rc, epoll_fd = -1, timer_fd = -1;
+  bool rshim_pcie_lf_init_done = false;
+  uint8_t index;
+#ifdef __FreeBSD__
+  const int MAXEVENTS = 16;
+#else
+  const int MAXEVENTS = 64;
+#endif
+  struct epoll_event events[MAXEVENTS];
+  rshim_backend_t *bd;
+  time_t t0, t1;
+  uint8_t tmp;
+
+  memset(events, 0, sizeof(events));
+
+#ifdef HAVE_RSHIM_FUSE
+#ifdef __linux__
+  rc = system("modprobe cuse");
+  if (rc == -1)
+    RSHIM_DBG("Failed the load cuse\n");
+#endif
+
+#ifdef __FreeBSD__
+  if (feature_present("cuse") == 0)
+    if (system("kldload cuse") == -1)
+      RSHIM_DBG("Failed the load cuse\n");
+#endif
+#endif
+
+  rc = rshim_init(&epoll_fd, &timer_fd);
+  if (rc) {
+    RSHIM_ERR("rshim_init failed\n");
+    exit(-1);
+  }
+
   /* Scan rshim backends. */
   rc = 0;
   if (!rshim_backend_name && rshim_static_dev_name) {
@@ -2926,15 +2967,14 @@ static void rshim_main(int argc, char *argv[])
       rshim_backend_name = "pcie";
   }
   if (!rshim_backend_name) {
-    if (!rshim_cmdmode)
-      rshim_pcie_init();
+    rshim_pcie_init();
     rshim_usb_init(epoll_fd);
   } else {
     if (!strcmp(rshim_backend_name, "usb"))
       rc = rshim_usb_init(epoll_fd);
-    else if (!strcmp(rshim_backend_name, "pcie-lf") && !rshim_cmdmode)
+    else if (!strcmp(rshim_backend_name, "pcie-lf"))
       rc = rshim_pcie_lf_init();
-    else if (!strcmp(rshim_backend_name, "pcie") && !rshim_cmdmode)
+    else if (!strcmp(rshim_backend_name, "pcie"))
       rc = rshim_pcie_init();
   }
   if (rc) {
@@ -2942,12 +2982,6 @@ static void rshim_main(int argc, char *argv[])
     exit(-1);
   }
 
-  /* Run command mode if specified. */
-  if (rshim_cmdmode) {
-    rshim_cmdmode_run(argc, argv);
-    return;
-  }
-
   time(&t0);
 
   while (rshim_run) {
@@ -3004,7 +3038,7 @@ static void rshim_main(int argc, char *argv[])
     }
 
     /* Delayed initialization for livefish probe. */
-    if (!rshim_pcie_lf_init_done && !rshim_cmdmode) {
+    if (!rshim_pcie_lf_init_done) {
       time(&t1);
       if (difftime(t1, t0) > 3) {
         if (!rshim_backend_name)
@@ -3301,13 +3335,12 @@ static void print_help(void)
   printf("\n");
   printf("OPTIONS:\n");
   printf("  -b, --backend             backend name (usb, pcie or pcie-lf)\n");
-  printf("  -c, --cmdmode             run in command line mode\n");
-  printf("    -g, --get-debug         get debug mode\n");
-  printf("    -p, --get-config <NAME | all> get config value\n");
-  printf("    -s, --set-debug <0 | 1> set debug mode\n");
+  printf("  -c, --cmdmode             command line mode");
+  printf(" ('-c --help' for sub-commands)\n");
   printf("  -d, --device              device to attach\n");
   printf("  -f, --foreground          run in foreground\n");
   printf("  -F, --force               run in force mode\n");
+  printf("  -h, --help                show help info\n");
   printf("  -i, --index               use device path /dev/rshim<i>/\n");
   printf("  -l, --log-level           log level");
   printf("(0:none, 1:error, 2:warning, 3:notice, 4:debug)\n");
@@ -3317,20 +3350,17 @@ static void print_help(void)
 
 int main(int argc, char *argv[])
 {
-  static const char short_options[] = "b:cd:fgFhi:l:np:sv";
+  static const char short_options[] = "b:cd:fFhi:l:nv";
   static struct option long_options[] = {
     { "backend", required_argument, NULL, 'b' },
     { "cmdmode", no_argument, NULL, 'c' },
     { "device", required_argument, NULL, 'd' },
     { "foreground", no_argument, NULL, 'f' },
     { "force", no_argument, NULL, 'F' },
-    { "get-debug", no_argument, NULL, 'g' },
-    { "get-config", required_argument, NULL, 'p' },
     { "help", no_argument, NULL, 'h' },
     { "index", required_argument, NULL, 'i' },
     { "log-level", required_argument, NULL, 'l' },
     { "nonet", no_argument, NULL, 'n' },
-    { "set-debug", required_argument, NULL, 's' },
     { "version", no_argument, NULL, 'v' },
     { NULL, 0, NULL, 0 }
   };
@@ -3338,6 +3368,7 @@ int main(int argc, char *argv[])
   pthread_t thread;
 
   /* Parse arguments. */
+  opterr = 0;
   while ((c = getopt_long(argc, argv, short_options, long_options, NULL))
          != -1) {
     switch (c) {
@@ -3387,17 +3418,33 @@ int main(int argc, char *argv[])
       printf("Rshim Driver for BlueField SoC 2.0\n");
 #endif
       return 0;
+    case '?':
+      if (!rshim_cmdmode) {
+        if (optopt) {
+          fprintf(stderr, "Invalid option: -%c\n", optopt);
+        } else {
+          fprintf(stderr, "Invalid option\n");
+        }
+        return -EINVAL;
+      }
+      break;
     case 'h':
     default:
       if (!rshim_cmdmode) {
         print_help();
         return 0;
       }
+      break;
     }
   }
 
+  /* Handle command mode separately. */
+  if (rshim_cmdmode) {
+    return rshim_cmdmode_run(argc, argv);
+  }
+
   /* Put into daemon mode. */
-  if (rshim_daemon_mode && !rshim_cmdmode) {
+  if (rshim_daemon_mode) {
     int pid = fork();
 
     if (pid < 0) {
@@ -3429,8 +3476,7 @@ int main(int argc, char *argv[])
    * 1. Command mode
    * 2. Direct device attachment ('-d')
    */
-  if (!rshim_cmdmode && !rshim_static_dev_name) {
-    // Single instance otherwise
+  if (!rshim_static_dev_name) {
     pid_fd = create_pid_file(PID_FILE);
     if (pid_fd < 0) {
       exit(EXIT_FAILURE);
@@ -3457,9 +3503,7 @@ int main(int argc, char *argv[])
 
   rshim_main(argc, argv);
 
-  if (!rshim_cmdmode) {
-    cleanup_pid_file(PID_FILE, pid_fd);
-  }
+  cleanup_pid_file(PID_FILE, pid_fd);
 
   closelog();
 
