@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/mount.h>
 #include <termios.h>
 #include <unistd.h>
 #ifdef HAVE_CONFIG_H
@@ -41,7 +42,6 @@ extern int rshim_log_level;
 extern bool rshim_daemon_mode;
 extern int rshim_drop_mode;
 extern bool rshim_force_mode;
-extern int rshim_boot_timeout;
 extern int rshim_usb_timeout;
 extern int rshim_usb_reset_delay;
 extern bool rshim_has_usb_reset_delay;
@@ -257,6 +257,9 @@ struct rshim_backend {
   /* Device name. */
   char dev_name[RSHIM_DEV_NAME_LEN];
 
+  /* Network polling interval in timer ticks */
+  int net_poll_interval_ticks;
+
   /* Backend device. */
   void *dev;
 
@@ -464,6 +467,9 @@ struct rshim_backend {
   /* API to enable the device. */
   int (*enable_device)(rshim_backend_t *bd, bool enable);
 
+  /* API to rescan PCIe for potential rshim device name change */
+  int (*pcie_refresh_dev)(struct rshim_backend *bd);
+
   /* Platform specific register addresses */
   const struct rshim_regs *regs;
 };
@@ -479,6 +485,7 @@ struct rshim_regs {
   uint32_t reset_control;
   uint32_t scratchpad1;
   uint32_t scratchpad2;
+  uint32_t scratchpad3;
   uint32_t scratchpad6;
   uint32_t tm_htt_sts;
   uint32_t tm_tth_sts;
@@ -498,6 +505,27 @@ struct rshim_regs {
   uint32_t scratch_buf_ctl;
 };
 
+/* Ioctl() message header. */
+typedef struct {
+  uint32_t addr;
+  uint64_t data;
+} __attribute__((packed)) rshim_ioctl_msg;
+
+/* Ioctl() message header with size. */
+typedef struct {
+  uint32_t addr;
+  uint64_t data;
+  uint8_t data_size;
+} __attribute__((packed)) rshim_ioctl_msg2;
+
+/* Ioctl() code. */
+enum {
+  RSHIM_IOC_READ = _IOWR('R', 0, rshim_ioctl_msg),
+  RSHIM_IOC_WRITE = _IOWR('R', 1, rshim_ioctl_msg),
+  RSHIM_IOC_READ2 = _IOWR('R', 0, rshim_ioctl_msg2),
+  RSHIM_IOC_WRITE2 = _IOWR('R', 1, rshim_ioctl_msg2),
+};
+
 extern const struct rshim_regs bf1_bf2_rshim_regs;
 extern const struct rshim_regs bf3_rshim_regs;
 
@@ -553,8 +581,8 @@ int rshim_cons_early_enable(rshim_backend_t *bd);
 #ifdef HAVE_RSHIM_NET
 int rshim_net_init(rshim_backend_t *bd);
 int rshim_net_del(rshim_backend_t *bd);
-void rshim_net_rx(rshim_backend_t *bd);
-void rshim_net_tx(rshim_backend_t *bd);
+int rshim_net_rx(rshim_backend_t *bd);
+int rshim_net_tx(rshim_backend_t *bd);
 #else
 static inline int rshim_net_init(rshim_backend_t *bd)
 {
@@ -564,11 +592,13 @@ static inline int rshim_net_del(rshim_backend_t *bd)
 {
   return 0;
 }
-static inline void rshim_net_rx(rshim_backend_t *bd)
+static inline int rshim_net_rx(rshim_backend_t *bd)
 {
+  return 0;
 }
-static inline void rshim_net_tx(rshim_backend_t *bd)
+static inline int rshim_net_tx(rshim_backend_t *bd)
 {
+  return 0;
 }
 #endif
 
@@ -658,4 +688,7 @@ int rshim_set_drop_mode(rshim_backend_t *bd, int value);
 /* Run rshim command mode. */
 int rshim_cmdmode_run(int argc, char *argv[]);
 
+/* Common initialization. */
+int rshim_init(int *epoll_fd, int *timer_fd);
+
 #endif /* _RSHIM_H */
