@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: GPL-2.0-only
+// SPDX-License-Identifier: GPL-2.0 or BSD-3-Clause
 /*
  * Copyright (C) 2019 Mellanox Technologies. All Rights Reserved.
  *
@@ -97,7 +97,7 @@ enum {
 };
 
 /* Min delay in seconds after RSHIM_PCIE_RST_START_ACK */
-#define RSHIM_PCIE_RST_START_MIN_DELAY    2
+#define RSHIM_PCIE_RST_START_MIN_DELAY    4
 
 /* Interrupt information between NIC_FW and rshim driver. */
 typedef union {
@@ -120,7 +120,10 @@ typedef union {
     /* RSHIM_PCIE_RST_TYPE_xxx */
     uint64_t rst_type : 3;
 
-    uint64_t unused_3 : 9;
+    uint64_t unused_3 : 8;
+
+    /* rshim running over pcie */
+    uint64_t pcie : 1;
   };
 
   uint64_t word;
@@ -320,8 +323,10 @@ static void rshim_pcie_mmap_release(rshim_pcie_t *dev)
 
 static void rshim_pcie_bind(rshim_pcie_t *dev, bool enable)
 {
+  uint16_t device_id[] = {BLUEFIELD1_DEVICE_ID, BLUEFIELD2_DEVICE_ID,
+                          BLUEFIELD3_DEVICE_ID, BLUEFIELD3_DEVICE_ID2};
   char cmd[RSHIM_CMD_MAX];
-  int rc;
+  int i, rc;
 
   /*
    * Linux kernel prior 4.18 has a bug which could cause crash when uio is
@@ -342,21 +347,24 @@ static void rshim_pcie_bind(rshim_pcie_t *dev, bool enable)
                dev->pci_path);
       if (system(cmd) == -1)
         RSHIM_DBG("Failed to unbind device\n");
+    } else if (dev->mmap_mode == RSHIM_PCIE_MMAP_VFIO) {
+      /* Clear driver_override. */
+      snprintf(cmd, sizeof(cmd),
+               "echo \"\" > %s/%04x:%02x:%02x.%1u/driver_override 2>/dev/null",
+               SYS_BUS_PCI_PATH, dev->domain, dev->bus,
+               dev->dev, dev->func);
+      if (system(cmd) == -1)
+        RSHIM_DBG("Failed to enable pcie\n");
     }
 
-    snprintf(cmd, sizeof(cmd), "echo '%x %x' > %s/%s 2>/dev/null",
-             TILERA_VENDOR_ID, BLUEFIELD1_DEVICE_ID, dev->pci_path,
-             enable ? "new_id" : "remove_id");
-    rc = system(cmd);
-    if (rc == -1)
-      RSHIM_DBG("Failed to write device id %m\n");
-
-    snprintf(cmd, sizeof(cmd), "echo '%x %x' > %s/%s 2>/dev/null",
-             TILERA_VENDOR_ID, BLUEFIELD2_DEVICE_ID, dev->pci_path,
-             enable ? "new_id" : "remove_id");
-    rc = system(cmd);
-    if (rc == -1)
-      RSHIM_DBG("Failed to write device id %m\n");
+    for (i = 0; i < sizeof(device_id) / sizeof(uint16_t); i++) {
+      snprintf(cmd, sizeof(cmd), "echo '%x %x' > %s/%s 2>/dev/null",
+               TILERA_VENDOR_ID, device_id[i], dev->pci_path,
+               enable ? "new_id" : "remove_id");
+      rc = system(cmd);
+      if (rc == -1)
+        RSHIM_DBG("Failed to write device id %m\n");
+    }
 
     if (enable) {
       snprintf(cmd, sizeof(cmd),
@@ -403,7 +411,7 @@ static int rshim_pcie_mmap_direct(rshim_pcie_t *dev)
 
   dev->device_fd = open(path, O_RDWR | O_SYNC);
   if (dev->device_fd < 0) {
-    RSHIM_ERR("Failed to open %s\n", path);
+    RSHIM_ERR("rshim%d failed to open %s\n", dev->bd.index, path);
     return -ENODEV;
   }
   dev->rshim_regs = mmap(NULL, dev->bar_size,
@@ -413,7 +421,7 @@ static int rshim_pcie_mmap_direct(rshim_pcie_t *dev)
                          0);
   if (dev->rshim_regs == MAP_FAILED) {
     dev->rshim_regs = NULL;
-    RSHIM_ERR("Failed to map RShim registers\n");
+    RSHIM_ERR("rshim%d failed to map RShim registers\n", dev->bd.index);
     return -ENOMEM;
   }
 
@@ -466,7 +474,7 @@ static int rshim_pcie_enable_irq(rshim_pcie_t *dev, bool enable)
 
     ret = ioctl(dev->device_fd, VFIO_DEVICE_SET_IRQS, irq_set);
     if (ret) {
-      RSHIM_ERR("Failed to mask INTx\n");
+      RSHIM_ERR("rshim%d failed to mask INTx\n", dev->bd.index);
       return -1;
     }
   }
@@ -482,7 +490,7 @@ static int rshim_pcie_enable_irq(rshim_pcie_t *dev, bool enable)
 
   ret = ioctl(dev->device_fd, VFIO_DEVICE_SET_IRQS, irq_set);
   if (ret) {
-    RSHIM_ERR("Failed to enable INTx\n");
+    RSHIM_ERR("rshim%d failed to enable INTx\n", dev->bd.index);
     return -1;
   }
 
@@ -498,7 +506,7 @@ static int rshim_pcie_enable_irq(rshim_pcie_t *dev, bool enable)
 
     ret = ioctl(dev->device_fd, VFIO_DEVICE_SET_IRQS, irq_set);
     if (ret) {
-      RSHIM_ERR("Failed to unmask INTx\n");
+      RSHIM_ERR("rshim%d failed to unmask INTx\n", dev->bd.index);
       return -1;
     }
   }
@@ -551,13 +559,13 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
            dev->dev, dev->func);
   rc = readlink(path, name, sizeof(name));
   if (rc < 0 || !name[0] || rc >= sizeof(name)) {
-    RSHIM_ERR("%s: failed to read iommu link\n", path);
+    RSHIM_ERR("rshim%d failed to read iommu link %s\n", dev->bd.index, path);
     goto fail;
   }
   name[rc] = 0;
   p = strrchr(name, '/');
   if (!p) {
-    RSHIM_ERR("Failed to find vfio group\n");
+    RSHIM_ERR("rshim%d failed to find vfio group\n", dev->bd.index);
     rc = -ENOENT;
     goto fail;
   }
@@ -574,7 +582,7 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
 
   rc = ioctl(group_fd, VFIO_GROUP_GET_STATUS, &group_status);
   if (rc) {
-    RSHIM_ERR("VFIO_GROUP_GET_STATUS failed\n");
+    RSHIM_ERR("rshim%d VFIO_GROUP_GET_STATUS failed\n", dev->bd.index);
     goto fail;
   }
 
@@ -593,14 +601,14 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
            dev->bus, dev->dev, dev->func);
   device_fd = ioctl(group_fd, VFIO_GROUP_GET_DEVICE_FD, path);
   if (device_fd < 0) {
-    RSHIM_ERR("Failed to get vfio device %s\n", path);
+    RSHIM_ERR("rshim%d: failed to get vfio device %s\n", dev->bd.index, path);
     rc = device_fd;
     goto fail;
   }
 
   rc = ioctl(device_fd, VFIO_DEVICE_GET_INFO, &device_info);
   if (rc) {
-    RSHIM_ERR("Failed to get vfio device info\n");
+    RSHIM_ERR("rshim%d failed to get vfio device info\n", dev->bd.index);
     goto fail;
   }
 
@@ -612,7 +620,7 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
   region_info.index = 0;
   rc = ioctl(device_fd, VFIO_DEVICE_GET_REGION_INFO, &region_info);
   if (rc) {
-    RSHIM_ERR("Failed to get vfio region info\n");
+    RSHIM_ERR("rshim%d failed to get vfio region info\n", dev->bd.index);
     goto fail;
   }
 
@@ -623,7 +631,7 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
                      device_fd,
                      (off_t)region_info.offset);
     if (map == MAP_FAILED) {
-      RSHIM_ERR("Vfio mmap failed %m\n");
+      RSHIM_ERR("rshim%d vfio mmap failed\n", dev->bd.index);
       rc = -1;
       goto fail;
     } else {
@@ -634,7 +642,8 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
                  VFIO_GET_REGION_ADDR(VFIO_PCI_CONFIG_REGION_INDEX) +
                  PCI_COMMAND);
       if (rc != sizeof(reg)) {
-        RSHIM_ERR("Failed to read command from PCI config space!\n");
+        RSHIM_ERR("rshim%d failed to read command from PCI config space!\n",
+                  dev->bd.index);
         rc = -1;
         goto fail;
       }
@@ -643,7 +652,7 @@ static int rshim_pcie_mmap_vfio(rshim_pcie_t *dev)
                   VFIO_GET_REGION_ADDR(VFIO_PCI_CONFIG_REGION_INDEX) +
                   PCI_COMMAND);
       if (rc != sizeof(reg)) {
-        RSHIM_ERR("Failed to set PCI bus mastering!\n");
+        RSHIM_ERR("rshim%d failed to set PCI bus mastering!\n", dev->bd.index);
         rc = -1;
         goto fail;
       }
@@ -710,6 +719,10 @@ static int rshim_pcie_mmap_uio(rshim_pcie_t *dev)
     if (str)
       break;
   }
+
+  if (str)
+    uio_num = atoi(str);
+
   closedir(dir);
 
   if (!str)
@@ -721,7 +734,6 @@ static int rshim_pcie_mmap_uio(rshim_pcie_t *dev)
     return rc;
 
   /* Open the control fd to handle interrupt. */
-  uio_num = atoi(str);
   snprintf(devname, sizeof(devname), "/dev/uio%u", uio_num);
   if (dev->intr_fd >= 0) {
     dev->intr_reset_seq++;
@@ -754,6 +766,29 @@ static int rshim_pcie_mmap(rshim_pcie_t *dev, bool enable)
   return rc;
 }
 
+/*
+ * Check and set the pcie bit.
+ * This function is called from timer callback since the pcie bit
+ * could be cleared during ARM reset.
+ */
+void rshim_pcie_check(rshim_backend_t *bd)
+{
+  rshim_pcie_intr_info_t info = {.word = 0};
+  int rc;
+
+  pthread_mutex_lock(&bd->mutex);
+
+  rc = bd->read_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad6,
+                      &info.word, RSHIM_REG_SIZE_8B);
+  if (!rc && !RSHIM_BAD_CTRL_REG(info.word) && !info.pcie) {
+    info.pcie = 1;
+    bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad6,
+                    info.word, RSHIM_REG_SIZE_8B);
+  }
+
+  pthread_mutex_unlock(&bd->mutex);
+}
+
 static void rshim_pcie_intr(rshim_pcie_t *dev)
 {
   rshim_pcie_intr_info_t info = {.word = 0};
@@ -777,7 +812,7 @@ static void rshim_pcie_intr(rshim_pcie_t *dev)
                       &info.word, RSHIM_REG_SIZE_8B);
   if (rc || RSHIM_BAD_CTRL_REG(info.word)) {
     if (!bd->drop_mode)
-      RSHIM_WARN("Failed to read irq request\n");
+      RSHIM_WARN("rshim%d failed to read irq request\n", bd->index);
     goto intr_done;
   }
 
@@ -787,33 +822,35 @@ static void rshim_pcie_intr(rshim_pcie_t *dev)
     goto intr_done;
   }
 
-  RSHIM_INFO("Receive interrupt for %s reset\n",
+  RSHIM_INFO("rshim%d receive interrupt for %s reset\n", bd->index,
     (info.rst_type == RSHIM_PCIE_RST_TYPE_NIC_RESET) ? "NIC" :
     ((info.rst_type == RSHIM_PCIE_RST_TYPE_DPU_RESET) ? "DPU" : ""));
 
   switch (info.rst_state) {
   case RSHIM_PCIE_RST_STATE_REQUEST:
-    RSHIM_INFO("NIC reset ACK\n");
+    RSHIM_INFO("rshim%d NIC reset ACK\n", bd->index);
+    info.pcie = 1;
     info.rst_reply = RSHIM_PCIE_RST_REPLY_ACK;
-    dev->nic_reset = true;
     __sync_synchronize();
     bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad6,
                     info.word, RSHIM_REG_SIZE_8B);
-    sleep(RSHIM_PCIE_NIC_RESET_WAIT);
-    dev->nic_reset = false;
     break;
 
   case RSHIM_PCIE_RST_STATE_ABORT:
-    RSHIM_INFO("NIC reset ABORT\n");
+    RSHIM_INFO("rshim%d NIC reset ABORT\n", bd->index);
+    info.pcie = 1;
     info.word &= 0xFFFFFFFFUL;
     bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad6,
                     info.word, RSHIM_REG_SIZE_8B);
     break;
 
   case RSHIM_PCIE_RST_STATE_START:
-    RSHIM_INFO("NIC reset START\n");
+    RSHIM_INFO("rshim%d NIC reset START\n", bd->index);
 
     info.rst_reply = RSHIM_PCIE_RST_START_ACK;
+    info.pcie = 1;
+    dev->nic_reset = true;
+    __sync_synchronize();
     bd->write_rshim(bd, RSHIM_CHANNEL, bd->regs->scratchpad6,
                     info.word, RSHIM_REG_SIZE_8B);
 
@@ -824,6 +861,7 @@ static void rshim_pcie_intr(rshim_pcie_t *dev)
      * - Clear FIFO state;
      * - Add a small delay for ARM to be ready;
      */
+    __sync_synchronize();
     drop_mode = bd->drop_mode;
     bd->drop_mode = 1;
     rshim_fifo_reset(bd);
@@ -832,6 +870,7 @@ static void rshim_pcie_intr(rshim_pcie_t *dev)
       delay = RSHIM_PCIE_RST_START_MIN_DELAY;
     sleep(delay);
     bd->drop_mode = drop_mode;
+    dev->nic_reset = false;
     break;
 
   default:
@@ -920,12 +959,12 @@ static int rshim_pcie_mmap(rshim_pcie_t *dev, bool enable)
 
   dev->device_fd = open("/dev/pci", O_RDWR, 0);
   if (dev->device_fd < 0) {
-    RSHIM_ERR("Failed to open /dev/pci\n");
+    RSHIM_ERR("rshim%d failed to open /dev/pci\n", dev->bd.index);
     return -ENODEV;
   }
 
   if (ioctl(dev->device_fd, PCIOCBARMMAP, &pbm) < 0) {
-    RSHIM_ERR("PCIOCBARMMAP IOCTL failed\n");
+    RSHIM_ERR("rshim%d PCIOCBARMMAP IOCTL failed\n", dev->bd.index);
     rc = -ENODEV;
     goto rshim_map_failed;
   }
@@ -934,7 +973,7 @@ static int rshim_pcie_mmap(rshim_pcie_t *dev, bool enable)
       (uintptr_t)pbm.pbm_bar_off);
   if (pbm.pbm_bar_length < dev->bar_size) {
     dev->rshim_regs = NULL;
-    RSHIM_ERR("BAR length is too small\n");
+    RSHIM_ERR("rshim%d BAR length is too small\n", dev->bd.index);
     rc = -ENOMEM;
     goto rshim_map_failed;
   }
@@ -974,7 +1013,7 @@ rshim_pcie_read(rshim_backend_t *bd, uint32_t chan, uint32_t addr,
       (chan != RSHIM_CHANNEL || addr != bd->regs->scratchpad6))
     sleep(RSHIM_PCIE_NIC_RESET_WAIT);
 
-  if (bd->drop_mode) {
+  if (bd->drop_mode && !bd->requesting_rshim) {
     *result = 0;
     return 0;
   }
@@ -1016,7 +1055,7 @@ rshim_pcie_write(rshim_backend_t *bd, uint32_t chan, uint32_t addr,
       (chan != RSHIM_CHANNEL || addr != bd->regs->scratchpad6))
     sleep(RSHIM_PCIE_NIC_RESET_WAIT);
 
-  if (bd->drop_mode)
+  if (bd->drop_mode && !bd->requesting_rshim)
     return 0;
 
   if (!bd->has_rshim || !bd->has_tm || !dev->rshim_regs)
@@ -1081,6 +1120,8 @@ static int rshim_pcie_enable(rshim_backend_t *bd, bool enable)
   if (!enable) {
     rshim_pcie_write(bd, RSHIM_CHANNEL, bd->regs->scratchpad1, 0,
                      RSHIM_REG_SIZE_8B);
+    rshim_pcie_write(bd, RSHIM_CHANNEL, bd->regs->scratchpad6, 0,
+                     RSHIM_REG_SIZE_8B);
   }
 
   /* Unmap existing resource first. */
@@ -1096,7 +1137,7 @@ static int rshim_pcie_enable(rshim_backend_t *bd, bool enable)
     /* Fall-back to uio if failed. */
     if (rc < 0 && dev->mmap_mode == RSHIM_PCIE_MMAP_VFIO &&
         rshim_pcie_has_uio()) {
-      RSHIM_INFO("Fall-back to uio\n");
+      RSHIM_INFO("rshim%d fall-back to uio\n", bd->index);
       rshim_pcie_bind(dev, false);
       dev->pci_path = SYS_UIO_PCI_PATH;
       dev->mmap_mode = RSHIM_PCIE_MMAP_UIO;
@@ -1106,7 +1147,7 @@ static int rshim_pcie_enable(rshim_backend_t *bd, bool enable)
 
     /* Fall-back to direct map if failed. */
     if (rc < 0 && dev->mmap_mode != RSHIM_PCIE_MMAP_DIRECT) {
-      RSHIM_INFO("Fall-back to direct io\n");
+      RSHIM_INFO("rshim%d fall-back to direct io\n", bd->index);
       rshim_pcie_bind(dev, false);
       dev->pci_path = NULL;
       dev->mmap_mode = RSHIM_PCIE_MMAP_DIRECT;
@@ -1122,7 +1163,78 @@ static int rshim_pcie_enable(rshim_backend_t *bd, bool enable)
     rc = rshim_pcie_mmap(dev, true);
 #endif /* __linux__ */
 
-  RSHIM_INFO("rshim %s %s\n", bd->dev_name, enable ? "enable" : "disable");
+  RSHIM_INFO("%s %s\n", bd->dev_name, enable ? "enable" : "disable");
+
+  return rc;
+}
+
+static int rshim_pcie_refresh_dev(rshim_backend_t *bd)
+{
+  rshim_pcie_t *dev = container_of(bd, rshim_pcie_t, bd);
+  struct pci_access *pci;
+  struct pci_dev *pci_dev;
+  char dev_path[RSHIM_PATH_MAX] = {0};
+  int rc = -ENODEV;
+  bool found = false;
+
+  pci = pci_alloc();
+  if (!pci)
+    return -ENOMEM;
+
+  pci_init(pci);
+  pci_scan_bus(pci);
+
+  /*
+   * Iterate over all PCI devices to find the matching device that has the same
+   * PCIe base name "pcie-<domain>:<bus>:<device>". The only expected change
+   * is the <function> part of the device name.
+   */
+  for (pci_dev = pci->devices; pci_dev; pci_dev = pci_dev->next) {
+    pci_fill_info(pci_dev, PCI_FILL_IDENT | PCI_FILL_BASES | PCI_FILL_CLASS);
+
+    if (pci_dev->vendor_id != TILERA_VENDOR_ID ||
+        (!rshim_is_bluefield1(pci_dev->device_id) &&
+         !rshim_is_bluefield2(pci_dev->device_id) &&
+         !rshim_is_bluefield3(pci_dev->device_id)))
+      continue;
+
+    /*
+    * Check if the device path (without function number) matches the
+    * beginning of the current device name (which includes function number)
+    */
+    snprintf(dev_path, sizeof(dev_path), "pcie-%04x:%02x:%02x",
+        pci_dev->domain, pci_dev->bus, pci_dev->dev);
+    if (strncmp(dev_path, bd->dev_name, strlen(dev_path)) == 0) {
+      found = true;
+      RSHIM_INFO("rshim%d found matching device %s.%d\n", 
+          bd->index, dev_path, pci_dev->func);
+
+      if (bd->enable_device)
+        bd->enable_device(bd, false);
+
+      /* Update device function which should be the only change */
+      dev->func = pci_dev->func;
+
+      snprintf(bd->dev_name, RSHIM_DEV_NAME_LEN - 1, "pcie-%04x:%02x:%02x.%x",
+          pci_dev->domain, pci_dev->bus, pci_dev->dev, pci_dev->func);
+
+      if (bd->enable_device) {
+        rc = bd->enable_device(bd, true);
+        if (rc)
+          RSHIM_ERR("rshim%d failed to re-enable device %s.%d\n", bd->index,
+                    dev_path, pci_dev->func);
+      }
+      break;
+    }
+  }
+
+  pci_cleanup(pci);
+
+  if (!found) {
+    RSHIM_ERR("rshim%d no matching PCIe device found for %s\n", 
+        bd->index, bd->dev_name);
+    return -ENODEV;
+  }
 
   return rc;
 }
@@ -1160,11 +1272,14 @@ static int rshim_pcie_probe(struct pci_dev *pci_dev)
 
     bd = &dev->bd;
     strcpy(bd->dev_name, dev_name);
+    bd->type = RSH_BACKEND_PCIE;
     bd->drop_mode = (rshim_drop_mode >= 0) ? rshim_drop_mode : 0;
+    bd->locked_mode = 0;
     bd->read_rshim = rshim_pcie_read;
     bd->write_rshim = rshim_pcie_write;
     bd->destroy = rshim_pcie_delete;
     bd->enable_device = rshim_pcie_enable;
+    bd->pcie_refresh_dev = rshim_pcie_refresh_dev;
     dev->write_count = 0;
     dev->device_fd = -1;
     dev->group_fd = -1;
@@ -1203,7 +1318,7 @@ static int rshim_pcie_probe(struct pci_dev *pci_dev)
   if (rshim_has_pcie_reset_delay || bd->ver_id < RSHIM_BLUEFIELD_3)
     bd->reset_delay = rshim_pcie_reset_delay;
   else
-    bd->reset_delay = 1; /* minimum delay for BF3 */
+    bd->reset_delay = 3; /* minimum delay for BF3 */
 
   /* Initialize object */
   dev->device_id = pci_dev->device_id;
@@ -1230,9 +1345,8 @@ static int rshim_pcie_probe(struct pci_dev *pci_dev)
   bd->has_rshim = 1;
   bd->has_tm = 1;
   rc = rshim_register(bd);
-
   /* Notify that the device is attached */
-  if (!rc)
+  if (!rc && !bd->drop_mode)
     rc = rshim_notify(bd, RSH_EVENT_ATTACH, 0);
   pthread_mutex_unlock(&bd->mutex);
   if (rc)
@@ -1243,7 +1357,7 @@ static int rshim_pcie_probe(struct pci_dev *pci_dev)
   if (pci_dev->device_id != BLUEFIELD1_DEVICE_ID) {
     rc = pthread_create(&dev->intr_thread, NULL, rshim_pcie_intr_thread, dev);
     if (rc)
-      RSHIM_ERR("Failed to create intr thread\n");
+      RSHIM_ERR("rshim%d failed to create intr thread\n", bd->index);
   }
 #endif
 
