@@ -1,33 +1,7 @@
 /*
- * Copyright (c) 2001-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifdef HAVE_CONFIG_H
@@ -133,25 +107,6 @@ typedef enum { e_K = 1024, e_M = 1048576 } units_t;
 #define SEC_TO_MICRO(n)         ((n)*1000000)
 #define TIME_DIFF_in_MICRO(start, end)                                                             \
     (SEC_TO_MICRO((end).tv_sec - (start).tv_sec) + (NANO_TO_MICRO((end).tv_nsec - (start).tv_nsec)))
-// printf formating when IP is in network byte ordering (for LITTLE_ENDIAN)
-#define NETWORK_IP_PRINTQUAD_LITTLE_ENDIAN(ip)                                                     \
-    (uint8_t)((ip)&0xff), (uint8_t)(((ip) >> 8) & 0xff), (uint8_t)(((ip) >> 16) & 0xff),           \
-        (uint8_t)(((ip) >> 24) & 0xff)
-
-// printf formating when IP is in host byte ordering (for LITTLE_ENDIAN)
-#define HOST_IP_PRINTQUAD_LITTLE_ENDIAN(ip)                                                        \
-    (uint8_t)(((ip) >> 24) & 0xff), (uint8_t)(((ip) >> 16) & 0xff), (uint8_t)(((ip) >> 8) & 0xff), \
-        (uint8_t)((ip)&0xff)
-
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-/* The host byte order is the same as network byte order, so these functions are all just identity.
- */
-#define NIPQUAD(ip) NETWORK_IP_PRINTQUAD_LITTLE_ENDIAN(ip)
-#else
-#if __BYTE_ORDER == __BIG_ENDIAN
-#define NIPQUAD(ip) HOST_IP_PRINTQUAD_LITTLE_ENDIAN(ip)
-#endif
-#endif
 
 bool g_b_exit = false;
 struct sigaction g_sigact;
@@ -252,7 +207,6 @@ void update_delta_stat(socket_stats_t *p_curr_stat, socket_stats_t *p_prev_stat)
         (p_curr_stat->counters.n_rx_poll_hit - p_prev_stat->counters.n_rx_poll_hit) / delay;
     p_prev_stat->n_rx_ready_byte_count = p_curr_stat->n_rx_ready_byte_count;
     p_prev_stat->n_tx_ready_byte_count = p_curr_stat->n_tx_ready_byte_count;
-    p_prev_stat->n_rx_ready_byte_limit = p_curr_stat->n_rx_ready_byte_limit;
     p_prev_stat->counters.n_rx_ready_byte_max = p_curr_stat->counters.n_rx_ready_byte_max;
     p_prev_stat->counters.n_rx_ready_byte_drop =
         (p_curr_stat->counters.n_rx_ready_byte_drop - p_prev_stat->counters.n_rx_ready_byte_drop) /
@@ -378,6 +332,12 @@ void update_delta_ring_stat(ring_stats_t *p_curr_ring_stats, ring_stats_t *p_pre
             (p_curr_ring_stats->simple.n_tx_dropped_wqes -
              p_prev_ring_stats->simple.n_tx_dropped_wqes) /
             delay;
+        p_prev_ring_stats->simple.n_tx_num_bufs =
+            (p_curr_ring_stats->simple.n_tx_num_bufs - p_prev_ring_stats->simple.n_tx_num_bufs) /
+            delay;
+        p_prev_ring_stats->simple.n_zc_num_bufs =
+            (p_curr_ring_stats->simple.n_zc_num_bufs - p_prev_ring_stats->simple.n_zc_num_bufs) /
+            delay;
 #ifdef DEFINED_UTLS
         p_prev_ring_stats->n_tx_tls_contexts =
             (p_curr_ring_stats->n_tx_tls_contexts - p_prev_ring_stats->n_tx_tls_contexts) / delay;
@@ -393,6 +353,14 @@ void update_delta_ring_stat(ring_stats_t *p_curr_ring_stats, ring_stats_t *p_pre
             p_prev_ring_stats->tap.n_vf_plugouts =
                 (p_curr_ring_stats->tap.n_vf_plugouts - p_prev_ring_stats->tap.n_vf_plugouts);
         } else {
+            p_prev_ring_stats->simple.n_tx_tso_pkt_count =
+                (p_curr_ring_stats->simple.n_tx_tso_pkt_count -
+                 p_prev_ring_stats->simple.n_tx_tso_pkt_count) /
+                delay;
+            p_prev_ring_stats->simple.n_tx_tso_byte_count =
+                (p_curr_ring_stats->simple.n_tx_tso_byte_count -
+                 p_prev_ring_stats->simple.n_tx_tso_byte_count) /
+                delay;
             p_prev_ring_stats->simple.n_rx_interrupt_received =
                 (p_curr_ring_stats->simple.n_rx_interrupt_received -
                  p_prev_ring_stats->simple.n_rx_interrupt_received) /
@@ -500,6 +468,7 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_RINGS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_ring_inst_arr[i].b_enabled) {
             p_ring_stats = &p_ring_inst_arr[i].ring_stats;
             printf("======================================================\n");
@@ -548,6 +517,12 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
                 printf(FORMAT_STATS_32bit, "Tap fd:", p_ring_stats->tap.n_tap_fd);
                 printf(FORMAT_RING_TAP_NAME, "Tap Device:", p_ring_stats->tap.s_tap_name);
             } else {
+                if (p_ring_stats->simple.n_tx_tso_pkt_count ||
+                    p_ring_stats->simple.n_tx_tso_byte_count) {
+                    printf(FORMAT_RING_PACKETS, "TSO Offload:",
+                           p_ring_stats->simple.n_tx_tso_byte_count / BYTES_TRAFFIC_UNIT,
+                           p_ring_stats->simple.n_tx_tso_pkt_count, post_fix);
+                }
                 if (p_ring_stats->simple.n_rx_interrupt_requests ||
                     p_ring_stats->simple.n_rx_interrupt_received) {
                     printf(FORMAT_RING_INTERRUPT,
@@ -568,6 +543,11 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
                            p_ring_stats->simple.n_tx_dev_mem_pkt_count,
                            p_ring_stats->simple.n_tx_dev_mem_oob, post_fix);
                 }
+
+                printf(FORMAT_STATS_32bit,
+                       "TX buffers inflight:", p_ring_stats->simple.n_tx_num_bufs);
+                printf(FORMAT_STATS_32bit,
+                       "TX ZC buffers inflight:", p_ring_stats->simple.n_zc_num_bufs);
             }
         }
     }
@@ -584,6 +564,7 @@ void print_cq_stats(cq_instance_block_t *p_cq_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_CQS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_cq_inst_arr[i].b_enabled) {
             p_cq_stats = &p_cq_inst_arr[i].cq_stats;
             printf("======================================================\n");
@@ -637,6 +618,7 @@ void print_bpool_stats(bpool_instance_block_t *p_bpool_inst_arr)
     }
 
     for (int i = 0; i < NUM_OF_SUPPORTED_BPOOLS; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
         if (p_bpool_inst_arr && p_bpool_inst_arr[i].b_enabled) {
             p_bpool_stats = &p_bpool_inst_arr[i].bpool_stats;
             printf("======================================================\n");
@@ -648,6 +630,8 @@ void print_bpool_stats(bpool_instance_block_t *p_bpool_inst_arr)
                 printf("\tBUFFER_POOL=[%u]\n", i);
             }
             printf(FORMAT_STATS_32bit, "Size:", p_bpool_stats->n_buffer_pool_size);
+            printf(FORMAT_STATS_32bit, "Buffers in use:",
+                   p_bpool_stats->n_buffer_pool_created - p_bpool_stats->n_buffer_pool_size);
             printf(FORMAT_STATS_32bit, "No buffers error:", p_bpool_stats->n_buffer_pool_no_bufs);
             if (p_bpool_stats->n_buffer_pool_expands) {
                 printf(FORMAT_STATS_32bit, "Expands:", p_bpool_stats->n_buffer_pool_expands);
@@ -1200,6 +1184,7 @@ void add_fd_to_array(int fd, ip_addr mc_grp, mc_group_fds_t *mc_group_fds, int *
     }
     // the mc_group wasnt found
     // Add this mc group to the array
+    // coverity[copy_assignment_call] /* Turn off coverity warning for COPY_INSTEAD_OF_MOVE */
     mc_group_fds[i].mc_grp = mc_grp;
     int fd1 = fd;
     mc_group_fds[i].fd_list.push_back(fd1);
@@ -1214,7 +1199,6 @@ void print_mc_group_fds(mc_group_fds_t *mc_group_fds, int array_size)
     printf("------------------------------\n");
     for (int i = 0; i < array_size; i++) {
         char mcg_str[256];
-        /* cppcheck-suppress wrongPrintfScanfArgNum */
         sprintf(mcg_str, "[%s]", mc_group_fds[i].mc_grp.to_str().c_str());
         printf("%-22s", mcg_str);
         for (const auto &fd : mc_group_fds[i].fd_list) {
@@ -1241,10 +1225,9 @@ void show_mc_group_stats(mc_grp_info_t *p_mc_grp_info, socket_instance_block_t *
         size_t fd = (size_t)p_instance[i].skt_stats.fd;
         if (p_instance[i].b_enabled && g_fd_mask[fd]) {
             socket_stats_t *p_si_stats = &p_instance[i].skt_stats;
+            // coverity[missing_lock] /* Turn off coverity missing_lock check*/
             for (int grp_idx = 0; grp_idx < p_mc_grp_info->max_grp_num; grp_idx++) {
                 if (p_si_stats->mc_grp_map.test(grp_idx)) {
-                    // printf("fd %d Member of = [%d.%d.%d.%d]\n",p_si_stats->fd,
-                    // NIPQUAD(p_si_stats->mc_grp[grp_idx]));
                     add_fd_to_array(p_si_stats->fd, p_mc_grp_info->mc_grp_tbl[grp_idx].mc_grp,
                                     mc_group_fds, &array_size);
                 }
@@ -1806,12 +1789,16 @@ void zero_ring_stats(ring_stats_t *p_ring_stats)
     if (p_ring_stats->n_type == RING_TAP) {
         p_ring_stats->tap.n_vf_plugouts = 0;
     } else {
+        p_ring_stats->simple.n_tx_tso_pkt_count = 0;
+        p_ring_stats->simple.n_tx_tso_byte_count = 0;
         p_ring_stats->simple.n_rx_interrupt_received = 0;
         p_ring_stats->simple.n_rx_interrupt_requests = 0;
         p_ring_stats->simple.n_tx_dropped_wqes = 0;
         p_ring_stats->simple.n_tx_dev_mem_byte_count = 0;
         p_ring_stats->simple.n_tx_dev_mem_pkt_count = 0;
         p_ring_stats->simple.n_tx_dev_mem_oob = 0;
+        p_ring_stats->simple.n_tx_num_bufs = 0;
+        p_ring_stats->simple.n_zc_num_bufs = 0;
     }
 }
 
