@@ -1,33 +1,7 @@
 /*
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 
@@ -170,7 +144,6 @@ io_mux_call::io_mux_call(int *off_fds_buffer, offloaded_mode_t *off_modes_buffer
 	m_n_sysvar_select_skip_os_fd_check(safe_mce_sys().select_skip_os_fd_check),
 	m_n_sysvar_select_poll_os_ratio(safe_mce_sys().select_poll_os_ratio),
 	m_n_sysvar_select_poll_num(safe_mce_sys().select_poll_num),
-	m_b_sysvar_select_poll_os_force(safe_mce_sys().select_poll_os_force),
 	m_b_sysvar_select_handle_cpu_usage_stats(safe_mce_sys().select_handle_cpu_usage_stats),
 	m_p_all_offloaded_fds(off_fds_buffer),
 	m_p_offloaded_modes(off_modes_buffer),
@@ -468,8 +441,7 @@ int io_mux_call::call()
 
 	__log_funcall("");
 
-	if (!m_b_sysvar_select_poll_os_force  // TODO: evaluate/consider this logic
-			&& (*m_p_num_all_offloaded_fds == 0))
+	if (*m_p_num_all_offloaded_fds == 0)
 	{
 		// 1st scenario
 		timer_update();
