@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: GPL-2.0
+// SPDX-License-Identifier: GPL-3.0
 /*
  * NVMe I/O command implementation.
  * Copyright (c) 2015-2016 HGST, a Western Digital Company.
@@ -11,9 +11,7 @@
 #ifdef HAVE_BLK_INTEGRITY_H
 #include <linux/blk-integrity.h>
 #endif
-#ifdef HAVE_NET_MEMREMAP_H
 #include <linux/memremap.h>
-#endif
 #include <linux/module.h>
 #include "nvmet.h"
 
@@ -46,7 +44,7 @@ void nvmet_bdev_set_limits(struct block_device *bdev, struct nvme_id_ns *id)
 	 */
 	id->nsfeat |= 1 << 4;
 	/* NPWG = Namespace Preferred Write Granularity. 0's based */
-	id->npwg = lpp0b;
+	id->npwg = to0based(bdev_io_min(bdev) / bdev_logical_block_size(bdev));
 	/* NPWA = Namespace Preferred Write Alignment. 0's based */
 	id->npwa = id->npwg;
 	/* NPDG = Namespace Preferred Deallocate Granularity. 0's based */
@@ -70,14 +68,22 @@ void nvmet_bdev_ns_disable(struct nvmet_ns *ns)
 		ns->bdev = NULL;
 		ns->bdev_file = NULL;
 	}
+#else
+#ifdef HAVE_BDEV_RELEASE
+	if (ns->bdev_handle) {
+		bdev_release(ns->bdev_handle);
 #else
 	if (ns->bdev) {
 #ifdef HAVE_BLKDEV_PUT_HOLDER
 		blkdev_put(ns->bdev, NULL);
 #else
 		blkdev_put(ns->bdev, FMODE_WRITE | FMODE_READ);
+#endif
 #endif
 		ns->bdev = NULL;
+#ifdef HAVE_BDEV_RELEASE
+		ns->bdev_handle = NULL;
+#endif
 	}
 #endif /* HAVE_BDEV_FILE_OPEN_BY_PATH */
 }
@@ -86,18 +92,32 @@ static void nvmet_bdev_ns_enable_integrity(struct nvmet_ns *ns)
 {
 #if defined(CONFIG_BLK_DEV_INTEGRITY) && \
 	defined(HAVE_BLKDEV_BIO_INTEGRITY_BYTES)
+
 	struct blk_integrity *bi = bdev_get_integrity(ns->bdev);
 
-	if (bi) {
+	if (!bi)
+		return;
+
+#ifdef HAVE_BLK_INTEGRITY_CSUM_CRC64
+	if (bi->csum_type == BLK_INTEGRITY_CSUM_CRC) {
 		ns->metadata_size = bi->tuple_size;
-		if (bi->profile == &t10_pi_type1_crc)
+		if (bi->flags & BLK_INTEGRITY_REF_TAG)
 			ns->pi_type = NVME_NS_DPS_PI_TYPE1;
-		else if (bi->profile == &t10_pi_type3_crc)
-			ns->pi_type = NVME_NS_DPS_PI_TYPE3;
 		else
-			/* Unsupported metadata type */
-			ns->metadata_size = 0;
+			ns->pi_type = NVME_NS_DPS_PI_TYPE3;
+	} else {
+		ns->metadata_size = 0;
 	}
+#else
+	ns->metadata_size = bi->tuple_size;
+	if (bi->profile == &t10_pi_type1_crc)
+		ns->pi_type = NVME_NS_DPS_PI_TYPE1;
+	else if (bi->profile == &t10_pi_type3_crc)
+		ns->pi_type = NVME_NS_DPS_PI_TYPE3;
+	else
+		/* Unsupported metadata type */
+		ns->metadata_size = 0;
+#endif
 #endif
 }
 
@@ -115,34 +135,56 @@ int nvmet_bdev_ns_enable(struct nvmet_ns *ns)
 
 #ifdef HAVE_BDEV_FILE_OPEN_BY_PATH
 	ns->bdev_file = bdev_file_open_by_path(ns->device_path,
-			BLK_OPEN_READ | BLK_OPEN_WRITE, NULL, NULL);
+				BLK_OPEN_READ | BLK_OPEN_WRITE, NULL, NULL);
 	if (IS_ERR(ns->bdev_file)) {
 		ret = PTR_ERR(ns->bdev_file);
 		if (ret != -ENOTBLK) {
 			pr_err("failed to open block device %s: (%d)\n",
-			       ns->device_path, ret);
+					ns->device_path, ret);
 		}
 		ns->bdev_file = NULL;
 		return ret;
 	}
 	ns->bdev = file_bdev(ns->bdev_file);
 #else
-#ifdef HAVE_GENDISK_OPEN_MODE
+#ifdef HAVE_BDEV_RELEASE
+	ns->bdev_handle = bdev_open_by_path(ns->device_path,
+			BLK_OPEN_READ | BLK_OPEN_WRITE, NULL, NULL);
+
+#elif defined(HAVE_GENDISK_OPEN_MODE)
 	ns->bdev = blkdev_get_by_path(ns->device_path,
 			BLK_OPEN_READ | BLK_OPEN_WRITE, NULL, NULL);
+#else
+#ifdef HAVE_BLK_HOLDER_OPS
+	ns->bdev = blkdev_get_by_path(ns->device_path,
+			FMODE_READ | FMODE_WRITE, NULL, NULL);
 #else
 	ns->bdev = blkdev_get_by_path(ns->device_path,
 			FMODE_READ | FMODE_WRITE, NULL);
 #endif
+#endif
+#ifdef HAVE_BDEV_RELEASE
+	if (IS_ERR(ns->bdev_handle)) {
+		ret = PTR_ERR(ns->bdev_handle);
+		if (ret != -ENOTBLK) {
+			pr_err("failed to open block device %s: (%d)\n",
+					ns->device_path, ret);
+		}
+		ns->bdev_handle = NULL;
+#else
 	if (IS_ERR(ns->bdev)) {
 		ret = PTR_ERR(ns->bdev);
 		if (ret != -ENOTBLK) {
-			pr_err("failed to open block device %s: (%ld)\n",
-					ns->device_path, PTR_ERR(ns->bdev));
+			pr_err("failed to open block device %s: (%d)\n",
+					ns->device_path, ret);
 		}
 		ns->bdev = NULL;
+#endif
 		return ret;
 	}
+#ifdef HAVE_BDEV_RELEASE
+	ns->bdev = ns->bdev_handle->bdev;
+#endif
 #endif /* HAVE_BDEV_FILE_OPEN_BY_PATH */
 #ifdef HAVE_BDEV_NR_BYTES
 	ns->size = bdev_nr_bytes(ns->bdev);
@@ -180,7 +222,6 @@ void nvmet_bdev_ns_revalidate(struct nvmet_ns *ns)
 #endif
 }
 
-#ifdef HAVE_BLK_STATUS_T
 u16 blk_to_nvme_status(struct nvmet_req *req, blk_status_t blk_sts)
 {
 	u16 status = NVME_SC_SUCCESS;
@@ -195,11 +236,11 @@ u16 blk_to_nvme_status(struct nvmet_req *req, blk_status_t blk_sts)
 	 */
 	switch (blk_sts) {
 	case BLK_STS_NOSPC:
-		status = NVME_SC_CAP_EXCEEDED | NVME_SC_DNR;
+		status = NVME_SC_CAP_EXCEEDED | NVME_STATUS_DNR;
 		req->error_loc = offsetof(struct nvme_rw_command, length);
 		break;
 	case BLK_STS_TARGET:
-		status = NVME_SC_LBA_RANGE | NVME_SC_DNR;
+		status = NVME_SC_LBA_RANGE | NVME_STATUS_DNR;
 		req->error_loc = offsetof(struct nvme_rw_command, slba);
 		break;
 	case BLK_STS_NOTSUPP:
@@ -207,10 +248,10 @@ u16 blk_to_nvme_status(struct nvmet_req *req, blk_status_t blk_sts)
 		switch (req->cmd->common.opcode) {
 		case nvme_cmd_dsm:
 		case nvme_cmd_write_zeroes:
-			status = NVME_SC_ONCS_NOT_SUPPORTED | NVME_SC_DNR;
+			status = NVME_SC_ONCS_NOT_SUPPORTED | NVME_STATUS_DNR;
 			break;
 		default:
-			status = NVME_SC_INVALID_OPCODE | NVME_SC_DNR;
+			status = NVME_SC_INVALID_OPCODE | NVME_STATUS_DNR;
 		}
 		break;
 	case BLK_STS_MEDIUM:
@@ -219,7 +260,7 @@ u16 blk_to_nvme_status(struct nvmet_req *req, blk_status_t blk_sts)
 		break;
 	case BLK_STS_IOERR:
 	default:
-		status = NVME_SC_INTERNAL | NVME_SC_DNR;
+		status = NVME_SC_INTERNAL | NVME_STATUS_DNR;
 		req->error_loc = offsetof(struct nvme_common_command, opcode);
 	}
 
@@ -237,23 +278,12 @@ u16 blk_to_nvme_status(struct nvmet_req *req, blk_status_t blk_sts)
 	}
 	return status;
 }
-#endif
 
-#ifdef HAVE_BIO_ENDIO_1_PARAM
 static void nvmet_bio_done(struct bio *bio)
-#else
-static void nvmet_bio_done(struct bio *bio, int error)
-#endif
 {
 	struct nvmet_req *req = bio->bi_private;
 
-#ifdef HAVE_BLK_STATUS_T
 	nvmet_req_complete(req, blk_to_nvme_status(req, bio->bi_status));
-#elif defined(HAVE_STRUCT_BIO_BI_ERROR)
-	nvmet_req_complete(req, bio->bi_error ? NVME_SC_INTERNAL | NVME_SC_DNR : 0);
-#else
-	nvmet_req_complete(req, error ? NVME_SC_INTERNAL | NVME_SC_DNR : 0);
-#endif
 	nvmet_req_bio_put(req, bio);
 }
 
@@ -286,12 +316,15 @@ static int nvmet_bdev_alloc_bip(struct nvmet_req *req, struct bio *bio,
 		return PTR_ERR(bip);
 	}
 
+/* Using 5e70d0acf082 instead of 80814b8e359 as it has no new symbols */
+#ifndef HAVE_PCIE_CAPABILITY_CLEAR_AND_SET_WORD_LOCKED
 	bip->bip_iter.bi_size = bio_integrity_bytes(bi, bio_sectors(bio));
+#endif
 	/* virtual start sector must be in integrity interval units */
 	bip_set_seed(bip, bio->bi_iter.bi_sector >>
 		     (bi->interval_exp - SECTOR_SHIFT));
 
-	resid = bip->bip_iter.bi_size;
+	resid = bio_integrity_bytes(bi, bio_sectors(bio));
 	while (resid > 0 && sg_miter_next(miter)) {
 		len = min_t(size_t, miter->length, resid);
 		rc = bio_integrity_add_page(bio, miter->page, len,
@@ -334,9 +367,6 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 	int i, rc;
 #else
 	int op, i, rc;
-#endif
-#ifndef HAVE_BLK_TYPE_OP_IS_SYNC
-	int op_flags = 0;
 #endif
 	struct sg_mapping_iter prot_miter;
 	unsigned int iter_flags;
@@ -353,23 +383,14 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 	if (req->cmd->rw.opcode == nvme_cmd_write) {
 #ifdef HAVE_BLK_OPF_T
 		opf = REQ_OP_WRITE | REQ_SYNC | REQ_IDLE;
-#elif defined (HAVE_BLK_TYPE_OP_IS_SYNC)
-#ifdef HAVE_REQ_IDLE
+#else
 		op = REQ_OP_WRITE | REQ_SYNC | REQ_IDLE;
-#else
-		op = REQ_OP_WRITE | WRITE_ODIRECT;
 #endif
-#else
-		op = REQ_OP_WRITE;
-		op_flags = REQ_SYNC;
-#endif /* HAVE_BLK_OPF_T */
 		if (req->cmd->rw.control & cpu_to_le16(NVME_RW_FUA))
 #ifdef HAVE_BLK_OPF_T
 			opf |= REQ_FUA;
-#elif defined (HAVE_BLK_TYPE_OP_IS_SYNC)
-			op |= REQ_FUA;
 #else
-			op_flags |= REQ_FUA;
+			op |= REQ_FUA;
 #endif
 		iter_flags = SG_MITER_TO_SG;
 	} else {
@@ -381,13 +402,18 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 		iter_flags = SG_MITER_FROM_SG;
 	}
 
+	if (req->cmd->rw.control & cpu_to_le16(NVME_RW_LR))
+#ifdef HAVE_BLK_OPF_T
+		opf |= REQ_FAILFAST_DEV;
+#else
+		op |= REQ_FAILFAST_DEV;
+#endif
+
 	if (is_pci_p2pdma_page(sg_page(req->sg)))
 #ifdef HAVE_BLK_OPF_T
 		opf |= REQ_NOMERGE;
-#elif defined (HAVE_BLK_TYPE_OP_IS_SYNC)
-		op |= REQ_NOMERGE;
 #else
-		op_flags |= REQ_NOMERGE;
+		op |= REQ_NOMERGE;
 #endif
 
 	sector = nvmet_lba_to_sect(req->ns, req->cmd->rw.slba);
@@ -435,23 +461,12 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 #else
 	bio->bi_bdev = req->ns->bdev;
 #endif /* HAVE_BIO_BI_DISK || HAVE_ENUM_BIO_REMAPPED */
-#ifdef HAVE_STRUCT_BIO_BI_ITER
 	bio->bi_iter.bi_sector = sector;
-#else
-	bio->bi_sector = sector;
-#endif
 	bio->bi_private = req;
 	bio->bi_end_io = nvmet_bio_done;
-#ifdef HAVE_BLK_TYPE_OP_IS_SYNC
 #ifndef HAVE_BIO_INIT_5_PARAMS
 	bio->bi_opf = op;
 #endif
-#else
-	bio_set_op_attrs(bio, op, op_flags);
-#endif /* HAVE_BLK_TYPE_OP_IS_SYNC */
-#ifdef HAVE_RH7_STRUCT_BIO_AUX
-	bio_init_aux(bio, &req->bio_aux);
-#endif
 
 	blk_start_plug(&plug);
 	if (req->metadata_len)
@@ -494,25 +509,13 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 #else
 			bio->bi_bdev = req->ns->bdev;
 #endif
-#ifdef HAVE_STRUCT_BIO_BI_ITER
 			bio->bi_iter.bi_sector = sector;
-#else
-			bio->bi_sector = sector;
-#endif
-#ifdef HAVE_BLK_TYPE_OP_IS_SYNC
 #ifndef HAVE_BIO_INIT_5_PARAMS
 			bio->bi_opf = op;
 #endif
-#else
-			bio_set_op_attrs(bio, op, op_flags);
-#endif
 
 			bio_chain(bio, prev);
-#ifdef HAVE_SUBMIT_BIO_1_PARAM
 			submit_bio(prev);
-#else
-			submit_bio(bio_data_dir(prev), prev);
-#endif
 		}
 
 		sector += sg->length >> 9;
@@ -527,11 +530,7 @@ static void nvmet_bdev_execute_rw(struct nvmet_req *req)
 		}
 	}
 
-#ifdef HAVE_SUBMIT_BIO_1_PARAM
 	submit_bio(bio);
-#else
-	submit_bio(bio_data_dir(bio), bio);
-#endif
 	blk_finish_plug(&plug);
 }
 
@@ -570,19 +569,11 @@ static void nvmet_bdev_execute_flush(struct nvmet_req *req)
 #endif
 	bio->bi_private = req;
 	bio->bi_end_io = nvmet_bio_done;
-#ifdef HAVE_BLK_TYPE_OP_IS_SYNC
 #ifndef HAVE_BIO_INIT_5_PARAMS
 	bio->bi_opf = REQ_OP_WRITE | REQ_PREFLUSH;
 #endif
-#else
-	bio_set_op_attrs(bio, REQ_OP_WRITE, WRITE_FLUSH);
-#endif
 
-#ifdef HAVE_SUBMIT_BIO_1_PARAM
 	submit_bio(bio);
-#else
-	submit_bio(bio_data_dir(bio), bio);
-#endif
 }
 
 u16 nvmet_bdev_flush(struct nvmet_req *req)
@@ -601,7 +592,7 @@ u16 nvmet_bdev_flush(struct nvmet_req *req)
 	if (blkdev_issue_flush(req->ns->bdev, GFP_KERNEL, NULL))
 #endif
 #endif
-		return NVME_SC_INTERNAL | NVME_SC_DNR;
+		return NVME_SC_INTERNAL | NVME_STATUS_DNR;
 	return 0;
 }
 
@@ -660,11 +651,7 @@ static void nvmet_bdev_execute_discard(struct nvmet_req *req)
 		if (status)
 			bio_io_error(bio);
 		else
-#ifdef HAVE_SUBMIT_BIO_1_PARAM
 			submit_bio(bio);
-#else
-			submit_bio(bio_data_dir(bio), bio);
-#endif
 	} else {
 		nvmet_req_complete(req, status);
 	}
@@ -688,7 +675,6 @@ static void nvmet_bdev_execute_dsm(struct nvmet_req *req)
 	}
 }
 
-#ifdef HAVE_BLKDEV_ISSUE_ZEROOUT
 static void nvmet_bdev_execute_write_zeroes(struct nvmet_req *req)
 {
 	struct nvme_write_zeroes_cmd *write_zeroes = &req->cmd->write_zeroes;
@@ -704,29 +690,16 @@ static void nvmet_bdev_execute_write_zeroes(struct nvmet_req *req)
 	nr_sector = (((sector_t)le16_to_cpu(write_zeroes->length) + 1) <<
 		(req->ns->blksize_shift - 9));
 
-#ifdef CONFIG_COMPAT_IS_BLKDEV_ISSUE_ZEROOUT_HAS_FLAGS
 	ret = __blkdev_issue_zeroout(req->ns->bdev, sector, nr_sector,
 			GFP_KERNEL, &bio, 0);
-#else
-	if (__blkdev_issue_zeroout(req->ns->bdev, sector, nr_sector,
-			GFP_KERNEL, &bio, true))
-		ret = -EIO;
-	else
-		ret = 0;
-#endif
 	if (bio) {
 		bio->bi_private = req;
 		bio->bi_end_io = nvmet_bio_done;
-#ifdef HAVE_SUBMIT_BIO_1_PARAM
 		submit_bio(bio);
-#else
-		submit_bio(bio_data_dir(bio), bio);
-#endif
 	} else {
 		nvmet_req_complete(req, errno_to_nvme_status(req, ret));
 	}
 }
-#endif
 
 u16 nvmet_bdev_parse_io_cmd(struct nvmet_req *req)
 {
@@ -743,11 +716,9 @@ u16 nvmet_bdev_parse_io_cmd(struct nvmet_req *req)
 	case nvme_cmd_dsm:
 		req->execute = nvmet_bdev_execute_dsm;
 		return 0;
-#ifdef HAVE_BLKDEV_ISSUE_ZEROOUT
 	case nvme_cmd_write_zeroes:
 		req->execute = nvmet_bdev_execute_write_zeroes;
 		return 0;
-#endif
 	default:
 		return nvmet_report_invalid_opcode(req);
 	}
